@@ -3,17 +3,17 @@
  * Copyright (C) 2007 Alexey Balakin <balakin@appl.sci-nnov.ru>            *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
- *   it under the terms of the GNU General Public License as published by  *
- *   the Free Software Foundation; either version 2 of the License, or     *
- *   (at your option) any later version.                                   *
+ *   it under the terms of the GNU Library General Public License as       *
+ *   published by the Free Software Foundation; either version 3 of the    *
+ *   License, or (at your option) any later version.                       *
  *                                                                         *
  *   This program is distributed in the hope that it will be useful,       *
  *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
  *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
  *   GNU General Public License for more details.                          *
  *                                                                         *
- *   You should have received a copy of the GNU General Public License     *
- *   along with this program; if not, write to the                         *
+ *   You should have received a copy of the GNU Library General Public     *
+ *   License along with this program; if not, write to the                 *
  *   Free Software Foundation, Inc.,                                       *
  *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
  ***************************************************************************/
@@ -33,14 +33,15 @@ typedef float mreal;
 extern "C" {
 #endif
 /*****************************************************************************/
+//#ifdef _MGL_DATA_H_
 #ifdef __cplusplus
-struct mglDraw;
+class mglDraw;
 typedef mglDraw* HMDR;
-struct mglGraph;
+class mglGraph;
 typedef mglGraph* HMGL;
-struct mglData;
+class mglData;
 typedef mglData* HMDT;
-struct mglParse;
+class mglParse;
 typedef mglParse* HMPR;
 #else
 typedef void* HMDR;
@@ -74,6 +75,7 @@ void mgl_qt_run();
 void mgl_set_show_mouse_pos(HMGL gr, int enable);
 void mgl_get_last_mouse_pos(HMGL gr, mreal *x, mreal *y, mreal *z);
 void mgl_calc_xyz(HMGL gr, int xs, int ys, mreal *x, mreal *y, mreal *z);
+void mgl_calc_scr(HMGL gr, mreal x, mreal y, mreal z, int *xs, int *ys);
 //void mgl_fltk_thread();
 //void mgl_qt_thread();
 void mgl_update(HMGL graph);
@@ -165,7 +167,7 @@ void mgl_set_ambbr(HMGL gr, mreal i);
 /*****************************************************************************/
 /*		Scale and rotate													 */
 /*****************************************************************************/
-void mgl_identity(HMGL graph);
+void mgl_identity(HMGL graph, int rel);
 void mgl_clf(HMGL graph);
 void mgl_flush(HMGL gr);
 void mgl_clf_rgb(HMGL graph, mreal r, mreal g, mreal b);
@@ -174,6 +176,7 @@ void mgl_subplot_d(HMGL graph, int nx,int ny,int m, mreal dx, mreal dy);
 void mgl_inplot(HMGL graph, mreal x1,mreal x2,mreal y1,mreal y2);
 void mgl_relplot(HMGL graph, mreal x1,mreal x2,mreal y1,mreal y2);
 void mgl_columnplot(HMGL graph, int num, int ind);
+void mgl_stickplot(HMGL graph, int num, int ind, mreal tet, mreal phi);
 void mgl_aspect(HMGL graph, mreal Ax,mreal Ay,mreal Az);
 void mgl_rotate(HMGL graph, mreal TetX,mreal TetZ,mreal TetY);
 void mgl_rotate_vector(HMGL graph, mreal Tet,mreal x,mreal y,mreal z);
@@ -199,6 +202,7 @@ void mgl_set_yrange(HMGL graph, const HMDT a, int add);
 void mgl_set_zrange(HMGL graph, const HMDT a, int add);
 void mgl_set_auto(HMGL graph, mreal x1, mreal x2, mreal y1, mreal y2, mreal z1, mreal z2);
 void mgl_set_func(HMGL graph, const char *EqX,const char *EqY,const char *EqZ);
+void mgl_set_func_ext(HMGL graph, const char *EqX,const char *EqY,const char *EqZ,const char *EqA);
 void mgl_set_coor(HMGL gr, int how);
 void mgl_set_ternary(HMGL gr, int enable);
 void mgl_set_cutoff(HMGL graph, const char *EqC);
@@ -208,7 +212,10 @@ void mgl_box_rgb(HMGL graph, mreal r, mreal g, mreal b, int ticks);
 void mgl_axis(HMGL graph, const char *dir);
 void mgl_axis_grid(HMGL graph, const char *dir,const char *pen);
 void mgl_label(HMGL graph, char dir, const char *text);
-void mgl_label_ext(HMGL graph, char dir, const char *text, int pos, mreal size, mreal shift);
+void mgl_label_ext(HMGL graph, char dir, const char *text, mreal pos, mreal size, mreal shift);
+void mgl_labelw_ext(HMGL graph, char dir, const wchar_t *text, mreal pos, mreal size, mreal shift);
+void mgl_label_xy(HMGL graph, mreal x, mreal y, const char *text, const char *fnt, mreal size);
+void mgl_labelw_xy(HMGL graph, mreal x, mreal y, const wchar_t *text, const char *fnt, mreal size);
 void mgl_tune_ticks(HMGL graph, int tune, mreal fact_pos);
 void mgl_set_xttw(HMGL graph, const wchar_t *templ);
 void mgl_set_yttw(HMGL graph, const wchar_t *templ);
@@ -223,7 +230,7 @@ void mgl_set_ctt(HMGL graph, const char *templ);
 /*****************************************************************************/
 void mgl_ball(HMGL graph, mreal x,mreal y,mreal z);
 void mgl_ball_rgb(HMGL graph, mreal x, mreal y, mreal z, mreal r, mreal g, mreal b, mreal alpha);
-void mgl_ball_str(HMGL graph, mreal x, mreal y, mreal z, const char *col);
+void mgl_ball_str(HMGL graph, mreal x, mreal y, mreal z, char col);
 void mgl_line(HMGL graph, mreal x1, mreal y1, mreal z1, mreal x2, mreal y2, mreal z2, const char *pen,int n);
 void mgl_facex(HMGL graph, mreal x0, mreal y0, mreal z0, mreal wy, mreal wz, const char *stl, mreal dx, mreal dy);
 void mgl_facey(HMGL graph, mreal x0, mreal y0, mreal z0, mreal wx, mreal wz, const char *stl, mreal dx, mreal dy);
@@ -261,12 +268,15 @@ void mgl_plot(HMGL graph, const HMDT y, const char *pen);
 void mgl_plot_2(HMGL graph, const HMDT a, const char *pen);
 void mgl_plot_3(HMGL graph, const HMDT a, const char *pen);
 void mgl_radar(HMGL graph, const HMDT a, const char *pen, mreal r);
+void mgl_boxplot_xy(HMGL graph, const HMDT x, const HMDT a, const char *pen);
+void mgl_boxplot(HMGL graph, const HMDT a, const char *pen);
 void mgl_tens_xyz(HMGL graph, const HMDT x, const HMDT y, const HMDT z, const HMDT c, const char *pen);
 void mgl_tens_xy(HMGL graph, const HMDT x, const HMDT y, const HMDT c, const char *pen);
 void mgl_tens(HMGL graph, const HMDT y, const HMDT c,	const char *pen);
 void mgl_area_xyz(HMGL graph, const HMDT x, const HMDT y, const HMDT z, const char *pen);
 void mgl_area_xy(HMGL graph, const HMDT x, const HMDT y, const char *pen);
 void mgl_area_xys(HMGL graph, const HMDT x, const HMDT y, const char *pen);
+void mgl_area_s(HMGL graph, const HMDT y, const char *pen);
 void mgl_area(HMGL graph, const HMDT y, const char *pen);
 void mgl_area_2(HMGL graph, const HMDT a, const char *pen);
 void mgl_area_3(HMGL graph, const HMDT a, const char *pen);
@@ -435,8 +445,8 @@ void mgl_cont_all_xyz(HMGL graph, const HMDT x, const HMDT y, const HMDT z, cons
 void mgl_cont_all(HMGL graph, const HMDT a, const char *sch, int Num);
 void mgl_cloudp_xyz(HMGL graph, const HMDT x, const HMDT y, const HMDT z, const HMDT a, const char *stl, mreal alpha);
 void mgl_cloudp(HMGL graph, const HMDT a, const char *stl, mreal alpha);
-void mgl_cloudq_xyz(HMGL graph, const HMDT x, const HMDT y, const HMDT z, const HMDT a, const char *stl, mreal alpha);
-void mgl_cloudq(HMGL graph, const HMDT a, const char *stl, mreal alpha);
+void mgl_cloud_xyz(HMGL graph, const HMDT x, const HMDT y, const HMDT z, const HMDT a, const char *stl, mreal alpha);
+void mgl_cloud(HMGL graph, const HMDT a, const char *stl, mreal alpha);
 void mgl_contf3_xyz_val(HMGL graph, const HMDT v, const HMDT x, const HMDT y, const HMDT z, const HMDT a, char dir, int sVal, const char *sch);
 void mgl_contf3_val(HMGL graph, const HMDT v, const HMDT a, char dir, int sVal, const char *sch);
 void mgl_contf3_xyz(HMGL graph, const HMDT x, const HMDT y, const HMDT z, const HMDT a, char dir, int sVal, const char *sch, int Num);
@@ -478,11 +488,11 @@ void mgl_contf_z_val(HMGL graph, const HMDT v, const HMDT a, const char *stl, mr
 /*		Data creation functions												 */
 /*****************************************************************************/
 void mgl_data_rearrange(HMDT dat, int mx, int my, int mz);
-void mgl_data_set_mreal(HMDT dat, const mreal *A,int NX,int NY,int NZ);
+void mgl_data_set_float(HMDT dat, const float *A,int NX,int NY,int NZ);
 void mgl_data_set_double(HMDT dat, const double *A,int NX,int NY,int NZ);
-void mgl_data_set_mreal2(HMDT d, const mreal **A,int N1,int N2);
+void mgl_data_set_float2(HMDT d, const float **A,int N1,int N2);
 void mgl_data_set_double2(HMDT d, const double **A,int N1,int N2);
-void mgl_data_set_mreal3(HMDT d, const mreal ***A,int N1,int N2,int N3);
+void mgl_data_set_float3(HMDT d, const float ***A,int N1,int N2,int N3);
 void mgl_data_set_double3(HMDT d, const double ***A,int N1,int N2,int N3);
 void mgl_data_set(HMDT dat, const HMDT a);
 void mgl_data_set_vector(HMDT dat, gsl_vector *v);
@@ -519,6 +529,17 @@ mreal mgl_data_max(const HMDT dat);
 mreal mgl_data_min(const HMDT dat);
 mreal *mgl_data_value(HMDT dat, int i,int j,int k);
 const mreal *mgl_data_data(const HMDT dat);
+
+mreal mgl_data_first(const HMDT dat, const char *cond, int *i, int *j, int *k);
+mreal mgl_data_last(const HMDT dat, const char *cond, int *i, int *j, int *k);
+int mgl_data_find(const HMDT dat, const char *cond, char dir, int i, int j, int k);
+int mgl_data_find_any(const HMDT dat, const char *cond);
+mreal mgl_data_max_int(const HMDT dat, int *i, int *j, int *k);
+mreal mgl_data_max_real(const HMDT dat, mreal *x, mreal *y, mreal *z);
+mreal mgl_data_min_int(const HMDT dat, int *i, int *j, int *k);
+mreal mgl_data_min_real(const HMDT dat, mreal *x, mreal *y, mreal *z);
+mreal mgl_data_momentum_mw(const HMDT dat, char dir, mreal *m, mreal *w);
+
 HMDT mgl_data_combine(const HMDT dat1, const HMDT dat2);
 void mgl_data_extend(HMDT dat, int n1, int n2);
 void mgl_data_insert(HMDT dat, char dir, int at, int num);
@@ -567,6 +588,12 @@ void mgl_data_sub_num(HMDT dat, mreal d);
 /*****************************************************************************/
 /*		Nonlinear fitting													 */
 /*****************************************************************************/
+void mgl_hist_x(HMGL gr, HMDT res, const HMDT x, const HMDT a);
+void mgl_hist_xy(HMGL gr, HMDT res, const HMDT x, const HMDT y, const HMDT a);
+void mgl_hist_xyz(HMGL gr, HMDT res, const HMDT x, const HMDT y, const HMDT z, const HMDT a);
+/*****************************************************************************/
+/*		Nonlinear fitting													 */
+/*****************************************************************************/
 mreal mgl_fit_1(HMGL gr, HMDT fit, const HMDT y, const char *eq, const char *var, mreal *ini);
 mreal mgl_fit_2(HMGL gr, HMDT fit, const HMDT z, const char *eq, const char *var, mreal *ini);
 mreal mgl_fit_3(HMGL gr, HMDT fit, const HMDT a, const char *eq, const char *var, mreal *ini);
@@ -590,6 +617,7 @@ mreal mgl_fit_xyzs_d(HMGL gr, HMDT fit, const HMDT x, const HMDT y, const HMDT z
 mreal mgl_fit_xyzas_d(HMGL gr, HMDT fit, const HMDT x, const HMDT y, const HMDT z, const HMDT a, const HMDT s, const char *eq, const char *var, HMDT ini);
 
 void mgl_puts_fit(HMGL gr, mreal x, mreal y, mreal z, const char *prefix, const char *font, mreal size);
+const char *mgl_get_fit(HMGL gr);
 /*****************************************************************************/
 void mgl_sphere(HMGL graph, mreal x, mreal y, mreal z, mreal r, const char *stl);
 void mgl_drop(HMGL graph, mreal x1, mreal y1, mreal z1, mreal x2, mreal y2, mreal z2, mreal r, const char *stl, mreal shift, mreal ap);
