@@ -3,17 +3,17 @@
  * Copyright (C) 2007 Alexey Balakin <balakin@appl.sci-nnov.ru>            *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
- *   it under the terms of the GNU General Public License as published by  *
- *   the Free Software Foundation; either version 2 of the License, or     *
- *   (at your option) any later version.                                   *
+ *   it under the terms of the GNU Library General Public License as       *
+ *   published by the Free Software Foundation; either version 3 of the    *
+ *   License, or (at your option) any later version.                       *
  *                                                                         *
  *   This program is distributed in the hope that it will be useful,       *
  *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
  *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
  *   GNU General Public License for more details.                          *
  *                                                                         *
- *   You should have received a copy of the GNU General Public License     *
- *   along with this program; if not, write to the                         *
+ *   You should have received a copy of the GNU Library General Public     *
+ *   License along with this program; if not, write to the                 *
  *   Free Software Foundation, Inc.,                                       *
  *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
  ***************************************************************************/
@@ -85,7 +85,7 @@ public:
 	long nx;		///< number of points in 1st dimensions ('x' dimension)
 	long ny;		///< number of points in 2nd dimensions ('y' dimension)
 	long nz;		///< number of points in 3d dimensions ('z' dimension)
-	mreal *a;	  	///< data array
+	mreal *a;		///< data array
 	char *id;		///< column (or slice) names
 
 	/// Initiate by other mglData variable
@@ -95,6 +95,8 @@ public:
 	mglData(int rows, int cols, const float *d)	{	a=0;	Set(d,cols,rows);	};
 	mglData(int size, const double *d)	{	a=0;	Set(d,size);	};
 	mglData(int rows, int cols, const double *d)	{	a=0;	Set(d,cols,rows);	};
+	mglData(const double *d, int size)	{	a=0;	Set(d,size);	};
+	mglData(const double *d, int rows, int cols)	{	a=0;	Set(d,cols,rows);	};
 	/// Read data from file
 	mglData(const char *fname)			{	a=0;	Read(fname);	}
 	/// Allocate the memory for data array and initialize it zero
@@ -102,6 +104,11 @@ public:
 	/// Delete the array
 	~mglData()			{	if(a)	{	delete []a;	delete []id;	}	};
 	// ~~~~~~~~~~~~~~~~~~~~  ~~~~~~~~~~~~~~~~~~~~
+	// Get sizes
+	inline long GetNx()	{	return nx;	};
+	inline long GetNy()	{	return ny;	};
+	inline long GetNz()	{	return nz;	};
+
 	/// Allocate memory and copy the data from the gsl_vector
 	void Set(gsl_vector *v);
 	/// Allocate memory and copy the data from the gsl_matrix
@@ -113,7 +120,7 @@ public:
 	/// Allocate memory and copy the data from the (float **) array
 	void Set(const float **A,int N1,int N2);
 	/// Allocate memory and copy the data from the (double **) array
-	void Set(const double **A,int N1,int N2);
+	void Set(const double **A,int N1,int N2); 
 	/// Allocate memory and copy the data from the (float ***) array
 	void Set(const float ***A,int N1,int N2,int N3);
 	/// Allocate memory and copy the data from the (double ***) array
@@ -181,7 +188,7 @@ public:
 	/// Make new id
 	void NewId();
 	/// Reduce size of the data
-	void Squeeze(int rx,int ry=1,int rz=1,bool smooth=true);
+	void Squeeze(int rx,int ry=1,int rz=1,bool smooth=false);
 	/// Crop the data
 	void Crop(int n1, int n2,char dir='x');
 
@@ -270,13 +277,14 @@ public:
 	mreal Spline5(mreal x,mreal y,mreal z,mreal &dx,mreal &dy,mreal &dz) const;
 	/// Interpolate by qubic splain the data to given point x=[0...nx-1], y=[0...ny-1], z=[0...nz-1]
 	mreal Spline(mreal x,mreal y=0,mreal z=0) const;
+	mreal Spline3(mreal x,mreal y,mreal z,mreal &dx,mreal &dy,mreal &dz) const;
 	/// Interpolate by qubic splain the data to given point \a x,\a y,\a z which normalized in range [0, 1]
-	mreal Spline1(mreal x,mreal y=0,mreal z=0) const
+	inline mreal Spline1(mreal x,mreal y=0,mreal z=0) const
 	{	return Spline(x*(nx-1),y*(ny-1),z*(nz-1));	};
 	/// Interpolate by linear function the data to given point x=[0...nx-1], y=[0...ny-1], z=[0...nz-1]
 	mreal Linear(mreal x,mreal y=0,mreal z=0) const;
 	/// Interpolate by line the data to given point \a x,\a y,\a z which normalized in range [0, 1]
-	mreal Linear1(mreal x,mreal y=0,mreal z=0) const
+	inline mreal Linear1(mreal x,mreal y=0,mreal z=0) const
 	{	return Linear(x*(nx-1),y*(ny-1),z*(nz-1));	};
 
 	/// Insert \a num rows after \a at and fill it by formula \a eq
