@@ -3,17 +3,17 @@
  * Copyright (C) 2007 Alexey Balakin <balakin@appl.sci-nnov.ru>            *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
- *   it under the terms of the GNU General Public License as published by  *
- *   the Free Software Foundation; either version 2 of the License, or     *
- *   (at your option) any later version.                                   *
+ *   it under the terms of the GNU Library General Public License as       *
+ *   published by the Free Software Foundation; either version 3 of the    *
+ *   License, or (at your option) any later version.                       *
  *                                                                         *
  *   This program is distributed in the hope that it will be useful,       *
  *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
  *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
  *   GNU General Public License for more details.                          *
  *                                                                         *
- *   You should have received a copy of the GNU General Public License     *
- *   along with this program; if not, write to the                         *
+ *   You should have received a copy of the GNU Library General Public     *
+ *   License along with this program; if not, write to the                 *
  *   Free Software Foundation, Inc.,                                       *
  *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
  ***************************************************************************/
@@ -54,6 +54,7 @@ void mgl_qt_run_();
 void mgl_set_show_mouse_pos_(uintptr_t *gr, int *enable);
 void mgl_get_last_mouse_pos_(uintptr_t *gr, mreal *x, mreal *y, mreal *z);
 void mgl_calc_xyz_(uintptr_t *gr, int *xs, int *ys, mreal *x, mreal *y, mreal *z);
+void mgl_calc_scr_(uintptr_t *gr, mreal *x, mreal *y, mreal *z, int *xs, int *ys);
 //void mgl_fltk_thread_();
 //void mgl_qt_thread_();
 /*****************************************************************************/
@@ -129,7 +130,7 @@ void mgl_set_ambbr_(uintptr_t *gr, mreal *i);
 /*****************************************************************************/
 /*		Scale and rotate													 */
 /*****************************************************************************/
-void mgl_identity_(uintptr_t *graph);
+void mgl_identity_(uintptr_t *graph, int *rel);
 void mgl_clf_(uintptr_t *graph);
 void mgl_flush_(uintptr_t *gr);
 void mgl_clf_rgb_(uintptr_t *graph, mreal *r, mreal *g, mreal *b);
@@ -138,6 +139,7 @@ void mgl_subplot_d_(uintptr_t *graph, int *nx,int *ny,int *m,mreal *dx,mreal *dy
 void mgl_inplot_(uintptr_t *graph, mreal *x1,mreal *x2,mreal *y1,mreal *y2);
 void mgl_relplot_(uintptr_t *graph, mreal *x1,mreal *x2,mreal *y1,mreal *y2);
 void mgl_columnplot_(uintptr_t *graph, int *num, int *i);
+void mgl_stickplot_(uintptr_t *graph, int *num, int *i, mreal *tet, mreal *phi);
 void mgl_aspect_(uintptr_t *graph, mreal *Ax,mreal *Ay,mreal *Az);
 void mgl_rotate_(uintptr_t *graph, mreal *TetX,mreal *TetZ,mreal *TetY);
 void mgl_rotate_vector_(uintptr_t *graph, mreal *Tet,mreal *x,mreal *y,mreal *z);
@@ -162,6 +164,7 @@ void mgl_set_yrange_(uintptr_t *graph, uintptr_t *a, int *add);
 void mgl_set_zrange_(uintptr_t *graph, uintptr_t *a, int *add);
 void mgl_set_auto_(uintptr_t *graph, mreal *x1, mreal *x2, mreal *y1, mreal *y2, mreal *z1, mreal *z2);
 void mgl_set_func_(uintptr_t *graph, const char *EqX, const char *EqY, const char *EqZ, int, int, int);
+void mgl_set_func_ext_(uintptr_t *graph, const char *EqX, const char *EqY, const char *EqZ, const char *EqA, int, int, int, int);
 void mgl_set_coor_(uintptr_t *gr, int *how);
 void mgl_set_ternary_(uintptr_t *graph, int *enable);
 void mgl_set_cutoff_(uintptr_t *graph, const char *EqC, int);
@@ -171,7 +174,8 @@ void mgl_box_rgb_(uintptr_t *graph, mreal *r, mreal *g, mreal *b, int *ticks);
 void mgl_axis_(uintptr_t *graph, const char *dir,int);
 void mgl_axis_grid_(uintptr_t *graph, const char *dir,const char *pen,int,int);
 void mgl_label_(uintptr_t *graph, const char *dir, const char *text,int,int);
-void mgl_label_ext_(uintptr_t *graph, const char *dir, const char *text, int *pos, mreal *size, mreal *shift,int,int);
+void mgl_label_ext_(uintptr_t *graph, const char *dir, const char *text, mreal *pos, mreal *size, mreal *shift,int,int);
+void mgl_label_xy_(uintptr_t *graph, mreal *x, mreal *y, const char *text, const char *fnt, mreal *size,int,int);
 void mgl_set_xtt_(uintptr_t * graph, const char *templ,int);
 void mgl_set_ytt_(uintptr_t * graph, const char *templ,int);
 void mgl_set_ztt_(uintptr_t * graph, const char *templ,int);
@@ -215,11 +219,14 @@ void mgl_plot_(uintptr_t *graph, uintptr_t *y,	const char *pen,int);
 void mgl_plot_2_(uintptr_t *graph, uintptr_t *a, const char *pen,int);
 void mgl_plot_3_(uintptr_t *graph, uintptr_t *a, const char *pen,int);
 void mgl_radar_(uintptr_t *graph, uintptr_t *a,	const char *pen, mreal *r, int);
+void mgl_boxplot_xy_(uintptr_t *graph, uintptr_t *x, uintptr_t *y, const char *pen,int);
+void mgl_boxplot_(uintptr_t *graph, uintptr_t *y, const char *pen,int);
 void mgl_tens_xyz_(uintptr_t *graph, uintptr_t *x, uintptr_t *y, uintptr_t *z, uintptr_t *c, const char *pen,int);
 void mgl_tens_xy_(uintptr_t *graph, uintptr_t *x, uintptr_t *y, uintptr_t *c, const char *pen,int);
 void mgl_tens_(uintptr_t *graph, uintptr_t *y, uintptr_t *c, const char *pen,int);
 void mgl_area_xyz_(uintptr_t *graph, uintptr_t *x, uintptr_t *y, uintptr_t *z, const char *pen,int);
 void mgl_area_xy_(uintptr_t *graph, uintptr_t *x, uintptr_t *y, const char *pen,int);
+void mgl_area_s_(uintptr_t *graph, uintptr_t *y, const char *pen,int);
 void mgl_area_(uintptr_t *graph, uintptr_t *y, const char *pen,int);
 void mgl_area_2_(uintptr_t *graph, uintptr_t *a, const char *pen,int);
 void mgl_area_3_(uintptr_t *graph, uintptr_t *a, const char *pen,int);
@@ -391,9 +398,9 @@ void mgl_cont_all_(uintptr_t *graph, uintptr_t *a, const char *sch, int *Num,int
 void mgl_cloudp_xyz_(uintptr_t *graph, uintptr_t *x, uintptr_t *y, uintptr_t *z, uintptr_t *a,
 			const char *stl, mreal *alpha,int);
 void mgl_cloudp_(uintptr_t *graph, uintptr_t *a, const char *stl, mreal *alpha,int);
-void mgl_cloudq_xyz_(uintptr_t *graph, uintptr_t *x, uintptr_t *y, uintptr_t *z, uintptr_t *a,
+void mgl_cloud_xyz_(uintptr_t *graph, uintptr_t *x, uintptr_t *y, uintptr_t *z, uintptr_t *a,
 			const char *stl, mreal *alpha,int);
-void mgl_cloudq_(uintptr_t *graph, uintptr_t *a, const char *stl, mreal *alpha,int);
+void mgl_cloud_(uintptr_t *graph, uintptr_t *a, const char *stl, mreal *alpha,int);
 void mgl_contf3_xyz_val_(uintptr_t *graph, uintptr_t *v, uintptr_t *x, uintptr_t *y, uintptr_t *z, uintptr_t *a,
 			const char *dir, int *sVal, const char *sch,int,int);
 void mgl_contf3_val_(uintptr_t *graph, uintptr_t *v, uintptr_t *a, const char *dir, int *sVal, const char *sch,int,int);
@@ -440,13 +447,13 @@ void mgl_contf_z_val_(uintptr_t *graph, uintptr_t *v, uintptr_t *a, const char *
 /*		Data creation functions												 */
 /*****************************************************************************/
 void mgl_data_rearrange_(uintptr_t *dat, int *mx, int *my, int *mz);
-void mgl_data_set_mreal_(uintptr_t *dat, const mreal *A,int *NX,int *NY,int *NZ);
+void mgl_data_set_float_(uintptr_t *dat, const float *A,int *NX,int *NY,int *NZ);
 void mgl_data_set_double_(uintptr_t *dat, const double *A,int *NX,int *NY,int *NZ);
-void mgl_data_set_mreal1_(uintptr_t *d, const mreal *A,int *N1);
+void mgl_data_set_float1_(uintptr_t *d, const float *A,int *N1);
 void mgl_data_set_double1_(uintptr_t *d, const double *A,int *N1);
-void mgl_data_set_mreal2_(uintptr_t *d, const mreal *A,int *N1,int *N2);
+void mgl_data_set_float2_(uintptr_t *d, const float *A,int *N1,int *N2);
 void mgl_data_set_double2_(uintptr_t *d, const double *A,int *N1,int *N2);
-void mgl_data_set_mreal3_(uintptr_t *d, const mreal *A,int *N1,int *N2,int *N3);
+void mgl_data_set_float3_(uintptr_t *d, const float *A,int *N1,int *N2,int *N3);
 void mgl_data_set_double3_(uintptr_t *d, const double *A,int *N1,int *N2,int *N3);
 void mgl_data_set_(uintptr_t *dat, uintptr_t *a);
 mreal mgl_data_get_value_(uintptr_t *d, int *i, int *j, int *k);
@@ -479,6 +486,17 @@ void mgl_data_squeeze_(uintptr_t *dat, int *rx,int *ry,int *rz,int *smooth);
 mreal mgl_data_max_(uintptr_t *dat);
 mreal mgl_data_min_(uintptr_t *dat);
 const mreal *mgl_data_data_(uintptr_t *dat);
+
+mreal mgl_data_first_(uintptr_t *dat, const char *cond, int *i, int *j, int *k, int);
+mreal mgl_data_last_(uintptr_t *dat, const char *cond, int *i, int *j, int *k, int);
+int mgl_data_find_(uintptr_t *dat, const char *cond, char *dir, int *i, int *j, int *k, int,int);
+int mgl_data_find_any_(uintptr_t *dat, const char *cond, int);
+mreal mgl_data_max_int_(uintptr_t *dat, int *i, int *j, int *k);
+mreal mgl_data_max_real_(uintptr_t *dat, mreal *x, mreal *y, mreal *z);
+mreal mgl_data_min_int_(uintptr_t *dat, int *i, int *j, int *k);
+mreal mgl_data_min_real_(uintptr_t *dat, mreal *x, mreal *y, mreal *z);
+mreal mgl_data_momentum_mw_(uintptr_t *dat, char *dir, mreal *m, mreal *w,int);
+
 uintptr_t mgl_data_combine_(uintptr_t *dat1, uintptr_t *dat2);
 void mgl_data_extend_(uintptr_t *dat, int *n1, int *n2);
 void mgl_data_insert_(uintptr_t *dat, const char *dir, int *at, int *num, int);
@@ -526,6 +544,12 @@ void mgl_data_sub_num_(uintptr_t *dat, mreal *d);
 /*****************************************************************************/
 /*		Nonlinear fitting													 */
 /*****************************************************************************/
+void mgl_hist_x_(uintptr_t* gr, uintptr_t* res, uintptr_t* x, uintptr_t* a);
+void mgl_hist_xy_(uintptr_t* gr, uintptr_t* res, uintptr_t* x, uintptr_t* y, uintptr_t* a);
+void mgl_hist_xyz_(uintptr_t* gr, uintptr_t* res, uintptr_t* x, uintptr_t* y, uintptr_t* z, uintptr_t* a);
+/*****************************************************************************/
+/*		Nonlinear fitting													 */
+/*****************************************************************************/
 mreal mgl_fit_1_(uintptr_t* gr, uintptr_t* fit, uintptr_t* y, const char *eq, const char *var, mreal *ini, int l, int n);
 mreal mgl_fit_2_(uintptr_t* gr, uintptr_t* fit, uintptr_t* z, const char *eq, const char *var, mreal *ini, int l, int n);
 mreal mgl_fit_3_(uintptr_t* gr, uintptr_t* fit, uintptr_t* a, const char *eq, const char *var, mreal *ini, int l, int n);
