@@ -3,17 +3,17 @@
  * Copyright (C) 2007 Alexey Balakin <balakin@appl.sci-nnov.ru>            *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
- *   it under the terms of the GNU General Public License as published by  *
- *   the Free Software Foundation; either version 2 of the License, or     *
- *   (at your option) any later version.                                   *
+ *   it under the terms of the GNU Library General Public License as       *
+ *   published by the Free Software Foundation; either version 3 of the    *
+ *   License, or (at your option) any later version.                       *
  *                                                                         *
  *   This program is distributed in the hope that it will be useful,       *
  *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
  *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
  *   GNU General Public License for more details.                          *
  *                                                                         *
- *   You should have received a copy of the GNU General Public License     *
- *   along with this program; if not, write to the                         *
+ *   You should have received a copy of the GNU Library General Public     *
+ *   License along with this program; if not, write to the                 *
  *   Free Software Foundation, Inc.,                                       *
  *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
  ***************************************************************************/
@@ -27,8 +27,8 @@ struct mglArg
 {
 	int type;		///< Type of argument {0-data,1-string,2-number}
 	mglData *d;		///< Pointer to data (used if type==0)
-	wchar_t w[2048];///< String with parameters (used if type==1)
-	char s[2048];	///< String with parameters (used if type==1)
+	wchar_t w[2048];///< String with parameters
+	char s[2048];	///< String with parameters
 	mreal v;		///< Numerical value (used if type==2)
 	mglArg()	{	type=-1;	d=0;	v=0;	s[0]=0;	w[0]=0;	};
 };
@@ -66,12 +66,27 @@ struct mglVar
 	void MoveAfter(mglVar *var);
 };
 //-----------------------------------------------------------------------------
+/// Structure for the number handling (see mglParse class).
+struct mglNum
+{
+	mreal d;		///< Number itself
+	wchar_t s[256];	///< Number name
+	mglNum *next;	///< Pointer to next instance in list
+	mglNum *prev;	///< Pointer to prev instance in list
+	mglNum()	{	d=0;	s[0]=0;		next=prev=0;	};
+	~mglNum();
+	/// Move variable after \a var and copy \a func from \a var (if \a func is 0)
+	void MoveAfter(mglNum *var);
+};
+//-----------------------------------------------------------------------------
 /// Structure for the command argument (see mglGraph::Exec()).
 class mglParse
 {
 friend void mgl_export(wchar_t *out, const wchar_t *in, int type);
 public:
+	static mglCommand Prg[];	///< List of program flow commands (parsed by itself)
 	mglVar *DataList;	///< List with data and its names
+	mglNum *NumList;	///< List with numbers and its names
 	bool AllowSetSize;	///< Allow using setsize command
 	bool Stop;			///< Stop command was. Flag prevent further execution
 	mglCommand *Cmd;	///< Table of recognizable MGL commands (can be changed by user). It MUST be sorted by 'name' field !!!
@@ -79,6 +94,8 @@ public:
 
 	mglParse(bool setsize=false);
 	~mglParse();
+	/// Find the command by the keyword name
+	mglCommand *FindCommand(const wchar_t *name, bool prog=false);
 	/// Parse and execute the string of MGL script
 	int Parse(mglGraph *gr, const char *str, long pos=0);
 	/// Parse and execute the unicode string of MGL script
@@ -101,6 +118,14 @@ public:
 	mglVar *AddVar(const char *name);
 	/// Find variable or create it if absent
 	mglVar *AddVar(const wchar_t *name);
+	/// Find number or return 0 if absent
+	mglNum *FindNum(const char *name);
+	/// Find number or return 0 if absent
+	mglNum *FindNum(const wchar_t *name);
+	/// Find number or create it if absent
+	mglNum *AddNum(const char *name);
+	/// Find number or create it if absent
+	mglNum *AddNum(const wchar_t *name);
 	/// Add string for parameter $1, ..., $9
 	bool AddParam(int n, const char *str, bool isstr=true);
 	/// Add unicode string for parameter $1, ..., $9
@@ -126,20 +151,27 @@ private:
 	bool Skip;		///< Flag that commands should be skiped (inside 'once' block)
 	int if_stack[20];	///< Stack for if-else-endif commands
 	int if_pos;		///< position in if_stack
+	int if_for[10];		///< position in if_stack for for-cycle start
 	mglData *fval;	///< Values for for-cycle. Note that nx - number of elements, ny - next element, nz - address (or string number) of first cycle command
-	int for_stack[10];		///< The order of for-variables
-	int for_addr;			///< Flag for saving address in variable (for_addr-1)
+	int for_stack[10];	///< The order of for-variables
+	int for_addr;	///< Flag for saving address in variable (for_addr-1)
+	bool for_br;	///< Break is switched on (skip all comands until 'next')
 
 	/// Parse command
 	int Exec(mglGraph *gr, const wchar_t *com, long n, mglArg *a, const wchar_t *var);
 	/// Fill arguments \a a from strings
-	void FillArg(int n, wchar_t **arg, mglArg *a);
+	void FillArg(mglGraph *gr, int n, wchar_t **arg, mglArg *a);
 	/// PreExecute stage -- parse some commands and create variables
 	int PreExec(mglGraph *gr, long n, wchar_t **arg, mglArg *a);
 	/// Process optional arguments
 	void ProcOpt(mglGraph *gr, wchar_t *str);
 	/// Execute program-flow control commands
 	int FlowExec(mglGraph *gr, const wchar_t *com, long n, mglArg *a);
+	/// Parse and execute the unicode string of MGL script
+	int ParseDat(mglGraph *gr, const wchar_t *str, mglData &res);
+	/// In skip mode
+	bool inline ifskip()	{	return (if_pos>0 && !(if_stack[if_pos-1]&1));	};
+	bool inline skip()		{	return (Skip || ifskip() || for_br);	};
 };
 //-----------------------------------------------------------------------------
 #endif
