@@ -3,17 +3,17 @@
  * Copyright (C) 2007 Alexey Balakin <balakin@appl.sci-nnov.ru>            *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
- *   it under the terms of the GNU General Public License as published by  *
- *   the Free Software Foundation; either version 2 of the License, or     *
- *   (at your option) any later version.                                   *
+ *   it under the terms of the GNU Library General Public License as       *
+ *   published by the Free Software Foundation; either version 3 of the    *
+ *   License, or (at your option) any later version.                       *
  *                                                                         *
  *   This program is distributed in the hope that it will be useful,       *
  *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
  *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
  *   GNU General Public License for more details.                          *
  *                                                                         *
- *   You should have received a copy of the GNU General Public License     *
- *   along with this program; if not, write to the                         *
+ *   You should have received a copy of the GNU Library General Public     *
+ *   License along with this program; if not, write to the                 *
  *   Free Software Foundation, Inc.,                                       *
  *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
  ***************************************************************************/
@@ -117,39 +117,87 @@ void mglGraph::Error(mglPoint p,mglPoint e,const char *pen)
 //-----------------------------------------------------------------------------
 void mglGraph::Plot(const char *eqY, const char *pen, mreal zVal, int n)
 {
-	// TODO Add strong function variation analisys
 	if(eqY==0 || eqY[0]==0)	return;		// nothing to plot
-	mglData y(n);
+	if(n<=0)	n=100;
+
+	mreal *x = (mreal *)malloc(n*sizeof(mreal));
+	mreal *y = (mreal *)malloc(n*sizeof(mreal));
 	mglFormula *eq = new mglFormula(eqY);
 	register int i;
-	mreal d = (Max.x - Min.x)/(n-1.);
-	for(i=0;i<n;i++)
+	mreal d = (Max.x - Min.x)/(n-1.), xs, ys, yr, ym=fabs(Max.y - Min.y)/1000;
+	for(i=0;i<n;i++)	// initial data filling
+	{	x[i]=Min.x + i*d;	y[i]=eq->Calc(x[i]);	}
+
+	for(i=0;i<n-1 && n<10000;)
 	{
-		y.a[i] = eq->Calc(Min.x + i*d);
+		xs=(x[i]+x[i+1])/2;
+		ys=(y[i]+y[i+1])/2;	yr=eq->Calc(xs);
+		if(fabs(yr-ys)>ym)	// bad approximation here
+		{
+			x = (mreal *)realloc(x,(n+1)*sizeof(mreal));
+			y = (mreal *)realloc(y,(n+1)*sizeof(mreal));
+			memmove(x+i+2,x+i+1,(n-i-1)*sizeof(mreal));
+			memmove(y+i+2,y+i+1,(n-i-1)*sizeof(mreal));
+			x[i+1] = xs;	y[i+1] = yr;	n++;
+		}
+		else i++;
 	}
-	Plot(y,pen, zVal);
+	
 	delete eq;
+	mglData yy,xx;
+	xx.Set(x,n);	free(x);
+	yy.Set(y,n);	free(y);
+	Plot(xx,yy,pen, zVal);
 }
 //-----------------------------------------------------------------------------
 void mglGraph::Plot(const char *eqX, const char *eqY, const char *eqZ, const char *pen, int n)
 {
-	// TODO Add strong function variation analisys
-	mglData x(n), y(n), z(n);
+	if(n<=0)	n=100;
+	mreal *x = (mreal *)malloc(n*sizeof(mreal));
+	mreal *y = (mreal *)malloc(n*sizeof(mreal));
+	mreal *z = (mreal *)malloc(n*sizeof(mreal));
+	mreal *t = (mreal *)malloc(n*sizeof(mreal));
 	mglFormula *ex, *ey, *ez;
 	ex = new mglFormula(eqX ? eqX : "0");
 	ey = new mglFormula(eqY ? eqY : "0");
 	ez = new mglFormula(eqZ ? eqZ : "0");
 	register int i;
-	register mreal t;
-	for(i=0;i<n;i++)
+	mreal ts, xs, ys, zs, xr, yr, zr, xm=fabs(Max.x - Min.x)/1000, ym=fabs(Max.y - Min.y)/1000, zm=fabs(Max.z - Min.z)/1000;
+	for(i=0;i<n;i++)	// initial data filling
 	{
-		t = i/(n-1.);
-		x.a[i] = ex->Calc(0,0,t);
-		y.a[i] = ey->Calc(0,0,t);
-		z.a[i] = ez->Calc(0,0,t);
+		t[i] = i/(n-1.);
+		x[i] = ex->Calc(0,0,t[i]);
+		y[i] = ey->Calc(0,0,t[i]);
+		z[i] = ez->Calc(0,0,t[i]);
+	}
+
+	for(i=0;i<n-1 && n<10000;)
+	{
+		ts=(t[i]+t[i+1])/2;
+		xs=(x[i]+x[i+1])/2;	xr=ex->Calc(0,0,ts);
+		ys=(y[i]+y[i+1])/2;	yr=ey->Calc(0,0,ts);
+		zs=(z[i]+z[i+1])/2;	zr=ez->Calc(0,0,ts);
+		if(fabs(xr-xs)>xm || fabs(yr-ys)>ym || fabs(zr-zs)>zm)	// bad approximation here
+		{
+			z = (mreal *)realloc(z,(n+1)*sizeof(mreal));
+			t = (mreal *)realloc(t,(n+1)*sizeof(mreal));
+			x = (mreal *)realloc(x,(n+1)*sizeof(mreal));
+			y = (mreal *)realloc(y,(n+1)*sizeof(mreal));
+			memmove(x+i+2,x+i+1,(n-i-1)*sizeof(mreal));
+			memmove(y+i+2,y+i+1,(n-i-1)*sizeof(mreal));
+			memmove(z+i+2,z+i+1,(n-i-1)*sizeof(mreal));
+			memmove(t+i+2,t+i+1,(n-i-1)*sizeof(mreal));
+			t[i+1]=ts;	x[i+1]=xr;	y[i+1]=yr;	z[i+1]=zr;	n++;
+		}
+		else i++;
 	}
-	Plot(x,y,z,pen);
 	delete ex;	delete ey;	delete ez;
+
+	// TODO Add s1trong function variation analisys
+	mglData xx,yy,zz;
+	xx.Set(x,n);	yy.Set(y,n);	zz.Set(z,n);
+	free(x);	free(y);	free(z);	free(t);
+	Plot(xx,yy,zz,pen);
 }
 //-----------------------------------------------------------------------------
 //
@@ -204,11 +252,11 @@ void mglGraph::Plot(const mglData &x, const mglData &y, const mglData &z, const
 	m = x.ny > y.ny ? x.ny : y.ny;	m = z.ny > m ? z.ny : m;
 	mreal *pp = new mreal[3*n];
 	bool *tt = new bool[n];
-	if(pen && *pen)	{	mk=SelectPen(pen);	SetPal(pen);	}
-	else 	Pen(NC, '-', BaseLineWidth);
+	if(pen && *pen)	mk=SelectPen(pen);	else	Pen(NC, '-', BaseLineWidth);
+	SetPal(pen);
 	for(j=0;j<m;j++)
 	{
-		Pen(Pal[(CurrPal = (CurrPal+1)%NumPal)],0,0);
+		Pen(GetPal(),0,0);
 		mx = j<x.ny ? j:0;	my = j<y.ny ? j:0;	mz = j<z.ny ? j:0;
 		register long i,k;
 		for(i=0;i<n;i++)
@@ -222,7 +270,7 @@ void mglGraph::Plot(const mglData &x, const mglData &y, const mglData &z, const
 		}
 		curv_plot(n,pp,tt);
 	}
-	EndGroup();
+	SetPal(0);	EndGroup();
 	delete []tt;	delete []pp;
 }
 //-----------------------------------------------------------------------------
@@ -248,7 +296,7 @@ void mglGraph::Plot(const mglData &y, const char *pen,mreal zVal)
 //-----------------------------------------------------------------------------
 void mglGraph::Tens(const mglData &x, const mglData &y, const mglData &z, const mglData &c, const char *pen)
 {
-	long j,m,mx,my,mz,n=y.nx;
+	long j,m,mx,my,mz,mc,n=y.nx;
 	Arrow1 = Arrow2 = '_';
 	char mk=0;
 	if(x.nx!=n || z.nx!=n || c.nx!=n)
@@ -258,10 +306,11 @@ void mglGraph::Tens(const mglData &x, const mglData &y, const mglData &z, const
 	m = x.ny > y.ny ? x.ny : y.ny;	m = z.ny > m ? z.ny : m;
 	mreal *pp = new mreal[3*n], *cc = new mreal[n];
 	bool *tt = new bool[n];
-	if(pen && *pen)	{	SetScheme(pen);	mk=SelectPen(pen);	}
+	if(pen && *pen)	{	SetScheme(pen,false);	mk=SelectPen(pen);	}
 	for(j=0;j<m;j++)
 	{
-		mx = j<x.ny ? j:0;	my = j<y.ny ? j:0;	mz = j<z.ny ? j:0;
+		mx = j<x.ny ? j:0;	my = j<y.ny ? j:0;
+		mz = j<z.ny ? j:0;	mc = j<c.ny ? j:0;
 		register long i,k;
 		for(i=0;i<n;i++)
 		{
@@ -270,9 +319,13 @@ void mglGraph::Tens(const mglData &x, const mglData &y, const mglData &z, const
 			pp[k+1] = y.a[i+my*n];
 			pp[k+2] = z.a[i+mz*n];
 			tt[i] = ScalePoint(pp[k],pp[k+1],pp[k+2]);
-//			if(mk && tt[i])	Mark(pp[k],pp[k+1],pp[k+2],mk);
+			if(mk && tt[i])
+			{
+				DefColor(GetC(c.a[i+mc*n]));
+				Mark(pp[k],pp[k+1],pp[k+2],mk);
+			}
 		}
-		memcpy(cc,c.a+(j<c.ny?j:0)*n,n*sizeof(mreal));
+		memcpy(cc,c.a+mc*n,n*sizeof(mreal));
 		curv_plot(n,pp,cc,tt);
 	}
 	EndGroup();
@@ -311,7 +364,7 @@ void mglGraph::Area(const mglData &x, const mglData &y, const mglData &z, const
 	SetPal(pen);
 	for(j=0;j<m;j++)
 	{
-		DefColor(Pal[(CurrPal = (CurrPal+1)%NumPal)], -1);
+		DefColor(GetPal(), -1);
 		mx = j<x.ny ? j:0;	my = j<y.ny ? j:0;	mz = j<z.ny ? j:0;
 
 		for(i=0;i<y.nx;i++)
@@ -326,7 +379,7 @@ void mglGraph::Area(const mglData &x, const mglData &y, const mglData &z, const
 		}
 		surf_plot(2,n,pp,0,tt);
 	}
-	EndGroup();
+	SetPal(0);	EndGroup();
 	delete []pp;	delete []tt;
 }
 //-----------------------------------------------------------------------------
@@ -345,7 +398,7 @@ void mglGraph::Area(const mglData &x, const mglData &y, const char *pen,bool sum
 	SetPal(pen);
 	for(j=0;j<m;j++)
 	{
-		DefColor(Pal[(CurrPal = (CurrPal+1)%NumPal)], -1);
+		DefColor(GetPal(), -1);
 		mx = j<x.ny ? j:0;	//my = j<y.ny ? j:0;
 		for(i=0;i<n;i++)
 		{
@@ -361,7 +414,7 @@ void mglGraph::Area(const mglData &x, const mglData &y, const char *pen,bool sum
 		}
 		surf_plot(2,n,pp,0,tt);
 	}
-	EndGroup();
+	SetPal(0);	EndGroup();
 	delete []pp;	delete []tt;
 }
 //-----------------------------------------------------------------------------
@@ -391,7 +444,7 @@ void mglGraph::Region(const mglData &x, const mglData &y1, const mglData &y2, co
 	SetPal(pen);
 	for(j=0;j<m;j++)
 	{
-		DefColor(Pal[(CurrPal = (CurrPal+1)%NumPal)], -1);
+		DefColor(GetPal(), -1);
 		mx = j<x.ny ? j:0;
 		for(i=0;i<n;i++)
 		{
@@ -405,7 +458,7 @@ void mglGraph::Region(const mglData &x, const mglData &y1, const mglData &y2, co
 		}
 		surf_plot(2,n,pp,0,tt);
 	}
-	EndGroup();
+	SetPal(0);	EndGroup();
 	delete []pp;	delete []tt;
 }
 //-----------------------------------------------------------------------------
@@ -431,11 +484,11 @@ void mglGraph::Step(const mglData &x, const mglData &y, const mglData &z, const
 	m = x.ny > y.ny ? x.ny : y.ny;	m = z.ny > m ? z.ny : m;
 	mreal *pp = new mreal[6*n];
 	bool *tt = new bool[2*n];
-	if(pen && *pen)	mk=SelectPen(pen);	else 	Pen(NC, '-', BaseLineWidth);
+	if(pen && *pen)	mk=SelectPen(pen);	else	Pen(NC, '-', BaseLineWidth);
 	SetPal(pen);
 	for(j=0;j<m;j++)
 	{
-		Pen(Pal[(CurrPal = (CurrPal+1)%NumPal)],0,0);
+		Pen(GetPal(),0,0);
 		mx = j<x.ny ? j:0;	my = j<y.ny ? j:0;	mz = j<z.ny ? j:0;
 		for(i=0;i<n;i++)
 		{
@@ -454,7 +507,7 @@ void mglGraph::Step(const mglData &x, const mglData &y, const mglData &z, const
 		}
 		curv_plot(2*n-1,pp,tt);
 	}
-	EndGroup();
+	SetPal(0);	EndGroup();
 	delete []tt;	delete []pp;
 }
 //-----------------------------------------------------------------------------
@@ -475,7 +528,7 @@ void mglGraph::Step(const mglData &x, const mglData &y, const char *pen,mreal zV
 	SetPal(pen);
 	for(j=0;j<m;j++)
 	{
-		Pen(Pal[(CurrPal = (CurrPal+1)%NumPal)],0,0);
+		Pen(GetPal(),0,0);
 		mx = j<x.ny ? j:0;	my = j<y.ny ? j:0;
 		for(i=0;i<n;i++)
 		{
@@ -494,7 +547,7 @@ void mglGraph::Step(const mglData &x, const mglData &y, const char *pen,mreal zV
 		}
 		curv_plot(2*n-1,pp,tt);
 	}
-	EndGroup();
+	SetPal(0);	EndGroup();
 	delete []tt;	delete []pp;
 }
 //-----------------------------------------------------------------------------
@@ -526,7 +579,7 @@ void mglGraph::Stem(const mglData &x, const mglData &y, const mglData &z, const
 	SetPal(pen);
 	for(j=0;j<m;j++)
 	{
-		Pen(Pal[(CurrPal = (CurrPal+1)%NumPal)],0,0);
+		Pen(GetPal(),0,0);
 		mx = j<x.ny ? j:0;	my = j<y.ny ? j:0;	mz = j<z.ny ? j:0;
 		for(i=0;i<n;i++)
 		{
@@ -541,7 +594,7 @@ void mglGraph::Stem(const mglData &x, const mglData &y, const mglData &z, const
 		}
 		curv_plot(2*n,pp,tt,nn);
 	}
-	EndGroup();
+	SetPal(0);	EndGroup();
 	delete []tt;	delete []pp;	delete []nn;
 }
 //-----------------------------------------------------------------------------
@@ -563,7 +616,7 @@ void mglGraph::Stem(const mglData &x, const mglData &y, const char *pen,mreal zV
 	SetPal(pen);
 	for(j=0;j<m;j++)
 	{
-		Pen(Pal[(CurrPal = (CurrPal+1)%NumPal)],0,0);
+		Pen(GetPal(),0,0);
 		mx = j<x.ny ? j:0;	my = j<y.ny ? j:0;
 		for(i=0;i<n;i++)
 		{
@@ -578,7 +631,7 @@ void mglGraph::Stem(const mglData &x, const mglData &y, const char *pen,mreal zV
 		}
 		curv_plot(2*n,pp,tt,nn);
 	}
-	EndGroup();
+	SetPal(0);	EndGroup();
 	delete []tt;	delete []pp;	delete []nn;
 }
 //-----------------------------------------------------------------------------
@@ -608,7 +661,7 @@ void mglGraph::Bars(const mglData &x, const mglData &y, const mglData &z, const
 	SetPal(pen);
 	for(j=0;j<m;j++)
 	{
-		DefColor(Pal[(CurrPal = (CurrPal+1)%NumPal)], -1);
+		DefColor(GetPal(), -1);
 		mx = j<x.ny ? j:0;	my = j<y.ny ? j:0;	mz = j<z.ny ? j:0;
 		for(i=0;i<n;i++)
 		{
@@ -652,7 +705,7 @@ void mglGraph::Bars(const mglData &x, const mglData &y, const mglData &z, const
 			if(tt[4*i+2] && tt[4*i+3])	line_plot(pt+6, pt+9, ct, ct);
 		}
 	}
-	EndGroup();
+	SetPal(0);	EndGroup();
 	delete []tt;	delete []pp;
 }
 //-----------------------------------------------------------------------------
@@ -671,7 +724,7 @@ void mglGraph::Bars(const mglData &x, const mglData &y, const char *pen,mreal zV
 	SetPal(pen);
 	for(j=0;j<m;j++)
 	{
-		DefColor(Pal[(CurrPal = (CurrPal+1)%NumPal)], -1);
+		DefColor(GetPal(), -1);
 		mx = j<x.ny ? j:0;	my = j<y.ny ? j:0;
 		for(i=0;i<n;i++)
 		{
@@ -703,7 +756,7 @@ void mglGraph::Bars(const mglData &x, const mglData &y, const char *pen,mreal zV
 			if(tt[4*i+2] && tt[4*i+3])	line_plot(pt+6, pt+9, ct, ct);
 		}
 	}
-	EndGroup();
+	SetPal(0);	EndGroup();
 	delete []tt;	delete []pp;
 }
 //-----------------------------------------------------------------------------
@@ -734,7 +787,7 @@ void mglGraph::Barh(const mglData &y, const mglData &v, const char *pen,mreal zV
 	SetPal(pen);
 	for(j=0;j<m;j++)
 	{
-		DefColor(Pal[(CurrPal = (CurrPal+1)%NumPal)], -1);
+		DefColor(GetPal(), -1);
 		my = j<y.ny ? j:0;	mx = j<v.ny ? j:0;
 		for(i=0;i<n;i++)
 		{
@@ -766,7 +819,7 @@ void mglGraph::Barh(const mglData &y, const mglData &v, const char *pen,mreal zV
 			if(tt[4*i+2] && tt[4*i+3])	line_plot(pt+6, pt+9, ct, ct);
 		}
 	}
-	EndGroup();
+	SetPal(0);	EndGroup();
 	delete []tt;	delete []pp;
 }
 //-----------------------------------------------------------------------------
@@ -779,6 +832,84 @@ void mglGraph::Barh(const mglData &v, const char *pen,mreal zVal, bool above)
 }
 //-----------------------------------------------------------------------------
 //
+//	Bars series
+//
+//-----------------------------------------------------------------------------
+double sgn(double a);
+int mgl_cmp_flt(const void *a, const void *b)
+{
+	const mreal *aa = (const mreal *)a;
+	const mreal *bb = (const mreal *)b;
+	return int(sgn(*aa-*bb));
+}
+//-----------------------------------------------------------------------------
+void mglGraph::BoxPlot(const mglData &x, const mglData &y, const char *pen,mreal zVal)
+{
+	long n=y.nx, m=y.ny;
+	if(x.nx!=n)	{	SetWarn(mglWarnDim,"BoxPlot");	return;	}
+//	if(n<2)		{	SetWarn(mglWarnLow,"BoxPlot");	return;	}
+	static int cgid=1;	StartGroup("BoxPlot",cgid++);
+	mreal *b = new mreal[5*n], *pp = new mreal[6*9*n], *d = new mreal[m], x1, x2;
+	bool *tt = new bool[2*9*n];
+	if(isnan(zVal))	zVal = Min.z;
+	register long i,j,i0;
+	for(i=0;i<n;i++)	// find quartiles by itself
+	{
+		register long mm,k;
+		for(mm=j=0;j<m;j++)	if(!isnan(y.a[i+n*j]))
+		{	d[mm]=y.a[i+n*j];	mm++;	}
+		if(m==0)	{	b[i]=NAN;	break;	}
+		qsort(d, mm, sizeof(mreal), mgl_cmp_flt);
+		b[i] = d[0];	b[i+4*n] = d[mm-1];		k = mm/4;
+		b[i+n] = (mm%4) ? d[k] : (d[k]+d[k-1])/2.;
+		b[i+2*n] = (mm%2) ? d[mm/2] : (d[mm/2]+d[mm/2-1])/2.;
+		b[i+3*n] = (mm%4) ? d[mm-k-1] : (d[mm-k-1]+d[mm-k])/2.;
+	}
+	delete []d;
+
+	SetPal(pen);//	DefColor(GetPal(), -1);
+	mglColor c1=cmap[0], c2=cmap[1];
+	cmap[0] = cmap[1] = GetPal();
+	for(j=0;j<18*n;j++)	pp[2+3*j] = zVal;
+	for(i=0;i<n;i++)
+	{
+		i0 = 54*i;
+		if(i<n-1)	x2 = x.a[i] + BarWidth*(x.a[i+1]-x.a[i])/2;
+		else		x2 = x.a[i];
+		if(i>0)		x1 = x.a[i] - BarWidth*(x.a[i]-x.a[i-1])/2;
+		else		x1 = x.a[i];
+		// x-coordinates of lines
+		pp[i0] = pp[i0+12]=pp[i0+18]=pp[i0+24]=pp[i0+30]=pp[i0+33]=pp[i0+48]=x1;
+		pp[i0+3]=pp[i0+15]=pp[i0+21]=pp[i0+27]=pp[i0+36]=pp[i0+39]=pp[i0+51]=x2;
+		pp[i0+6]=pp[i0+9] =pp[i0+42]=pp[i0+45]=x.a[i];
+		// y-coordinates of lines
+		pp[i0+1] = pp[i0+4] = pp[i0+7] = b[i];
+		pp[i0+49] = pp[i0+52] = pp[i0+46] = b[i+4*n];
+		pp[i0+19] = pp[i0+22] = b[i+2*n];
+		pp[i0+10] = pp[i0+13] = pp[i0+16] = pp[i0+31] = pp[i0+37] = b[i+n];
+		pp[i0+25] = pp[i0+28] = pp[i0+34] = pp[i0+40] = pp[i0+43] = b[i+3*n];
+		if(isnan(b[i]) || isnan(b[i+n]) || isnan(b[i+2*n]) || isnan(b[i+3*n]) || isnan(b[i+4*n]))
+			for(j=0;j<18;j++)	tt[j+18*i] = false;
+		else	for(j=0;j<18;j++)
+		{
+			i0 = j+18*i;
+			tt[i0] = ScalePoint(pp[3*i0],pp[3*i0+1],pp[3*i0+2]);
+		}
+	}
+	lines_plot(9*n,pp,0,tt,false);
+	cmap[0]=c1;	cmap[1]=c2;	SetPal(0);	EndGroup();
+	delete []b;		delete []tt;	delete []pp;
+}
+//-----------------------------------------------------------------------------
+void mglGraph::BoxPlot(const mglData &y, const char *pen,mreal zVal)
+{
+	if(y.nx<2)	{	SetWarn(mglWarnLow,"BoxPlot");	return;	}
+	mglData x(y.nx);
+	x.Fill(Min.x,Max.x);
+	BoxPlot(x,y,pen,zVal);
+}
+//-----------------------------------------------------------------------------
+//
 //		Torus series
 //
 //-----------------------------------------------------------------------------
@@ -995,7 +1126,7 @@ void mglGraph::Mark(const mglData &x, const mglData &y, const mglData &z, const
 	SetPal(pen);
 	for(j=0;j<m;j++)
 	{
-		Pen(Pal[(CurrPal = (CurrPal+1)%NumPal)],0,0);
+		Pen(GetPal(),0,0);
 		mx = j<x.ny ? j:0;	my = j<y.ny ? j:0;
 		mz = j<z.ny ? j:0;	mr = j<r.ny ? j:0;
 		register long i;
@@ -1007,8 +1138,7 @@ void mglGraph::Mark(const mglData &x, const mglData &y, const mglData &z, const
 			if(tt)	Mark(xx,yy,zz,mk);
 		}
 	}
-	MarkSize = ms;
-	EndGroup();
+	MarkSize = ms;	SetPal(0);	EndGroup();
 }
 //-----------------------------------------------------------------------------
 void mglGraph::Mark(const mglData &x, const mglData &y, const mglData &r, const char *pen,mreal zVal)
@@ -1127,7 +1257,9 @@ void mglGraph::tube_plot(long n,mreal *q,mreal *c,mreal *r)
 			ScalePoint(pp[3*i0],pp[3*i0+1],pp[3*i0+2]);
 		}
 	}
+	int mnt=mglNumThr;	mglNumThr = 1;	// too many faces
 	surf_plot(CirclePnts,m,pp,c?cc:0,tt);
+	mglNumThr = mnt;
 	delete []pp;	delete []cc;	delete []tt;
 }
 //-----------------------------------------------------------------------------
@@ -1144,7 +1276,7 @@ void mglGraph::Tube(const mglData &x, const mglData &y, const mglData &z, const
 	SelectPen(pen);	SetPal(pen);
 	for(j=0;j<m;j++)
 	{
-		DefColor(Pal[(CurrPal = (CurrPal+1)%NumPal)], -1);
+		DefColor(GetPal(), -1);
 		mx = j<x.ny ? j:0;	my = j<y.ny ? j:0;
 		mz = j<z.ny ? j:0;	mr = j<r.ny ? j:0;
 		register long i;
@@ -1156,7 +1288,7 @@ void mglGraph::Tube(const mglData &x, const mglData &y, const mglData &z, const
 		}
 		tube_plot(n,pp,0,rr);
 	}
-	EndGroup();
+	SetPal(0);	EndGroup();
 	delete []pp;	delete []rr;
 }
 //-----------------------------------------------------------------------------
@@ -1340,7 +1472,13 @@ void mgl_plot_2(HMGL gr, const HMDT a, const char *pen)
 /// Draw line plot for points in arrays \a a(0,:),\a a(1,:),\a a(2,:).
 void mgl_plot_3(HMGL gr, const HMDT a, const char *pen)
 {	if(gr && a)	gr->Plot3(*a,pen);	}
-/// Draw line plot for points in arrays \a y.
+/// Draw boxplot for points in arrays \a x, \a y.
+void mgl_boxplot_xy(HMGL gr, const HMDT x, const HMDT y, const char *pen)
+{	if(gr && x && y)	gr->BoxPlot(*x,*y,pen);	}
+/// Draw bozplot for points in arrays \a y.
+void mgl_boxplot(HMGL gr, const HMDT y, const char *pen)
+{	if(gr && y)	gr->BoxPlot(*y,pen);	}
+/// Draw radar plot for points in arrays \a y.
 void mgl_radar(HMGL gr, const HMDT a, const char *pen, mreal r)
 {	if(gr && a)	gr->Radar(*a,pen,r);	}
 /// Draw line plot for points in arrays \a x, \a y, \a z.
@@ -1362,6 +1500,8 @@ void mgl_area_xy(HMGL gr, const HMDT x, const HMDT y, const char *pen)
 void mgl_area_xys(HMGL gr, const HMDT x, const HMDT y, const char *pen)
 {	if(gr && x && y)	gr->Area(*x,*y,pen,true);	}
 /// Draw area plot for points in arrays \a y.
+void mgl_area_s(HMGL gr, const HMDT y, const char *pen)
+{	if(gr && y)	gr->Area(*y,pen,true);	}
 void mgl_area(HMGL gr, const HMDT y, const char *pen)
 {	if(gr && y)	gr->Area(*y,pen);	}
 /// Draw area plot for points in arrays \a a(0,:),\a a(1,:).
@@ -1514,7 +1654,21 @@ void mgl_plot_3_(uintptr_t *gr, uintptr_t *a, const char *pen,int l)
 	if(gr && a)	_GR_->Plot3(_D_(a),s);
 	delete []s;
 }
-/// Draw line plot for points in arrays \a y.
+/// Draw boxplot for points in arrays \a x, \a y.
+void mgl_boxplot_xy_(uintptr_t *gr, uintptr_t *x, uintptr_t *y, const char *pen,int l)
+{
+	char *s=new char[l+1];	memcpy(s,pen,l);	s[l]=0;
+	if(gr && x && y)	_GR_->BoxPlot(_D_(x),_D_(y),s);
+	delete []s;
+}
+/// Draw boxplot for points in arrays \a y.
+void mgl_boxplot_(uintptr_t *gr, uintptr_t *y,	const char *pen,int l)
+{
+	char *s=new char[l+1];	memcpy(s,pen,l);	s[l]=0;
+	if(gr && y)	_GR_->BoxPlot(_D_(y),s);
+	delete []s;
+}
+/// Draw radar plot for points in arrays \a y.
 void mgl_radar_(uintptr_t *gr, uintptr_t *a, const char *pen, mreal *r, int l)
 {
 	char *s=new char[l+1];	memcpy(s,pen,l);	s[l]=0;
@@ -1558,6 +1712,12 @@ void mgl_area_xy_(uintptr_t *gr, uintptr_t *x, uintptr_t *y, const char *pen,int
 	delete []s;
 }
 /// Draw area plot for points in arrays \a y.
+void mgl_area_s_(uintptr_t *gr, uintptr_t *y, const char *pen,int l)
+{
+	char *s=new char[l+1];	memcpy(s,pen,l);	s[l]=0;
+	if(gr && y)	_GR_->Area(_D_(y),s,true);
+	delete []s;
+}
 void mgl_area_(uintptr_t *gr, uintptr_t *y, const char *pen,int l)
 {
 	char *s=new char[l+1];	memcpy(s,pen,l);	s[l]=0;
