@@ -3,17 +3,17 @@
  * Copyright (C) 2007 Alexey Balakin <balakin@appl.sci-nnov.ru>            *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
- *   it under the terms of the GNU General Public License as published by  *
- *   the Free Software Foundation; either version 2 of the License, or     *
- *   (at your option) any later version.                                   *
+ *   it under the terms of the GNU Library General Public License as       *
+ *   published by the Free Software Foundation; either version 3 of the    *
+ *   License, or (at your option) any later version.                       *
  *                                                                         *
  *   This program is distributed in the hope that it will be useful,       *
  *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
  *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
  *   GNU General Public License for more details.                          *
  *                                                                         *
- *   You should have received a copy of the GNU General Public License     *
- *   along with this program; if not, write to the                         *
+ *   You should have received a copy of the GNU Library General Public     *
+ *   License along with this program; if not, write to the                 *
  *   Free Software Foundation, Inc.,                                       *
  *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
  ***************************************************************************/
@@ -24,7 +24,7 @@
 #include "mgl/mgl_ab.h"
 #define imax(a,b)	(a)>(b) ? (a) : (b)
 #define imin(a,b)	(a)<(b) ? (a) : (b)
-
+int mglNumThr;
 char *mgl_strdup(const char *s);
 //-----------------------------------------------------------------------------
 void strtrim_mgl(char *str);
@@ -36,8 +36,8 @@ mglGraphAB::mglGraphAB(int w,int h) : mglGraph()
 {
 	G = 0;	UseLight = false;	st_pos=-1;
 	memset(stack,0,MGL_STACK_ENTRY*13*sizeof(mreal));
-	SetSize(w,h);
-	AutoClf=true;	Delay = 100;
+	SetSize(w,h);	ClfOnUpdate = true;	SetDrawReg(1,1,0);
+	AutoClf=true;	Delay = 100;	ObjId=0;
 	NoAutoFactor = false;	ShowMousePos = true;
 	BDef[0] = BDef[1] = BDef[2] = BDef[3] = 255;
 	DefaultPlotParam();
@@ -123,24 +123,14 @@ void mglGraphAB::RotateN(mreal Tet,mreal x,mreal y,mreal z)
 	B[8] = C[2]*R[6] + C[5]*R[7] + C[8]*R[8];
 	if(AutoPlotFactor && !NoAutoFactor)
 	{
-		register mreal m = fabs(B[3]+B[4]+B[5]);
-		m = m<fabs(B[3]+B[4]-B[5]) ? fabs(B[3]+B[4]-B[5]) : m;
-		m = m<fabs(B[3]-B[4]+B[5]) ? fabs(B[3]-B[4]+B[5]) : m;
-		m = m<fabs(B[3]-B[4]-B[5]) ? fabs(B[3]-B[4]-B[5]) : m;
-		m /= B1[4];
-		register mreal n = fabs(B[0]+B[1]+B[2]);
-		n = n<fabs(B[0]+B[1]-B[2]) ? fabs(B[0]+B[1]-B[2]) : n;
-		n = n<fabs(B[0]-B[1]+B[2]) ? fabs(B[0]-B[1]+B[2]) : n;
-		n = n<fabs(B[0]-B[1]-B[2]) ? fabs(B[0]-B[1]-B[2]) : n;
-		n /= B1[0];
+		mreal m=(fabs(B[3])+fabs(B[4])+fabs(B[5]))/inH;
+		mreal n=(fabs(B[0])+fabs(B[1])+fabs(B[2]))/inW;
 		PlotFactor = 1.55+0.6147*(m<n ? (n-1):(m-1));
 	}
 }
 //-----------------------------------------------------------------------------
 void mglGraphAB::Perspective(mreal a)	// I'm too lazy for using 4*4 matrix
-{
-	Persp = fabs(a)/Depth;
-}
+{	Persp = fabs(a)/Depth;	}
 //-----------------------------------------------------------------------------
 void mglGraphAB::RestoreM()
 {
@@ -177,8 +167,9 @@ void mglGraphAB::InPlot(mreal x1,mreal x2,mreal y1,mreal y2, bool rel)
 		B1[11]= zPos = (1.f-B[8]/(2*Depth))*Depth;
 		memcpy(B1,B,9*sizeof(mreal));
 	}
+	inW = B[0];	inH=B[4];
 	font_factor = B[0] < B[4] ? B[0] : B[4];
-	PlotFactor = AutoPlotFactor ? 1.55 : 2;	// Automatically change plot factor !!!
+	if(AutoPlotFactor) PlotFactor = 1.55;	// Automatically change plot factor !!!
 	Persp = 0;
 }
 //-----------------------------------------------------------------------------
@@ -193,6 +184,24 @@ void mglGraphAB::Aspect(mreal Ax,mreal Ay,mreal Az)
 	B[2] *= Az;		B[5] *= Az;		B[8] *= Az;
 }
 //-----------------------------------------------------------------------------
+void mglGraphAB::StickPlot(int num, int id, mreal tet, mreal phi)
+{
+	mreal dx,dy,w0,h0, p[6]={-1,0,0,1,0,0};
+	InPlot(0,1,0,1,true);	Rotate(tet, phi);	PostScale(p,2);
+	w0=1/(1+(num-1)*fabs(p[3]-p[0])/inW);		dx=(p[3]-p[0])*w0/inW;
+	h0=1/(1+(num-1)*fabs(p[4]-p[1])/inH);		dy=(p[4]-p[1])*h0/inH;
+
+	p[0]=-1;	p[3]=1;		p[1]=p[2]=p[4]=p[5]=0;	// extra widening
+	InPlot(dx>0?0:1-w0, dx>0?w0:1, dy>0?0:1-h0, dy>0?h0:1, true);
+	Rotate(tet,phi);	PostScale(p,2);
+	w0=1/(1+(num-1)*fabs(p[3]-p[0])/inW);		dx=(p[3]-p[0])*w0/inW;
+	h0=1/(1+(num-1)*fabs(p[4]-p[1])/inH);		dy=(p[4]-p[1])*h0/inH;
+
+	mreal x1=dx>0?dx*id:1-w0+dx*id, x2=dx>0?w0+dx*id:1+dx*id;
+	mreal y1=dy>0?dy*id:1-h0+dy*id, y2=dy>0?h0+dy*id:1+dy*id;
+	InPlot(x1, x2, y1, y2, true);	Rotate(tet,phi);
+}
+//-----------------------------------------------------------------------------
 void mglGraphAB::NormScale(mreal *p,long n)
 {
 	register long i;
@@ -261,6 +270,18 @@ mglPoint mglGraphAB::CalcXYZ(int xs, int ys)
 	return LastMousePos;
 }
 //-----------------------------------------------------------------------------
+void mglGraphAB::CalcScr(mglPoint p, int *xs, int *ys)
+{
+	mreal pp[3]={p.x,p.y,p.z};
+	ScalePoint(pp[0],pp[1],pp[2]);
+	PostScale (pp,1);
+	if(xs)	*xs=int(pp[0]);
+	if(ys)	*ys=int(pp[1]);
+}
+//-----------------------------------------------------------------------------
+mglPoint mglGraphAB::CalcScr(mglPoint p)
+{	int x,y;	CalcScr(p,&x,&y);	return mglPoint(x,y);	}
+//-----------------------------------------------------------------------------
 void mglGraphAB::LightScale()
 {
 	mreal *x;
@@ -343,7 +364,26 @@ mreal mglGraphAB::Putsw(mglPoint p,mglPoint n,const wchar_t *str,char font,mreal
 //-----------------------------------------------------------------------------
 void mglGraphAB::Putsw(mglPoint p, const wchar_t *wcs, const char *font, mreal size, char dir, mreal sh)
 {
+	static int nl_shift = 0;
 	static int cgid=1;	StartGroup("Putsw",cgid++);
+	if(font && strchr(font, 'A'))	{	Labelw(p.x, p.y, wcs,font,size,false);	return;	}
+	if(font && strchr(font, 'a'))	{	Labelw(p.x, p.y, wcs,font,size,true);	return;	}
+
+	wchar_t *wcl;
+	int wn=0;
+	const wchar_t *wnl=0;
+	for(wn=0;wn<wcslen(wcs);wn++)
+		if(wcs[wn]=='\n' || (wcs[wn]=='\\' && wcs[wn+1]=='n'))
+		{	wnl = wcs+wn;	break;	}
+	if(wnl)
+	{
+		wcl = new wchar_t[wn+1];	memcpy(wcl,wcs,wn*sizeof(wchar_t));
+		wcl[wn]=0;	Putsw(p, wcl, font, size, dir, sh);
+		wnl = wcs[wn]=='\n'?wnl+1:wnl+2;
+		nl_shift++;	Putsw(p, wnl, font, size, dir, sh);	nl_shift--;
+		return;
+	}
+
 	bool upside = ( (((_sx==-1) ^ (Org.y==Max.y || Org.z==Max.z)) && (dir=='x' || dir=='X')) ||
 					(((_sy==-1) ^ (Org.x==Max.x || Org.z==Max.z)) && (dir=='y' || dir=='Y')) ||
 					(((_st==-1) ^ (Org.x==0 || Org.z==1)) && (dir=='t' || dir=='T')) ||
@@ -364,20 +404,21 @@ void mglGraphAB::Putsw(mglPoint p, const wchar_t *wcs, const char *font, mreal s
 	zoomx1=zoomy1=0;	zoomx2=zoomy2=1;
 
 	if(strchr("xyzt",dir))	shift = (sh/10+0.1)*2/PlotFactor;
+//	shift *= size/7;
 	switch(dir)
 	{
 	case 'x':
 	case 'X':
-		pp[3] += (Max.x - Min.x)/100;	break;
+		pp[3] += pp[3] ? fabs(pp[3])*.001 :(Max.x - Min.x)/100;	break;
 	case 'y':
 	case 'Y':
 		if(TernAxis)	upside = !upside;
-		pp[4] += (Max.y - Min.y)/10;	break;
+		pp[4] += pp[4] ? fabs(pp[4])*.001 : (Max.y - Min.y)/10;	break;
 	case 'z':
 	case 'Z':
 		if(TernAxis)	upside = !upside;
 		pp[4] -= 0;
-		pp[5] += (Max.z - Min.z)/100;	break;
+		pp[5] += pp[5] ? fabs(pp[5])*.001 :(Max.z - Min.z)/100;	break;
 	case 't':
 	case 'T':
 //		upside = !upside;
@@ -432,8 +473,8 @@ void mglGraphAB::Putsw(mglPoint p, const wchar_t *wcs, const char *font, mreal s
 		memset(B,0,9*sizeof(mreal));	B[0] = B[4] = B[8] = fsize;
 		mreal ss = (pp[3]>pp[0] || tet==-M_PI/2) ? 1 : -1;
 		fscl = fsize;	ftet = 0;
-		xPos = pp[0]+shift*ss*(pp[4]-pp[1])/sqrt(ll);
-		yPos = pp[1]-shift*ss*(pp[3]-pp[0])/sqrt(ll);
+		xPos = pp[0]+shift*ss*(pp[4]-pp[1])/sqrt(ll) - B[1]*0.02f;
+		yPos = pp[1]-shift*ss*(pp[3]-pp[0])/sqrt(ll) - B[4]*0.02f;
 		zPos = pp[2];
 
 		if(!strchr(font1,'R') && !strchr(font1,'C') && !strchr(font1,'L'))
@@ -445,6 +486,7 @@ void mglGraphAB::Putsw(mglPoint p, const wchar_t *wcs, const char *font, mreal s
 		}
 	}
 	zoomx1=x1;	zoomx2=x2;	zoomy1=y1;	zoomy2=y2;
+	yPos -= nl_shift*shift*1.25;
 	fnt->Puts(wcs,font1,col);
 	Pop();	delete []font1;	EndGroup();
 }
@@ -507,94 +549,17 @@ void mglGraphAB::Legend(int n, wchar_t **text,char **style, mreal x, mreal y,
 	Pop();	EndGroup();
 }
 //-----------------------------------------------------------------------------
-/*void mglGraphAB::Colorbar(int where, mreal x, mreal y, mreal w, mreal h)
-{
-	static int cgid=1;	StartGroup("Colorbar",cgid++);
-	register long i;
-	mreal *pp,*cc,d,s3=PlotFactor,ss=s3*0.9;
-	mglColor c;
-
-	pp = new mreal[6*256];	cc = new mreal[8*256];
-	x = 2*x-1;	y = 2*y-1;
-	for(i=0;i<255;i++)
-	{
-		d = 2*i/254.-1;
-		pp[6*i+0] = pp[6*i+3] = (ss*d+s3)*w+x*s3;
-		pp[6*i+1] = pp[6*i+4] = (ss*d+s3)*h+y*s3;
-		pp[6*i+2] = pp[6*i+5] = s3;
-		switch(where)
-		{
-		case 1:	pp[6*i]  = x*s3;	pp[6*i+3] = (x+0.1*w)*s3;	break;
-		case 2:	pp[6*i+1]= (y-0.1*h)*s3;	pp[6*i+4] = y*s3;	break;
-		case 3:	pp[6*i+1]= y*s3;	pp[6*i+4] = (y+0.1*h)*s3;	break;
-		default:pp[6*i]  = (x-0.1*w)*s3;	pp[6*i+3] = x*s3;	break;
-		}
-		c = GetC(d,false);
-		cc[8*i+0] = cc[8*i+4] = c.r;
-		cc[8*i+1] = cc[8*i+5] = c.g;
-		cc[8*i+2] = cc[8*i+6] = c.b;
-		cc[8*i+3] = cc[8*i+7] = 1;
-	}
-	Push();	memcpy(B,B1,9*sizeof(mreal));
-	bool ll = UseLight;				UseLight = false;
-	surf_plot(2, 255, pp, cc, 0);	UseLight = ll;
-	delete []pp;	delete []cc;
-
-	const char *a="rC";
-	if(where==0)	a = "rR";
-	if(where==1)	a = "rL";
-	if(where==2)	a = "rDC";
-	mglPoint p;
-	ScalePuts = false;
-	SelectPen(TranspType!=2 ? "k-1":"w-1");
-
-	mreal v=0,t;
-	int kind=0;
-	wchar_t s[32]=L"", str[64];
-	if(!ctt && TuneTicks) kind = _mgl_tick_ext(Cmax, Cmin, s, v);
-
-	for(i=0;i<5;i++)
-	{
-		d = i*0.5-1;	p.z = s3+1;
-		p.x = (ss*d+s3)*w+x*s3;
-		p.y = (ss*d+s3)*h+y*s3;
-		switch(where)
-		{
-		case 1:	p.x = (x+0.13*w)*s3;	break;
-		case 2:	p.y = (y-0.13*h)*s3;	break;
-		case 3:	p.y = (y+0.13*h)*s3;	break;
-		default:p.x = (x-0.13*w)*s3;	break;
-		}
-		t = Cmin+i*(Cmax-Cmin)/4;
-		if(ctt)	mglprintf(str, 64, ctt, t);
-		else	_mgl_tick_text(t,Cmin,(Cmax-Cmin)/100,v,kind,str);
-		wcstrim_mgl(str);
-		Putsw(p,str,a,FontSize);
-	}
-	switch(where)
-	{
-	case 1:	p = mglPoint((x+0.15*w)*s3, (y+1.75*h)*s3);	break;
-	case 2:	p = mglPoint((x+1.75*w)*s3, (y-0.15*h)*s3);	break;
-	case 3:	p = mglPoint((x+1.75*w)*s3, (y+0.15*h)*s3);	break;
-	default:p = mglPoint((x-0.15*w)*s3, (y+1.75*h)*s3);	break;
-	}
-	if(kind&2)	Putsw(p,s,a,FontSize);
-	ScalePuts = true;
-	Pop();	EndGroup();
-}*/
-//-----------------------------------------------------------------------------
 void mglGraphAB::colorbar(const mglData &vv, const mglColor *cs, int where, mreal x, mreal y, mreal w, mreal h)
 {
 	static int cgid=1;	StartGroup("Colorbar",cgid++);
 	register long i,n=vv.nx;
-	mreal *pp,*cc,d,s3=PlotFactor,ss=s3*0.9,v1=vv.Minimal(),dv;
-	dv=vv.Maximal()-v1;	if(dv)	dv=2/dv;
+	mreal *pp,*cc,d,s3=PlotFactor,ss=s3*0.9,v1=vv.Minimal();
 
 	pp = new mreal[12*n];	cc = new mreal[16*n];
 	x = 2*x-1;	y = 2*y-1;
 	for(i=0;i<n-1;i++)
 	{
-		d = (vv.a[i]-v1)*dv-1;
+		d = GetA(vv.a[i]);	//(vv.a[i]-v1)*dv-1;
 		pp[12*i+0] = pp[12*i+3] = (ss*d+s3)*w+x*s3;
 		pp[12*i+1] = pp[12*i+4] = (ss*d+s3)*h+y*s3;
 		pp[12*i+2] = pp[12*i+5] = s3;
@@ -605,7 +570,7 @@ void mglGraphAB::colorbar(const mglData &vv, const mglColor *cs, int where, mrea
 			case 3:	pp[12*i+1]= y*s3;	pp[12*i+4] = (y+0.1*h)*s3;	break;
 			default:pp[12*i]  = (x-0.1*w)*s3;	pp[12*i+3] = x*s3;	break;
 		}
-		d = (vv.a[i+1]-v1)*dv-1;
+		d = GetA(vv.a[i+1]);	//(vv.a[i+1]-v1)*dv-1;
 		pp[12*i+6] = pp[12*i+9] = (ss*d+s3)*w+x*s3;
 		pp[12*i+7] = pp[12*i+10]= (ss*d+s3)*h+y*s3;
 		pp[12*i+8] = pp[12*i+11]= s3;
@@ -625,7 +590,6 @@ void mglGraphAB::colorbar(const mglData &vv, const mglColor *cs, int where, mrea
 	bool ll = UseLight;			UseLight = false;
 //	surf_plot(2, n-1, pp, cc, 0);	UseLight = ll;
 	quads_plot(n-1, pp, cc, 0);	UseLight = ll;
-	delete []pp;	delete []cc;
 
 	const char *a="rC";
 	if(where==0)	a = "rR";
@@ -638,22 +602,27 @@ void mglGraphAB::colorbar(const mglData &vv, const mglColor *cs, int where, mrea
 	mreal v=0,t;
 	int kind=0;
 	wchar_t s[32]=L"", str[64];
-	if(!ctt && TuneTicks) kind = _mgl_tick_ext(Cmax, Cmin, s, v);
+	if(ctt[0]==0 && TuneTicks) kind = _mgl_tick_ext(Cmax, Cmin, s, v);
 
 	long m=n;
-	if(n>20)	// adjust ticks
+/*	if(n>20)	// adjust ticks
 	{
 		float d = fabs(vv.Maximal()-v1);
 		d = floor(d*pow(10,-floor(log10(d))));
 		if(d<4)	m = 1+int(2*d+0.5);
 		else	m = 1+int(d+0.5);
 		if(m<5)	m = 5;
-	}
-	
-//	long m = n<=20?n:6;
+	}*/
+	if(dc<0) m=int(0.5-dc);
+	if(dc>0) m=int(0.5+(Cmax-Cmin)/dc);
+	if(dc==0 && Cmax*Cmin>0)	m = 1+int(0.5+fabs(log10(Cmax/Cmin)));
+	if(m<2)	m=2;
+	if(m>n)	m=n;
+
+	mreal dv=(vv.Maximal()-v1)/m;
 	for(i=0;i<m;i++)
 	{
-		t = vv.a[((n-1)*i)/(m-1)];	d = (t-v1)*dv-1;	p.z = s3+1;
+		t = vv.a[(n-1)*i/(m-1)];	d = GetA(t);	p.z = s3+1;
 		p.x = (ss*d+s3)*w+x*s3;
 		p.y = (ss*d+s3)*h+y*s3;
 		switch(where)
@@ -663,21 +632,39 @@ void mglGraphAB::colorbar(const mglData &vv, const mglColor *cs, int where, mrea
 			case 3:	p.y = (y+0.13*h)*s3;	break;
 			default:p.x = (x-0.13*w)*s3;	break;
 		}
-		if(ctt)	mglprintf(str, 64, ctt, t);
+		if(ctt[0])	mglprintf(str, 64, ctt, t);
+		else if(dc==0)
+		{
+			long kk=int(floor(0.1+log10(fabs(t))));
+			mglprintf(str,64,L"%.2g\\cdot 10^{%d}",t/pow(10,kk), kk);
+		}
 		else	_mgl_tick_text(t,v1,dv/100,v,kind,str);
-		wcstrim_mgl(str);
-		Putsw(p,str,a,FontSize);
+		wcstrim_mgl(str);	Putsw(p,str,a,FontSize);
+		// draw "grid" lines
+		pp[0] = pp[3] = (ss*d+s3)*w+x*s3;
+		pp[1] = pp[4] = (ss*d+s3)*h+y*s3;
+		pp[2] = pp[5] = s3+1;
+		switch(where)
+		{
+			case 1:	pp[0]  = x*s3;	pp[3] = (x+0.1*w)*s3;	break;
+			case 2:	pp[1]= (y-0.1*h)*s3;	pp[4] = y*s3;	break;
+			case 3:	pp[1]= y*s3;	pp[4] = (y+0.1*h)*s3;	break;
+			default:pp[0]  = (x-0.1*w)*s3;	pp[3] = x*s3;	break;
+		}
+		memset(cc,0,6*sizeof(mreal));	PostScale (pp,2);
+		line_plot(pp,pp+3,cc,cc+3,true);
 	}
 	switch(where)
 	{
-		case 1:	p = mglPoint((x+0.15*w)*s3, (y+1.75*h)*s3);	break;
-		case 2:	p = mglPoint((x+1.75*w)*s3, (y-0.15*h)*s3);	break;
-		case 3:	p = mglPoint((x+1.75*w)*s3, (y+0.15*h)*s3);	break;
-		default:p = mglPoint((x-0.15*w)*s3, (y+1.75*h)*s3);	break;
+		case 1:	p = mglPoint((x+0.15*w)*s3, (y+1.75*h)*s3, s3+1);	break;
+		case 2:	p = mglPoint((x+1.75*w)*s3, (y-0.15*h)*s3, s3+1);	break;
+		case 3:	p = mglPoint((x+1.75*w)*s3, (y+0.15*h)*s3, s3+1);	break;
+		default:p = mglPoint((x-0.15*w)*s3, (y+1.75*h)*s3, s3+1);	break;
 	}
 	if(kind&2)	Putsw(p,s,a,FontSize);
 	ScalePuts = true;
 	Pop();	EndGroup();
+	delete []pp;	delete []cc;
 }
 //-----------------------------------------------------------------------------
 void mglGraphAB::SetSize(int w,int h)
@@ -685,11 +672,12 @@ void mglGraphAB::SetSize(int w,int h)
 	if(w<=0 || h<=0)	{	SetWarn(mglWarnSize);	return;	}
 	Width = w;	Height = h;	Depth = long(sqrt(mreal(w*h)));
 	zPos = 0;//long(2*Depth*sqrt(3.));
-	if(G)	{	delete []G;	delete []G4;	}
+	if(G)	{	delete []G;	delete []G4;	delete []OI;	}
 	G = new unsigned char[w*h*3];
 	G4= new unsigned char[w*h*4];
+	OI= new int[w*h];
 	Clf();
-	InPlot(0,1,0,1);	Persp = 0;
+	InPlot(0,1,0,1);	SetDrawReg(1,1,0);	Persp = 0;
 }
 //-----------------------------------------------------------------------------
 void mglGraphAB::WriteSVG(const char *fname, const char *descr)
@@ -928,6 +916,24 @@ mreal mglGraphAB::GetOrgZ(char dir)
 	return res;
 }
 //-----------------------------------------------------------------------------
+void mglGraphAB::DrawTick(mreal *pp,bool sub)
+{
+	ScalePoint(pp[0],pp[1],pp[2]);
+	ScalePoint(pp[3],pp[4],pp[5]);
+	ScalePoint(pp[6],pp[7],pp[8]);
+	PostScale(pp,3);
+	double dx,dy,dz,dd,ll=TickLen*(B1[0]<B1[4] ? B1[0]:B1[4])/3.5;
+	if(sub)	ll/=sqrt(1.+st_t);
+	dx=pp[0]-pp[3];	dy=pp[1]-pp[4];	dz=pp[2]-pp[5];
+	dd=sqrt(dx*dx+dy*dy+dz*dz);	if(dd)	dd = ll/dd;
+	pp[0]=pp[3]+dd*dx;	pp[1]=pp[4]+dd*dy;	pp[2]=pp[5]+dd*dz;
+	dx=pp[6]-pp[3];	dy=pp[7]-pp[4];	dz=pp[8]-pp[5];
+	dd=sqrt(dx*dx+dy*dy+dz*dz);	if(dd)	dd = ll/dd;
+	pp[6]=pp[3]+dd*dx;	pp[7]=pp[4]+dd*dy;	pp[8]=pp[5]+dd*dz;
+	line_plot(pp,pp+3,CDef,CDef);
+	line_plot(pp+3,pp+6,CDef,CDef);
+}
+//-----------------------------------------------------------------------------
 bool mglGraphAB::Alpha(bool enable)
 {	bool t=UseAlpha;	UseAlpha=enable;	return t;	}
 bool mglGraphAB::Light(bool enable)
@@ -944,7 +950,7 @@ void mglGraphAB::Adjust(){}
 void mglGraphAB::NextFrame(){}
 void mglGraphAB::PrevFrame(){}
 void mglGraphAB::Animation(){}
-void mglGraphAB::Window(int argc, char **argv, int (*draw)(mglGraph *gr, void *p), const char *title, void *par, void (*reload)(int next, void *p), bool maximize){}
+void mglGraphAB::Window(int , char **, int (*)(mglGraph *, void *), const char *, void *, void (*)(int , void *), bool ){}
 //-----------------------------------------------------------------------------
 int mgl_draw_class(mglGraph *gr, void *p)
 {	return p ? ((mglDraw *)p)->Draw(gr) : 0;	}
