@@ -3,17 +3,17 @@
  * Copyright (C) 2007 Alexey Balakin <balakin@appl.sci-nnov.ru>            *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
- *   it under the terms of the GNU General Public License as published by  *
- *   the Free Software Foundation; either version 2 of the License, or     *
- *   (at your option) any later version.                                   *
+ *   it under the terms of the GNU Library General Public License as       *
+ *   published by the Free Software Foundation; either version 3 of the    *
+ *   License, or (at your option) any later version.                       *
  *                                                                         *
  *   This program is distributed in the hope that it will be useful,       *
  *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
  *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
  *   GNU General Public License for more details.                          *
  *                                                                         *
- *   You should have received a copy of the GNU General Public License     *
- *   along with this program; if not, write to the                         *
+ *   You should have received a copy of the GNU Library General Public     *
+ *   License along with this program; if not, write to the                 *
  *   Free Software Foundation, Inc.,                                       *
  *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
  ***************************************************************************/
@@ -66,100 +66,96 @@ mreal mgl_okrugl(mreal x,int k)
 	return r;
 }
 //-----------------------------------------------------------------------------
-void mglGraph::DrawTick(mreal *pp,bool sub)
-{
-	ScalePoint(pp[0],pp[1],pp[2]);
-	ScalePoint(pp[3],pp[4],pp[5]);
-	ScalePoint(pp[6],pp[7],pp[8]);
-	curv_plot(3,pp,0);
-}
-//-----------------------------------------------------------------------------
 void mglGraph::DrawXGridLine(mreal t, mreal y0, mreal z0)
 {
 	register int i;
-	mreal pp[3*31];
-	bool tt[31];
-	for(i=0;i<31;i++)
+	mreal *pp = new mreal[3*GridPnts+3];
+	bool *tt = new bool[GridPnts+1];
+	for(i=0;i<GridPnts+1;i++)
 	{
 		pp[3*i]=t;//TernAxis ? t-t*i/30. : t;
 		pp[3*i+1]=y0;//TernAxis ? t*i/30. : y0;
-		pp[3*i+2]=Min.z+(Max.z-Min.z)*i/30.;
+		pp[3*i+2]=Min.z+(Max.z-Min.z)*i/GridPnts;
 		tt[i] = ScalePoint(pp[3*i],pp[3*i+1],pp[3*i+2]);
 	}
-	curv_plot(31,pp,tt);
-	for(i=0;i<31;i++)
+	curv_plot(GridPnts+1,pp,tt);
+	for(i=0;i<GridPnts+1;i++)
 	{
 		pp[3*i]=t;	pp[3*i+2]=z0;
-		pp[3*i+1]=Min.y+(Max.y-(TernAxis ? t:Min.y))*i/30.;
+		pp[3*i+1]=Min.y+(Max.y-(TernAxis ? t:Min.y))*i/GridPnts;
 		tt[i] = ScalePoint(pp[3*i],pp[3*i+1],pp[3*i+2]);
 	}
-	curv_plot(31,pp,tt);
+	curv_plot(GridPnts+1,pp,tt);
+	delete [] tt;	delete [] pp;
 }
 //-----------------------------------------------------------------------------
 void mglGraph::DrawYGridLine(mreal t, mreal x0, mreal z0)
 {
 	register int i;
-	mreal pp[3*31];
-	bool tt[31];
-	for(i=0;i<31;i++)
+	mreal *pp = new mreal[3*GridPnts+3];
+	bool *tt = new bool[GridPnts+1];
+	for(i=0;i<GridPnts+1;i++)
 	{
 		pp[3*i]=x0;//TernAxis ? t*i/30. : x0;
 		pp[3*i+1]=t;//TernAxis ? t-t*i/30. : t;
-		pp[3*i+2]=Min.z+(Max.z-Min.z)*i/30.;
+		pp[3*i+2]=Min.z+(Max.z-Min.z)*i/GridPnts;
 		tt[i] = ScalePoint(pp[3*i],pp[3*i+1],pp[3*i+2]);
 	}
-	curv_plot(31,pp,tt);
-	for(i=0;i<31;i++)
+	curv_plot(GridPnts+1,pp,tt);
+	for(i=0;i<GridPnts+1;i++)
 	{
 		pp[3*i+2]=z0;	pp[3*i+1]=t;
-		pp[3*i]=Min.x+(Max.x-(TernAxis ? t : Min.x))*i/30.;
+		pp[3*i]=Min.x+(Max.x-(TernAxis ? t : Min.x))*i/GridPnts;
 		tt[i] = ScalePoint(pp[3*i],pp[3*i+1],pp[3*i+2]);
 	}
-	curv_plot(31,pp,tt);
+	curv_plot(GridPnts+1,pp,tt);
+	delete [] tt;	delete [] pp;
 }
 //-----------------------------------------------------------------------------
 void mglGraph::DrawZGridLine(mreal t, mreal x0, mreal y0)
 {
 	register int i;
-	mreal pp[3*31];
-	bool tt[31];
-	for(i=0;i<31;i++)
+	mreal *pp = new mreal[3*GridPnts+3];
+	bool *tt = new bool[GridPnts+1];
+	for(i=0;i<GridPnts+1;i++)
 	{
 		pp[3*i+1]=y0;	pp[3*i+2]=t;
-		pp[3*i]=Min.x+(Max.x-Min.x)*i/30;
+		pp[3*i]=Min.x+(Max.x-Min.x)*i/GridPnts;
 		tt[i] = ScalePoint(pp[3*i],pp[3*i+1],pp[3*i+2]);
 	}
-	curv_plot(31,pp,tt);
-	for(i=0;i<31;i++)
+	curv_plot(GridPnts+1,pp,tt);
+	for(i=0;i<GridPnts+1;i++)
 	{
 		pp[3*i]=x0;	pp[3*i+2]=t;
-		pp[3*i+1]=Min.y+(Max.y-Min.y)*i/30.;
+		pp[3*i+1]=Min.y+(Max.y-Min.y)*i/GridPnts;
 		tt[i] = ScalePoint(pp[3*i],pp[3*i+1],pp[3*i+2]);
 	}
-	curv_plot(31,pp,tt);
+	curv_plot(GridPnts+1,pp,tt);
+	delete [] tt;	delete [] pp;
 }
 //-----------------------------------------------------------------------------
 void mglGraph::DrawTGridLine(mreal t, mreal z0)
 {
 	if(!TernAxis)	return;
 	register int i;
-	mreal pp[3*31];
-	bool tt[31];
-	for(i=0;i<31;i++)
+	mreal *pp = new mreal[3*GridPnts+3];
+	bool *tt = new bool[GridPnts+1];
+	for(i=0;i<GridPnts+1;i++)
 	{
-		pp[3*i]= t*i/30.;
-		pp[3*i+1]=t-t*i/30.;
+		pp[3*i]= t*i/GridPnts;
+		pp[3*i+1]=t-t*i/GridPnts;
 		pp[3*i+2]=z0;
 		tt[i] = ScalePoint(pp[3*i],pp[3*i+1],pp[3*i+2]);
 	}
-	curv_plot(31,pp,tt);
-	for(i=0;i<31;i++)
+	curv_plot(GridPnts+1,pp,tt);
+	for(i=0;i<GridPnts+1;i++)
 	{
-		pp[3*i]=t-t*i/30.;
-		pp[3*i+1]=t*i/30.;
+		pp[3*i]=t-t*i/GridPnts;
+		pp[3*i+1]=t*i/GridPnts;
 		pp[3*i+2]=z0;
 	}
-	curv_plot(31,pp,tt);
+	curv_plot(GridPnts+1,pp,tt);
+	delete [] tt;	delete [] pp;
 }
 //-----------------------------------------------------------------------------
 void mglGraph::Grid(const char *dir, const char *pen)
@@ -168,47 +164,68 @@ void mglGraph::Grid(const char *dir, const char *pen)
 	mreal x0,y0,z0,ddx,ddy,ddz,t;
 	if(pen)	SelectPen(pen);
 	else	SelectPen(TranspType!=2 ? "B;1" : "b;1");
-	ddx = dx>=0 ? dx : mgl_okrugl((Min.x-Max.x)/(dx+1),3);
-	ddy = dy>=0 ? dy : mgl_okrugl((Min.y-Max.y)/(dy+1),3);
-	ddz = dz>=0 ? dz : mgl_okrugl((Min.z-Max.z)/(dz+1),3);
+	ddx = dx>=0 ? dx : mgl_okrugl(-fabs(Min.x-Max.x)/(dx+1),3);
+	ddy = dy>=0 ? dy : mgl_okrugl(-fabs(Min.y-Max.y)/(dy+1),3);
+	ddz = dz>=0 ? dz : mgl_okrugl(-fabs(Min.z-Max.z)/(dz+1),3);
 	if(strchr(dir,'x'))
 	{
 		x0 = GetOrgX('x');	y0 = GetOrgY('x');	z0 = GetOrgZ('x');
 		x0 = isnan(OrgT.x) ? x0 : OrgT.x;
-		x0 = x0 - ddx*floor((x0-Min.x)/ddx);
-		if(xnum)	for(int i=0;i<xnum;i++)		DrawXGridLine(xval[i],y0,z0);
-		else if(dx)	for(t=x0;t<=Max.x;t+=ddx)	DrawXGridLine(t,y0,z0);
+		x0 = x0 - ddx*floor((x0-(Max.x>Min.x?Min.x:Max.x))/ddx);
+		if(xnum)	for(int i=0;i<xnum;i++)
+			DrawXGridLine(xval[i],y0,z0);
+		else if(dx)	for(t=x0;t<=(Min.x<Max.x?Max.x:Min.x);t+=ddx)
+			DrawXGridLine(t,y0,z0);
 		else if(Min.x>0)
 		{
 			x0 = exp(M_LN10*floor(0.1+log10(Min.x)));
 			for(t=x0;t<=Max.x*FLT_EPS;t*=10)	DrawXGridLine(t,y0,z0);
 		}
+		else if(Max.x<0)
+		{
+			x0 = -exp(M_LN10*floor(0.1+log10(-Min.x)));
+			for(t=x0;t>=Min.x*FLT_EPS;t*=10)	DrawXGridLine(t,y0,z0);
+		}
 	}
 	if(strchr(dir,'y'))
 	{
 		x0 = GetOrgX('y');	y0 = GetOrgY('y');	z0 = GetOrgZ('y');
 		y0 = isnan(OrgT.y) ? y0 : OrgT.y;
-		y0 = y0 - ddy*floor((y0-Min.y)/ddy);
-		if(ynum)	for(int i=0;i<ynum;i++)		DrawYGridLine(yval[i],x0,z0);
-		else if(dy)	for(t=y0;t<=Max.y;t+=ddy)	DrawYGridLine(t,x0,z0);
+		y0 = y0 - ddy*floor((y0-(Max.y>Min.y?Min.y:Max.y))/ddy);
+		if(ynum)	for(int i=0;i<ynum;i++)
+			DrawYGridLine(yval[i],x0,z0);
+		else if(dy)	for(t=y0;t<=(Max.y>Min.y?Max.y:Min.y);t+=ddy)
+			DrawYGridLine(t,x0,z0);
 		else if(Min.y>0)
 		{
 			y0 = exp(M_LN10*floor(0.1+log10(Min.y)));
 			for(t=y0;t<=Max.y*FLT_EPS;t*=10)	DrawYGridLine(t,x0,z0);
 		}
+		else if(Max.y<0)
+		{
+			y0 = -exp(M_LN10*floor(0.1+log10(-Max.y)));
+			for(t=y0;t>=Min.y*FLT_EPS;t*=10)	DrawYGridLine(t,x0,z0);
+		}
 	}
 	if(strchr(dir,'z'))
 	{
 		x0 = GetOrgX('z');	y0 = GetOrgY('z');	z0 = GetOrgZ('z');
 		z0 = isnan(OrgT.z) ? z0 : OrgT.z;
-		z0 = z0 - ddz*floor((z0-Min.z)/ddz);
-		if(znum)	for(int i=0;i<znum;i++)		DrawZGridLine(zval[i],x0,y0);
-		else if(dz)	for(t=z0;t<=Max.z;t+=ddz)	DrawZGridLine(t,x0,y0);
+		z0 = z0 - ddz*floor((z0-(Max.z>Min.z?Min.z:Max.z))/ddz);
+		if(znum)	for(int i=0;i<znum;i++)
+			DrawZGridLine(zval[i],x0,y0);
+		else if(dz)	for(t=z0;t<=(Max.z>Min.z?Max.z:Min.z);t+=ddz)
+			DrawZGridLine(t,x0,y0);
 		else if(Min.z>0)
 		{
 			z0 = exp(M_LN10*floor(0.1+log10(Min.z)));
 			for(t=z0;t<=Max.z*FLT_EPS;t*=10)	DrawZGridLine(t,x0,y0);
 		}
+		else if(Max.z<0)
+		{
+			z0 = -exp(M_LN10*floor(0.1+log10(-Max.z)));
+			for(t=z0;t>=Min.z*FLT_EPS;t*=10)	DrawZGridLine(t,x0,y0);
+		}
 	}
 	if(strchr(dir,'t'))
 	{
@@ -222,7 +239,7 @@ void mglGraph::Grid(const char *dir, const char *pen)
 	EndGroup();
 }
 //-----------------------------------------------------------------------------
-void mglGraph::Label(char dir,const wchar_t *text,int pos,mreal size,mreal shift)
+void mglGraph::Labelw(char dir,const wchar_t *text,mreal pos,mreal size,mreal shift)
 {
 	mreal t, x0, y0, z0;
 	SelectPen("k-1");
@@ -230,30 +247,34 @@ void mglGraph::Label(char dir,const wchar_t *text,int pos,mreal size,mreal shift
 	if(size<=0)	size = -size*FontSize;
 	if(dir=='x')
 	{
-		t = pos>0 ? Max.x : (pos<0 ? Min.x : (dx ? (Min.x+Max.x)/2 : sqrt(Min.x*Max.x)));
+		if(dx)	t = (Min.x+Max.x+pos*(Max.x-Min.x))/2;
+		else	t = Min.x*pow(Max.x/Min.x, (pos+1)/2);
 		Putsw(mglPoint(t,y0,z0),text,FontDef,size,'X',shift);
 	}
 	if(dir=='y')
 	{
 		if(TernAxis)
 		{
-			t = pos>0 ? 1 : (pos<0 ? 0.: (dz ? 0.5 : 0.));
+			t = (pos+1)/2;
 			Putsw(mglPoint(Max.x-t,t,z0),text,FontDef,size,'T',shift);
 		}
 		else
 		{
-			t = pos>0 ? Max.y : (pos<0 ? Min.y : (dy ? (Min.y+Max.y)/2 : sqrt(Min.y*Max.y)));
+			if(dx)	t = (Min.y+Max.y+pos*(Max.y-Min.y))/2;
+			else	t = Min.y*pow(Max.y/Min.y, (pos+1)/2);
 			Putsw(mglPoint(x0,t,z0),text,FontDef,size,'Y',shift);
 		}
 	}
 	if(dir=='z')
 	{
-		t = pos>0 ? Max.z : (pos<0 ? Min.z : (dz ? (Min.z+Max.z)/2 : sqrt(Min.z*Max.z)));
+		if(dx)	t = (Min.z+Max.z+pos*(Max.z-Min.z))/2;
+		else	t = Min.z*pow(Max.z/Min.z, (pos+1)/2);
 		Putsw(mglPoint(x0,y0,t),text,FontDef,size,'Z',shift);
 	}
 	if(dir=='t' && TernAxis)
 	{
-		t = pos<0 ? Max.y : (pos>0 ? Min.y : (dy ? (Min.y+Max.y)/2 : sqrt(Min.y*Max.y)));
+		if(dx)	t = (Min.x+Max.x-pos*(Max.x-Min.x))/2;
+		else	t = Max.x*pow(Min.x/Max.x, (pos+1)/2);
 		Putsw(mglPoint(x0,t,z0),text,FontDef,size,'Y',shift);
 	}
 }
@@ -320,6 +341,14 @@ void _mgl_tick_text(mreal z, mreal z0, mreal d, mreal v, int kind, wchar_t str[6
 	}
 }
 //-----------------------------------------------------------------------------
+void mglGraph::DrawTick(mreal *pp,bool /*sub*/)
+{
+	ScalePoint(pp[0],pp[1],pp[2]);
+	ScalePoint(pp[3],pp[4],pp[5]);
+	ScalePoint(pp[6],pp[7],pp[8]);
+	curv_plot(3,pp,0);
+}
+//-----------------------------------------------------------------------------
 void mglGraph::DrawXTick(mreal x, mreal y0, mreal z0, mreal dy, mreal dz, int f)
 {
 	mreal pp[9],ff=TickLen/sqrt(1.+f*st_t);
@@ -328,7 +357,7 @@ void mglGraph::DrawXTick(mreal x, mreal y0, mreal z0, mreal dy, mreal dz, int f)
 	pp[6]=x;	pp[7]=y0;	pp[8]=z0+dz*ff;
 	if(*TickStl && !f)	SelectPen(TickStl);
 	if(*SubTStl && f)	SelectPen(SubTStl);
-	DrawTick(pp,false);
+	DrawTick(pp,f!=0);
 }
 void mglGraph::AxisX(bool text)
 {
@@ -338,18 +367,15 @@ void mglGraph::AxisX(bool text)
 	long i;
 	mreal x0,y0,z0,x,ddx,ddy,ddz,v=0;
 	x0 = GetOrgX('x');	y0 = GetOrgY('x');	z0 = GetOrgZ('x');
-	ddx = dx>=0 ? dx : mgl_okrugl((Min.x-Max.x)/(dx+1),3);
-	ddy = dy>=0 ? dy : mgl_okrugl((Min.y-Max.y)/(dy+1),3);
-	ddz = dz>=0 ? dz : mgl_okrugl((Min.z-Max.z)/(dz+1),3);
-//	if(ddx>0.1*fabs(Max.x-Min.x)/TickLen)	ddx = 0.1*(Max.x-Min.x)/TickLen;
-//	if(ddy>0.1*fabs(Max.y-Min.y)/TickLen)	ddy = 0.1*(Max.y-Min.y)/TickLen;
-//	if(ddz>0.1*fabs(Max.z-Min.z)/TickLen)	ddz = 0.1*(Max.z-Min.z)/TickLen;
+	ddx = dx>=0 ? dx : mgl_okrugl(-fabs(Min.x-Max.x)/(dx+1),3);
+	ddy = dy>=0 ? dy : mgl_okrugl(-fabs(Min.y-Max.y)/(dy+1),3);
+	ddz = dz>=0 ? dz : mgl_okrugl(-fabs(Min.z-Max.z)/(dz+1),3);
 	if(ddy==0)	ddy=y0*10>fabs(Max.y-Min.y) ? 0.3*fabs(Max.y-Min.y) : y0*3;
 	if(ddz==0)	ddz=z0*10>fabs(Max.z-Min.z) ? 0.3*fabs(Max.z-Min.z) : z0*3;
 	if(y0>(Max.y+Min.y)/2)	ddy = -ddy;
 	if(z0>(Max.z+Min.z)/2)	ddz = -ddz;
 	SelectPen(TranspType!=2 ? "k-1":"w-1");
-	for(i=0;i<31;i++)	//  
+	for(i=0;i<31;i++)	// draw axis line
 	{
 		pp[3*i+1]=y0;	pp[3*i+2]=z0;
 		pp[3*i]=Min.x+(Max.x-Min.x)*i/30.;
@@ -362,22 +388,30 @@ void mglGraph::AxisX(bool text)
 		DrawXTick(xval[i],y0,z0,ddy,ddz);
 		if(text)	Putsw(mglPoint(xval[i],y0,z0),xstr[i],FontDef,-1,'x');
 	}
-	else if(ddx>0)	// ticks drawing
+	else if(ddx)	// ticks drawing
 	{
 		int kind=0;
 		wchar_t s[32]=L"";
-		if(!xtt) kind = _mgl_tick_ext(Max.x, Min.x, s, v);
+		if(xtt[0]==0) kind = _mgl_tick_ext(Max.x, Min.x, s, v);
 		if(!TuneTicks)	kind = 0;
 
 		NSx = NSx<0 ? 0 : NSx;
 		x0 = isnan(OrgT.x) ? x0 : OrgT.x;
-		x0 = x0 - ddx*floor((x0-Min.x)/ddx+1e-3);
-		for(x=x0;x<=Max.x+(Max.x-Min.x)*1e-6;x+=ddx)
+		double x1 = Max.x+(Max.x-Min.x)*1e-6;
+		if(Max.x>Min.x)
+			x0 = x0 - ddx*floor((x0-Min.x)/ddx+1e-3);
+		else
+		{
+			x1 = Min.x-(Max.x-Min.x)*1e-6;
+			x0 = x0 - ddx*floor((x0-Max.x)/ddx+1e-3);
+		}
+		for(x=x0;x<=x1;x+=ddx)
+//		for(x=x0;fabs(x-Max.x)<fabs(Max.x-Min.x)*1.000001;x+=ddx)
 		{
 			DrawXTick(x,y0,z0,ddy,ddz);
 			if(text)
 			{
-				if(xtt)	mglprintf(str, 64, xtt, fabs(x)<ddx/100 ? 0 : x);
+				if(xtt[0])	mglprintf(str, 64, xtt, fabs(x)<ddx/100 ? 0 : x);
 				else	_mgl_tick_text(x,x0,ddx/100,v,kind,str);
 				wcstrim_mgl(str);
 				Putsw(mglPoint(x,y0,z0),str,FontDef,-1,'x');
@@ -389,8 +423,11 @@ void mglGraph::AxisX(bool text)
 		{
 			ddx /= (NSx+1);	// subticks drawing
 			x0 = GetOrgX('x');	x0 = isnan(OrgT.x) ? x0 : OrgT.x;
-			x0 = x0 - ddx*floor((x0-Min.x)/ddx+1e-3);
-			for(x=x0;x<=Max.x;x+=ddx)	DrawXTick(x,y0,z0,ddy,ddz,1);
+			if(Max.x>Min.x)
+			{	x1=Max.x;	x0 = x0 - ddx*floor((x0-Min.x)/ddx+1e-3);	}
+			else
+			{	x1=Min.x;	x0 = x0 - ddx*floor((x0-Max.x)/ddx+1e-3);	}
+			for(x=x0;x<=x1;x+=ddx)	DrawXTick(x,y0,z0,ddy,ddz,1);
 		}
 	}
 	else if(Min.x>0)
@@ -403,13 +440,28 @@ void mglGraph::AxisX(bool text)
 			{
 				DrawXTick(x,y0,z0,ddy,ddz);
 				mglprintf(str,64,L"10^{%d}",int(floor(0.1+log10(x))));
-				if(text && cur%k==0)
-					Putsw(mglPoint(x,y0,z0),str,FontDef,-1,'x');
+				if(text && cur%k==0)	Putsw(mglPoint(x,y0,z0),str,FontDef,-1,'x');
 				cur++;
 			}
 			// subticks drawing
-			for(v=x;v<10*x;v+=x)	if(v>Min.x && v<Max.x)
-				DrawXTick(v,y0,z0,ddy,ddz,1);
+			for(v=x;v<10*x;v+=x)	if(v>Min.x && v<Max.x)	DrawXTick(v,y0,z0,ddy,ddz,1);
+		}
+	}
+	else if(Max.x<0)
+	{
+		int k=1+int(log10(Min.x/Max.x)/5), cur=0;
+		x0 = -exp(M_LN10*floor(0.1+log10(-Max.x)));
+		for(x=x0;x>=Min.x*FLT_EPS;x*=10)
+		{
+			if(x*FLT_EPS<=Max.x && x>=Min.x*FLT_EPS)
+			{
+				DrawXTick(x,y0,z0,ddy,ddz);
+				mglprintf(str,64,L"-10^{%d}",int(floor(0.1+log10(-x))));
+				if(text && cur%k==0)	Putsw(mglPoint(x,y0,z0),str,FontDef,-1,'x');
+				cur++;
+			}
+			// subticks drawing
+			for(v=x;v>10*x;v+=x)	if(v<Max.x && v>Min.x)	DrawXTick(v,y0,z0,ddy,ddz,1);
 		}
 	}
 }
@@ -422,7 +474,7 @@ void mglGraph::DrawYTick(mreal y, mreal x0, mreal z0, mreal dx, mreal dz, int f)
 	pp[6]=x0;		pp[7]=y;	pp[8]=z0+dz*ff;
 	if(*TickStl && !f)	SelectPen(TickStl);
 	if(*SubTStl && f)	SelectPen(SubTStl);
-	DrawTick(pp,false);
+	DrawTick(pp,f!=0);
 }
 void mglGraph::AxisY(bool text)
 {
@@ -432,12 +484,9 @@ void mglGraph::AxisY(bool text)
 	long i;
 	mreal x0,y0,z0,y,ddx,ddy,ddz,v=0;
 	x0 = GetOrgX('y');	y0 = GetOrgY('y');	z0 = GetOrgZ('y');
-	ddx = dx>=0 ? dx : mgl_okrugl((Min.x-Max.x)/(dx+1),3);
-	ddy = dy>=0 ? dy : mgl_okrugl((Min.y-Max.y)/(dy+1),3);
-	ddz = dz>=0 ? dz : mgl_okrugl((Min.z-Max.z)/(dz+1),3);
-//	if(ddx>0.1*fabs(Max.x-Min.x)/TickLen)	ddx = 0.1*(Max.x-Min.x)/TickLen;
-//	if(ddy>0.1*fabs(Max.y-Min.y)/TickLen)	ddy = 0.1*(Max.y-Min.y)/TickLen;
-//	if(ddz>0.1*fabs(Max.z-Min.z)/TickLen)	ddz = 0.1*(Max.z-Min.z)/TickLen;
+	ddx = dx>=0 ? dx : mgl_okrugl(-fabs(Min.x-Max.x)/(dx+1),3);
+	ddy = dy>=0 ? dy : mgl_okrugl(-fabs(Min.y-Max.y)/(dy+1),3);
+	ddz = dz>=0 ? dz : mgl_okrugl(-fabs(Min.z-Max.z)/(dz+1),3);
 	if(ddx==0)	ddx=x0*10>fabs(Max.x-Min.x) ? 0.3*fabs(Max.x-Min.x) : x0*3;
 	if(ddz==0)	ddz=z0*10>fabs(Max.z-Min.z) ? 0.3*fabs(Max.z-Min.z) : z0*3;
 	if(x0>(Max.x+Min.x)/2)	ddx = -ddx;
@@ -456,22 +505,29 @@ void mglGraph::AxisY(bool text)
 		DrawYTick(yval[i],x0,z0,ddx,ddz);
 		if(text)	Putsw(mglPoint(x0,yval[i],z0),ystr[i],FontDef,-1,'y');
 	}
-	else if(ddy>0)	// ticks drawing
+	else if(ddy)	// ticks drawing
 	{
 		int kind=0;
 		wchar_t s[32]=L"";
-		if(!ytt) kind = _mgl_tick_ext(Max.y, Min.y, s, v);
+		if(ytt[0]==0) kind = _mgl_tick_ext(Max.y, Min.y, s, v);
 		if(!TuneTicks)	kind = 0;
 
 		NSy = NSy<0 ? 0 : NSy;
 		y0 = isnan(OrgT.y) ? y0 : OrgT.y;
-		y0 = y0 - ddy*floor((y0-Min.y)/ddy+1e-3);
-		for(y=y0;y<=Max.y+(Max.y-Min.y)*1e-6;y+=ddy)
+		double y1 = Max.y+(Max.y-Min.y)*1e-6;
+		if(Max.y>Min.y)
+			y0 = y0 - ddy*floor((y0-Min.y)/ddy+1e-3);
+		else
+		{
+			y1 = Min.y-(Max.y-Min.y)*1e-6;
+			y0 = y0 - ddy*floor((y0-Max.y)/ddy+1e-3);
+		}
+		for(y=y0;y<=y1;y+=ddy)
 		{
 			DrawYTick(y,x0,z0,ddx,ddz);
 			if(text)
 			{
-				if(ytt)	mglprintf(str, 64, ytt, fabs(y)<ddy/100 ? 0 : y);
+				if(ytt[0])	mglprintf(str, 64, ytt, fabs(y)<ddy/100 ? 0 : y);
 				else	_mgl_tick_text(y,y0,ddy/100,v,kind,str);
 				wcstrim_mgl(str);
 				Putsw(mglPoint(x0,y,z0),str,FontDef,-1,'y');
@@ -483,8 +539,11 @@ void mglGraph::AxisY(bool text)
 		{
 			ddy /= (NSy+1);	// subticks drawing
 			y0 = GetOrgY('y');	y0 = isnan(OrgT.y) ? y0 : OrgT.y;
-			y0 = y0 - ddy*floor((y0-Min.y)/ddy+1e-3);
-			for(y=y0;y<=Max.y;y+=ddy)	DrawYTick(y,x0,z0,ddx,ddz,1);
+			if(Max.y>Min.y)
+			{	y1=Max.y;	y0 = y0 - ddy*floor((y0-Min.y)/ddy+1e-3);	}
+			else
+			{	y1=Min.y;	y0 = y0 - ddy*floor((y0-Max.y)/ddy+1e-3);	}
+			for(y=y0;y<=y1;y+=ddy)	DrawYTick(y,x0,z0,ddx,ddz,1);
 		}
 	}
 	else if(Min.y>0)
@@ -497,13 +556,28 @@ void mglGraph::AxisY(bool text)
 			{
 				DrawYTick(y,x0,z0,ddx,ddz);
 				mglprintf(str,64,L"10^{%d}",int(floor(0.1+log10(y))));
-				if(text && cur%k==0)
-					Putsw(mglPoint(x0,y,z0),str,FontDef,-1,'y');
+				if(text && cur%k==0)	Putsw(mglPoint(x0,y,z0),str,FontDef,-1,'y');
 				cur++;
 			}
 			// subticks drawing
-			for(v=y;v<10*y;v+=y)	if(v>Min.y && v<Max.y)
-				DrawYTick(v,x0,z0,ddx,ddz,1);
+			for(v=y;v<10*y;v+=y)	if(v>Min.y && v<Max.y)	DrawYTick(v,x0,z0,ddx,ddz,1);
+		}
+	}
+	else if(Max.y<0)
+	{
+		int k=1+int(log10(Min.y/Max.y)/5), cur=0;
+		y0 = -exp(M_LN10*floor(0.1+log10(-Max.y)));
+		for(y=y0;y>=Min.y*FLT_EPS;y*=10)
+		{
+			if(y*FLT_EPS<=Max.y && y>=Min.y*FLT_EPS)
+			{
+				DrawYTick(y,x0,z0,ddx,ddz);
+				mglprintf(str,64,L"-10^{%d}",int(floor(0.1+log10(-y))));
+				if(text && cur%k==0)	Putsw(mglPoint(x0,y,z0),str,FontDef,-1,'y');
+				cur++;
+			}
+			// subticks drawing
+			for(v=y;v>10*y;v+=y)	if(v<Max.y && v>Min.y)	DrawYTick(v,x0,z0,ddx,ddz,1);
 		}
 	}
 }
@@ -516,7 +590,7 @@ void mglGraph::DrawZTick(mreal z, mreal x0, mreal y0, mreal dx, mreal dy, int f)
 	pp[6]=x0+dx*ff;	pp[7]=y0;	pp[8]=z;
 	if(*TickStl && !f)	SelectPen(TickStl);
 	if(*SubTStl && f)	SelectPen(SubTStl);
-	DrawTick(pp,false);
+	DrawTick(pp,f!=0);
 }
 void mglGraph::AxisZ(bool text)
 {
@@ -526,12 +600,9 @@ void mglGraph::AxisZ(bool text)
 	long i;
 	mreal x0,y0,z0,z,ddx,ddy,ddz,v=0;
 	x0 = GetOrgX('z');	y0 = GetOrgY('z');	z0 = GetOrgZ('z');
-	ddx = dx>=0 ? dx : mgl_okrugl((Min.x-Max.x)/(dx+1),3);
-	ddy = dy>=0 ? dy : mgl_okrugl((Min.y-Max.y)/(dy+1),3);
-	ddz = dz>=0 ? dz : mgl_okrugl((Min.z-Max.z)/(dz+1),3);
-//	if(ddx>0.1*fabs(Max.x-Min.x)/TickLen)	ddx = 0.1*(Max.x-Min.x)/TickLen;
-//	if(ddy>0.1*fabs(Max.y-Min.y)/TickLen)	ddy = 0.1*(Max.y-Min.y)/TickLen;
-//	if(ddz>0.1*fabs(Max.z-Min.z)/TickLen)	ddz = 0.1*(Max.z-Min.z)/TickLen;
+	ddx = dx>=0 ? dx : mgl_okrugl(-fabs(Min.x-Max.x)/(dx+1),3);
+	ddy = dy>=0 ? dy : mgl_okrugl(-fabs(Min.y-Max.y)/(dy+1),3);
+	ddz = dz>=0 ? dz : mgl_okrugl(-fabs(Min.z-Max.z)/(dz+1),3);
 	if(ddx==0)	ddx=x0*10>fabs(Max.x-Min.x) ? 0.3*fabs(Max.x-Min.x) : x0*3;
 	if(ddy==0)	ddy=y0*10>fabs(Max.y-Min.y) ? 0.3*fabs(Max.y-Min.y) : y0*3;
 	if(x0>(Max.x+Min.x)/2)	ddx = -ddx;
@@ -550,22 +621,28 @@ void mglGraph::AxisZ(bool text)
 		DrawZTick(zval[i],x0,y0,ddx,ddy);
 		if(text)	Putsw(mglPoint(x0,y0,zval[i]),zstr[i],FontDef,-1,'z');
 	}
-	else if(ddz>0)	// ticks drawing
+	else if(ddz)	// ticks drawing
 	{
 		int kind=0;
 		wchar_t s[32]=L"";
-		if(!ytt) kind = _mgl_tick_ext(Max.z, Min.z, s, v);
+		if(ztt[0]==0) kind = _mgl_tick_ext(Max.z, Min.z, s, v);
 		if(!TuneTicks)	kind = 0;
 
 		NSz = NSz<0 ? 0 : NSz;
 		z0 = isnan(OrgT.z) ? z0 : OrgT.z;
-		z0 = z0 - ddz*floor((z0-Min.z)/ddz+1e-3);
-		for(z=z0;z<=Max.z+(Max.z-Min.z)*1e-6;z+=ddz)
+		double z1 = Max.z+(Max.z-Min.z)*1e-6;
+		if(Max.z>Min.z)		z0 = z0 - ddz*floor((z0-Min.z)/ddz+1e-3);
+		else
+		{
+			z1 = Min.z-(Max.z-Min.z)*1e-6;
+			z0 = z0 - ddz*floor((z0-Max.z)/ddz+1e-3);
+		}
+		for(z=z0;z<=z1;z+=ddz)
 		{
 			DrawZTick(z,x0,y0,ddx,ddy);
 			if(text)
 			{
-				if(ztt)	mglprintf(str,64,ztt,fabs(z)<ddz/100 ? 0 : z);
+				if(ztt[0])	mglprintf(str,64,ztt,fabs(z)<ddz/100 ? 0 : z);
 				else	_mgl_tick_text(z,z0,ddz/100,v,kind,str);
 				wcstrim_mgl(str);
 				Putsw(mglPoint(x0,y0,z),str,FontDef,-1,'z');
@@ -577,8 +654,11 @@ void mglGraph::AxisZ(bool text)
 		{
 			ddz /= (NSz+1);	// subticks drawing
 			z0 = GetOrgZ('z');	z0 = isnan(OrgT.z) ? z0 : OrgT.z;
-			z0 = z0 - ddz*floor((z0-Min.z)/ddz+1e-3);
-			for(z=z0;z<=Max.z;z+=ddz)	DrawZTick(z,x0,y0,ddx,ddy,1);
+			if(Max.z>Min.z)
+			{	z1=Max.z;	z0 = z0 - ddz*floor((z0-Min.z)/ddz+1e-3);	}
+			else
+			{	z1=Min.z;	z0 = z0 - ddz*floor((z0-Max.z)/ddz+1e-3);	}
+			for(z=z0;z<=z1;z+=ddz)	DrawZTick(z,x0,y0,ddx,ddy,1);
 		}
 	}
 	else if(Min.z>0)
@@ -591,14 +671,28 @@ void mglGraph::AxisZ(bool text)
 			{
 				DrawZTick(z,x0,y0,ddx,ddy);
 				mglprintf(str,64,L"10^{%d}",int(floor(0.1+log10(z))));
-				if(text && cur%k==0)
-					Putsw(mglPoint(x0,y0,z),str,FontDef,-1,'z');
+				if(text && cur%k==0)	Putsw(mglPoint(x0,y0,z),str,FontDef,-1,'z');
 				cur++;
 			}
 			// subticks drawing
-			for(v=z;v<10*z;v+=z)	if(v>Min.z && v<Max.z)
-				DrawZTick(v,x0,y0,ddx,ddy,1);
-
+			for(v=z;v<10*z;v+=z)	if(v>Min.z && v<Max.z)	DrawZTick(v,x0,y0,ddx,ddy,1);
+		}
+	}
+	else if(Max.z<0)
+	{
+		int k=1+int(log10(Min.z/Max.z)/5), cur=0;
+		z0 = -exp(M_LN10*floor(0.1+log10(-Max.z)));
+		for(z=z0;z>=Min.z*FLT_EPS;z*=10)
+		{
+			if(z*FLT_EPS<=Max.z && z>=Min.z*FLT_EPS)
+			{
+				DrawZTick(z,x0,y0,ddx,ddy);
+				mglprintf(str,64,L"-10^{%d}",int(floor(0.1+log10(-z))));
+				if(text && cur%k==0)	Putsw(mglPoint(x0,y0,z),str,FontDef,-1,'z');
+				cur++;
+			}
+			// subticks drawing
+			for(v=z;v>10*z;v+=z)	if(v<Max.z && v>Min.z)	DrawZTick(v,x0,y0,ddx,ddy,1);
 		}
 	}
 }
@@ -630,10 +724,16 @@ void mglGraph::Axis(const char *dir, bool adjust)
 void mglGraph::TickBox()
 {
 	mreal x0,y0,z0,x,y,z,ddx,ddy,ddz;
+	bool d_x,d_y,d_z;
 	x0 = GetOrgX(0);	y0 = GetOrgY(0);	z0 = GetOrgZ(0);
-	ddx = dx>=0 ? dx : mgl_okrugl((Min.x-Max.x)/(dx+1),3);
-	ddy = dy>=0 ? dy : mgl_okrugl((Min.y-Max.y)/(dy+1),3);
-	ddz = dz>=0 ? dz : mgl_okrugl((Min.z-Max.z)/(dz+1),3);
+	ddx = dx>=0 ? dx : mgl_okrugl(-fabs(Min.x-Max.x)/(dx+1),3);
+	ddy = dy>=0 ? dy : mgl_okrugl(-fabs(Min.y-Max.y)/(dy+1),3);
+	ddz = dz>=0 ? dz : mgl_okrugl(-fabs(Min.z-Max.z)/(dz+1),3);
+	d_x=(ddx!=0);	d_y=(ddy!=0);	d_z=(ddz!=0);
+	if(!d_x)	ddx=x0*10>fabs(Max.x-Min.x) ? 0.3*fabs(Max.x-Min.x) : x0*3;
+	if(!d_y)	ddy=y0*10>fabs(Max.y-Min.y) ? 0.3*fabs(Max.y-Min.y) : y0*3;
+	if(!d_z)	ddz=z0*10>fabs(Max.z-Min.z) ? 0.3*fabs(Max.z-Min.z) : z0*3;
+
 	if(xnum)	for(int i=0;i<xnum;i++)
 	{
 		x = xval[i];
@@ -642,11 +742,11 @@ void mglGraph::TickBox()
 		DrawXTick(x,Max.y,Min.z,-ddy,ddz);
 		DrawXTick(x,Max.y,Max.z,-ddy,-ddz);
 	}
-	else if(ddx>0)	//    
+	else if(d_x)	//    
 	{
 		x0 = isnan(OrgT.x) ? x0 : OrgT.x;
-		x0 = x0 - ddx*floor((x0-Min.x)/ddx);
-		for(x=x0;x<Max.x;x+=ddx)
+		x0 = x0 - ddx*floor((x0-(Max.x>Min.x?Min.x:Max.x))/ddx);
+		for(x=x0;x<(Max.x>Min.x?Max.x:Min.x);x+=ddx)
 		{
 			DrawXTick(x,Min.y,Min.z,ddy,ddz);
 			DrawXTick(x,Min.y,Max.z,ddy,-ddz);
@@ -666,6 +766,18 @@ void mglGraph::TickBox()
 			DrawXTick(x,Max.y,Max.z,-ddy,-ddz);
 		}
 	}
+	else if(Max.x<0)
+	{
+		x0 = -exp(M_LN10*floor(0.1+log10(-Max.x)));
+		for(x=x0;x>Min.x;x*=10)
+		{
+			if(x>Max.x)	continue;
+			DrawXTick(x,Min.y,Min.z,ddy,ddz);
+			DrawXTick(x,Min.y,Max.z,ddy,-ddz);
+			DrawXTick(x,Max.y,Min.z,-ddy,ddz);
+			DrawXTick(x,Max.y,Max.z,-ddy,-ddz);
+		}
+	}
 	x0 = GetOrgX(0);
 	if(ynum)	for(int i=0;i<ynum;i++)
 	{
@@ -675,11 +787,11 @@ void mglGraph::TickBox()
 		DrawYTick(y,Max.x,Min.z,-ddx,ddz);
 		DrawYTick(y,Max.x,Max.z,-ddx,-ddz);
 	}
-	else if(ddy>0)	//    y
+	else if(d_y)	//    y
 	{
 		y0 = isnan(OrgT.y) ? y0 : OrgT.y;
-		y0 = y0 - ddy*floor((y0-Min.y)/ddy);
-		for(y=y0;y<Max.y;y+=ddy)
+		y0 = y0 - ddy*floor((y0-(Max.y>Min.y?Min.y:Max.y))/ddy);
+		for(y=y0;y<(Max.y>Min.y?Max.y:Min.y);y+=ddy)
 		{
 			DrawYTick(y,Min.x,Min.z,ddx,ddz);
 			DrawYTick(y,Min.x,Max.z,ddx,-ddz);
@@ -699,6 +811,18 @@ void mglGraph::TickBox()
 			DrawYTick(y,Max.x,Max.z,-ddx,-ddz);
 		}
 	}
+	else if(Max.y<0)
+	{
+		y0 = -exp(M_LN10*floor(0.1+log10(-Max.y)));
+		for(y=y0;y>Min.y;y*=10)
+		{
+			if(y>Max.y)	continue;
+			DrawYTick(y,Min.x,Min.z,ddx,ddz);
+			DrawYTick(y,Min.x,Max.z,ddx,-ddz);
+			DrawYTick(y,Max.x,Min.z,-ddx,ddz);
+			DrawYTick(y,Max.x,Max.z,-ddx,-ddz);
+		}
+	}
 	y0 = GetOrgY(0);
 	if(znum)	for(int i=0;i<znum;i++)
 	{
@@ -708,11 +832,11 @@ void mglGraph::TickBox()
 		DrawZTick(z,Max.x,Min.y,-ddx,ddy);
 		DrawZTick(z,Max.x,Max.y,-ddx,-ddy);
 	}
-	else if(ddz>0)	//  
+	else if(d_z)	//  
 	{
 		z0 = isnan(OrgT.z) ? z0 : OrgT.z;
-		z0 = z0 - ddz*floor((z0-Min.z)/ddz);
-		for(z=z0;z<Max.z;z+=ddz)
+		z0 = z0 - ddz*floor((z0-(Max.z>Min.z?Min.z:Max.z))/ddz);
+		for(z=z0;z<(Max.z>Min.z?Max.z:Min.z);z+=ddz)
 		{
 			DrawZTick(z,Min.x,Min.y,ddx,ddy);
 			DrawZTick(z,Min.x,Max.y,ddx,-ddy);
@@ -732,13 +856,24 @@ void mglGraph::TickBox()
 			DrawZTick(z,Max.x,Max.y,-ddx,-ddy);
 		}
 	}
+	else if(Max.z<0)
+	{
+		z0 = -exp(M_LN10*floor(0.1+log10(-Max.z)));
+		for(z=z0;z>Min.z;z*=10)
+		{
+			if(z>Max.z)	continue;
+			DrawZTick(z,Min.x,Min.y,ddx,ddy);
+			DrawZTick(z,Min.x,Max.y,ddx,-ddy);
+			DrawZTick(z,Max.x,Min.y,-ddx,ddy);
+			DrawZTick(z,Max.x,Max.y,-ddx,-ddy);
+		}
+	}
 }
 //-----------------------------------------------------------------------------
 void mglGraph::Box(const char *col,bool ticks)
 {
-	mglColor c;
-	if(!col || col[0]==0)	Box(NC,ticks);
-	c.Set(col[0]);
+	mglColor c(NC);
+	if(col && col[0])	c.Set(col[0]);
 	Box(c,ticks);
 }
 //-----------------------------------------------------------------------------
@@ -848,7 +983,7 @@ void mglGraph::DrawTTick(mreal y, mreal x0, mreal z0, mreal dx, mreal dz, int f)
 	pp[6]=x0-y;			pp[7]=y;	pp[8]=z0+dz*ff;
 	if(*TickStl && !f)	SelectPen(TickStl);
 	if(*SubTStl && f)	SelectPen(SubTStl);
-	DrawTick(pp,false);
+	DrawTick(pp,f!=0);
 }
 void mglGraph::AxisT(bool text)
 {
@@ -912,6 +1047,7 @@ void mglGraph::SetTicks(char dir, mreal d, int ns, mreal org)
 	{	dy = d;	NSy = ns;	OrgT.y = org;	if(ynum) delete []ybuf;	ynum=0;	}
 	else if(dir=='z')
 	{	dz = d;	NSz = ns;	OrgT.z = org;	if(znum) delete []zbuf;	znum=0;	}
+	else if(dir=='c' || dir=='a')	{	dc = d;	OrgC = org;	}
 }
 //-----------------------------------------------------------------------------
 void mglGraph::SetTicksVal(char dir, int n, mreal *val, const wchar_t **lbl)
@@ -1051,11 +1187,18 @@ void mglGraph::adjust(char dir, mreal d)
 	else if(d<7)SetTicks(dir,pow(10,n),4,0);
 	else		SetTicks(dir,2*pow(10,n),3,0);
 }
+bool mgl_islog(mreal a, mreal b)
+{	return (a>0 && b>10*a) || (b<0 && a<10*b);	}
 void mglGraph::AdjustTicks(const char *dir)
 {
-	if(strchr(dir,'x') || strchr(dir,'X'))	adjust('x',Max.x-Min.x);
-	if(strchr(dir,'y') || strchr(dir,'Y'))	adjust('y',Max.y-Min.y);
-	if(strchr(dir,'z') || strchr(dir,'Z'))	adjust('z',Max.z-Min.z);
+	if(strchr(dir,'x') || strchr(dir,'X'))
+	{	if(fx && mgl_islog(Min.x,Max.x))	SetTicks ('x',0);	else	adjust('x',Max.x-Min.x);	}
+	if(strchr(dir,'y') || strchr(dir,'Y'))
+	{	if(fy && mgl_islog(Min.y,Max.y))	SetTicks ('y',0);	else	adjust('y',Max.y-Min.y);	}
+	if(strchr(dir,'z') || strchr(dir,'Z'))
+	{	if(fz && mgl_islog(Min.z,Max.z))	SetTicks ('z',0);	else	adjust('z',Max.z-Min.z);	}
+	if(strchr(dir,'c') || strchr(dir,'C') || strchr(dir,'a') || strchr(dir,'A'))
+	{	if(fa && mgl_islog(Cmin,Cmax))	SetTicks ('c',0);	else	adjust('c',Cmax-Cmin);	}
 	TuneTicks = true;
 }
 //-----------------------------------------------------------------------------
