@@ -3,25 +3,23 @@
  * Copyright (C) 2007 Alexey Balakin <balakin@appl.sci-nnov.ru>            *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
- *   it under the terms of the GNU General Public License as published by  *
- *   the Free Software Foundation; either version 2 of the License, or     *
- *   (at your option) any later version.                                   *
+ *   it under the terms of the GNU Library General Public License as       *
+ *   published by the Free Software Foundation; either version 3 of the    *
+ *   License, or (at your option) any later version.                       *
  *                                                                         *
  *   This program is distributed in the hope that it will be useful,       *
  *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
  *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
  *   GNU General Public License for more details.                          *
  *                                                                         *
- *   You should have received a copy of the GNU General Public License     *
- *   along with this program; if not, write to the                         *
+ *   You should have received a copy of the GNU Library General Public     *
+ *   License along with this program; if not, write to the                 *
  *   Free Software Foundation, Inc.,                                       *
  *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
  ***************************************************************************/
 #include <stdlib.h>
-#include "mgl/mgl_c.h"
-#include "mgl/mgl.h"
 #include "mgl/mgl_ab.h"
-#include "mgl/mgl_data.h"
+#include "mgl/mgl_c.h"
 //-----------------------------------------------------------------------------
 /// Delete mglGraph object. MUST be called for each mgl_create_* call.
 void mgl_delete_graph(HMGL gr)
@@ -160,8 +158,8 @@ void mgl_set_ambbr(HMGL gr, mreal i)
 //		Scale and rotate
 //-----------------------------------------------------------------------------
 /// Clear transformation matrix.
-void mgl_identity(HMGL gr)
-{	gr->Identity();	}
+void mgl_identity(HMGL gr, int rel)
+{	gr->Identity(rel);	}
 /// Clear up the frame
 void mgl_clf(HMGL gr)
 {	gr->Clf();	}
@@ -181,6 +179,8 @@ void mgl_relplot(HMGL gr, mreal x1,mreal x2,mreal y1,mreal y2)
 {	gr->InPlot(x1,x2,y1,y2,true);	}
 void mgl_columnplot(HMGL gr, int num, int i)
 {	gr->ColumnPlot(num,i);	}
+void mgl_stickplot(HMGL gr, int num, int i, mreal tet, mreal phi)
+{	gr->StickPlot(num, i, tet, phi);	}
 /// Set aspect ratio for further plotting.
 void mgl_aspect(HMGL gr, mreal Ax,mreal Ay,mreal Az)
 {	gr->Aspect(Ax,Ay,Az);	}
@@ -221,11 +221,12 @@ void mgl_set_axis(HMGL gr, mreal x1, mreal y1, mreal z1, mreal x2, mreal y2, mre
 {	gr->Axis(mglPoint(x1,y1,z1),mglPoint(x2,y2,z2),mglPoint(x0,y0,z0));	}
 /// Safetly set the transformation formulas for coordinate.
 void mgl_set_func(HMGL gr, const char *EqX,const char *EqY,const char *EqZ)
-{	gr->Axis(EqX,EqY,EqZ);	}
+{	gr->SetFunc(EqX,EqY,EqZ);	}
+void mgl_set_func_ext(HMGL gr, const char *EqX,const char *EqY,const char *EqZ,const char *EqA)
+{	gr->SetFunc(EqX,EqY,EqZ,EqA);	}
 /// Set predefined coordinate system
-void mgl_set_coor(HMGL gr, int how)	{	gr->Axis(how);	}
-void mgl_set_cutoff(HMGL gr, const char *EqC)
-{	gr->CutOff(EqC);	}
+void mgl_set_coor(HMGL gr, int how)	{	gr->SetCoor(how);	}
+void mgl_set_cutoff(HMGL gr, const char *EqC)	{	gr->CutOff(EqC);	}
 /// Draws bounding box outside the plotting volume by default color.
 void mgl_box(HMGL gr, int ticks)
 {	gr->Box(NC,ticks);	}
@@ -245,44 +246,22 @@ void mgl_axis_grid(HMGL gr, const char *dir,const char *pen)
 void mgl_label(HMGL gr, char dir, const char *text)
 {	gr->Label(dir, text);	}
 /// Print the label \a text for axis \a dir.
-void mgl_label_ext(HMGL gr, char dir, const char *text, int pos, mreal size, mreal shift)
+void mgl_label_ext(HMGL gr, char dir, const char *text, mreal pos, mreal size, mreal shift)
 {	gr->Label(dir, text, pos, size, shift);	}
-void mgl_set_xttw(HMGL gr, const wchar_t *templ)	{gr->xtt = templ;}
-void mgl_set_yttw(HMGL gr, const wchar_t *templ)	{gr->ytt = templ;}
-void mgl_set_zttw(HMGL gr, const wchar_t *templ)	{gr->ztt = templ;}
-void mgl_set_cttw(HMGL gr, const wchar_t *templ)	{gr->ctt = templ;}
-void mgl_set_xtt(HMGL gr, const char *templ)
-{
-	int n = strlen(templ)+1;
-	static wchar_t t[256];
-	memset(t,0,256*sizeof(wchar_t));
-	if(n<256)
-	{	mbstowcs(t,templ,n);	gr->xtt = t;	}
-}
-void mgl_set_ytt(HMGL gr, const char *templ)
-{
-	int n = strlen(templ)+1;
-	static wchar_t t[256];
-	memset(t,0,256*sizeof(wchar_t));
-	if(n<256)
-	{	mbstowcs(t,templ,n);	gr->ytt = t;	}
-}
-void mgl_set_ztt(HMGL gr, const char *templ)
-{
-	int n = strlen(templ)+1;
-	static wchar_t t[256];
-	memset(t,0,256*sizeof(wchar_t));
-	if(n<256)
-	{	mbstowcs(t,templ,n);	gr->ztt = t;	}
-}
-void mgl_set_ctt(HMGL gr, const char *templ)
-{
-	int n = strlen(templ)+1;
-	static wchar_t t[256];
-	memset(t,0,256*sizeof(wchar_t));
-	if(n<256)
-	{	mbstowcs(t,templ,n);	gr->ctt = t;	}
-}
+void mgl_labelw_ext(HMGL gr, char dir, const wchar_t *text, mreal pos, mreal size, mreal shift)
+{	gr->Labelw(dir, text, pos, size, shift);	}
+void mgl_label_xy(HMGL gr, mreal x, mreal y, const char *text, const char *fnt, mreal size)
+{	gr->Label(x,y,text,fnt,size);	}
+void mgl_labelw_xy(HMGL gr, mreal x, mreal y, const wchar_t *text, const char *fnt, mreal size)
+{	gr->Labelw(x,y,text,fnt,size);	}
+void mgl_set_xttw(HMGL gr, const wchar_t *templ)	{gr->SetXTT(templ);}
+void mgl_set_yttw(HMGL gr, const wchar_t *templ)	{gr->SetYTT(templ);}
+void mgl_set_zttw(HMGL gr, const wchar_t *templ)	{gr->SetZTT(templ);}
+void mgl_set_cttw(HMGL gr, const wchar_t *templ)	{gr->SetCTT(templ);}
+void mgl_set_xtt(HMGL gr, const char *templ)	{gr->SetXTT(templ);}
+void mgl_set_ytt(HMGL gr, const char *templ)	{gr->SetYTT(templ);}
+void mgl_set_ztt(HMGL gr, const char *templ)	{gr->SetZTT(templ);}
+void mgl_set_ctt(HMGL gr, const char *templ)	{gr->SetCTT(templ);}
 //-----------------------------------------------------------------------------
 //		Simple drawing
 //-----------------------------------------------------------------------------
@@ -293,8 +272,8 @@ void mgl_ball(HMGL gr, mreal x,mreal y,mreal z)
 void mgl_ball_rgb(HMGL gr, mreal x, mreal y, mreal z, mreal r, mreal g, mreal b, mreal alpha)
 {	gr->Ball(x,y,z,mglColor(r,g,b),alpha);	}
 /// Draws the point (ball) at position \a p with color \a col.
-void mgl_ball_str(HMGL gr, mreal x, mreal y, mreal z, const char *col)
-{	gr->Ball(mglPoint(x,y,z),col[0]);	}
+void mgl_ball_str(HMGL gr, mreal x, mreal y, mreal z, char col)
+{	gr->Ball(mglPoint(x,y,z),col);	}
 /// Draws the line between points with style \a sch.
 void mgl_line(HMGL gr, mreal x1, mreal y1, mreal z1, mreal x2, mreal y2, mreal z2, const char *pen,int n)
 {	gr->Line(mglPoint(x1,y1,z1),mglPoint(x2,y2,z2),pen,n);	}
@@ -397,7 +376,7 @@ void mgl_get_last_mouse_pos(HMGL gr, mreal *x, mreal *y, mreal *z)
 {
 	mglGraphAB *g = dynamic_cast<mglGraphAB *>(gr);
 	if(g)
-	{	*x=g->LastMousePos.x;	*y=g->LastMousePos.y;	*y=g->LastMousePos.y;}
+	{	*x=g->LastMousePos.x;	*y=g->LastMousePos.y;	*z=g->LastMousePos.z;}
 }
 void mgl_calc_xyz(HMGL gr, int xs, int ys, mreal *x, mreal *y, mreal *z)
 {
@@ -408,6 +387,11 @@ void mgl_calc_xyz(HMGL gr, int xs, int ys, mreal *x, mreal *y, mreal *z)
 		*x = p.x;	*y = p.y;	*z = p.z;
 	}
 }
+void mgl_calc_scr(HMGL gr, mreal x, mreal y, mreal z, int *xs, int *ys)
+{
+	mglGraphAB *g = dynamic_cast<mglGraphAB *>(gr);
+	if(g && xs && ys)	g->CalcScr(mglPoint(x,y,z),xs,ys);
+}
 //-----------------------------------------------------------------------------
 mreal mgl_data_get_value(const HMDT d, int i, int j, int k)
 {	return	d->a[i+d->nx*(j+d->ny*k)];	}
@@ -418,10 +402,7 @@ void mgl_data_set_value(HMDT d, mreal v, int i, int j, int k)
 void mgl_set_zoom(HMGL gr, mreal x1, mreal y1, mreal x2, mreal y2)
 {	gr->Zoom(x1,y1,x2,y2);	}
 void mgl_set_plotfactor(HMGL gr, mreal val)
-{
-	if(val>0)	{	gr->PlotFactor = val;	gr->AutoPlotFactor=false;	}
-	else		{	gr->PlotFactor = 1.55f;	gr->AutoPlotFactor=true;	}
-}
+{	gr->SetPlotFactor(val);	}
 void mgl_set_axis_3d(HMGL gr, mreal x1, mreal y1, mreal z1, mreal x2, mreal y2, mreal z2)
 {	gr->Axis(mglPoint(x1,y1,z1),mglPoint(x2,y2,z2));	}
 void mgl_set_axis_2d(HMGL gr, mreal x1, mreal y1, mreal x2, mreal y2)
