@@ -3,17 +3,17 @@
  * Copyright (C) 2007 Alexey Balakin <balakin@appl.sci-nnov.ru>            *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
- *   it under the terms of the GNU General Public License as published by  *
- *   the Free Software Foundation; either version 2 of the License, or     *
- *   (at your option) any later version.                                   *
+ *   it under the terms of the GNU Library General Public License as       *
+ *   published by the Free Software Foundation; either version 3 of the    *
+ *   License, or (at your option) any later version.                       *
  *                                                                         *
  *   This program is distributed in the hope that it will be useful,       *
  *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
  *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
  *   GNU General Public License for more details.                          *
  *                                                                         *
- *   You should have received a copy of the GNU General Public License     *
- *   along with this program; if not, write to the                         *
+ *   You should have received a copy of the GNU Library General Public     *
+ *   License along with this program; if not, write to the                 *
  *   Free Software Foundation, Inc.,                                       *
  *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
  ***************************************************************************/
@@ -62,6 +62,7 @@ void mglGraph::DensX(const mglData &a, const char *sch,mreal sVal)
 	}
 	surf_plot(n, m, pp, cc, 0);
 	EndGroup();
+	if(a.nz>1)	delete []b;
 	delete []pp;	delete []cc;	if(bb)	delete []b;
 }
 //-----------------------------------------------------------------------------
@@ -102,6 +103,7 @@ void mglGraph::DensY(const mglData &a, const char *sch,mreal sVal)
 	}
 	surf_plot(n, m, pp, cc, 0);
 	EndGroup();
+	if(a.nz>1)	delete []b;
 	delete []pp;	delete []cc;	if(bb)	delete []b;
 }
 //-----------------------------------------------------------------------------
@@ -141,6 +143,7 @@ void mglGraph::DensZ(const mglData &a, const char *sch,mreal sVal)
 	}
 	surf_plot(n, m, pp, cc, 0);
 	EndGroup();
+	if(a.nz>1)	delete []b;
 	delete []pp;	delete []cc;	if(bb)	delete []b;
 }
 //-----------------------------------------------------------------------------
@@ -210,6 +213,7 @@ void mglGraph::ContX(const mglData &v, const mglData &a, const char *sch, mreal
 		cont_plot(v.a[i],n,m,b,xx,yy,zz,sVal,false,false,text);
 	}
 	EndGroup();
+	if(a.nz>1)	delete []b;
 	delete []xx;	delete []yy;	delete []zz;	if(bb)	delete []b;
 }
 //-----------------------------------------------------------------------------
@@ -251,6 +255,7 @@ void mglGraph::ContY(const mglData &v, const mglData &a, const char *sch, mreal
 		cont_plot(v.a[i],n,m,b,xx,yy,zz,sVal,false,false,text);
 	}
 	EndGroup();
+	if(a.nz>1)	delete []b;
 	delete []xx;	delete []yy;	delete []zz;	if(bb)	delete []b;
 }
 //-----------------------------------------------------------------------------
@@ -291,6 +296,7 @@ void mglGraph::ContZ(const mglData &v, const mglData &a, const char *sch, mreal
 		cont_plot(v.a[i],n,m,b,xx,yy,zz,sVal,false,false,text);
 	}
 	EndGroup();
+	if(a.nz>1)	delete []b;
 	delete []xx;	delete []yy;	delete []zz;	if(bb)	delete []b;
 }
 //-----------------------------------------------------------------------------
@@ -360,6 +366,7 @@ void mglGraph::ContFX(const mglData &v, const mglData &a, const char *sch, mreal
 		contf_plot(v.a[i],v.a[i+1],n,m,b,xx,yy,zz,sVal);
 	}
 	EndGroup();
+	if(a.nz>1)	delete []b;
 	delete []xx;	delete []yy;	delete []zz;	if(bb)	delete []b;
 }
 //-----------------------------------------------------------------------------
@@ -401,6 +408,7 @@ void mglGraph::ContFY(const mglData &v, const mglData &a, const char *sch, mreal
 		contf_plot(v.a[i],v.a[i+1],n,m,b,xx,yy,zz,sVal);
 	}
 	EndGroup();
+	if(a.nz>1)	delete []b;
 	delete []xx;	delete []yy;	delete []zz;	if(bb)	delete []b;
 }
 //-----------------------------------------------------------------------------
@@ -441,6 +449,7 @@ void mglGraph::ContFZ(const mglData &v, const mglData &a, const char *sch, mreal
 		contf_plot(v.a[i],v.a[i+1],n,m,b,xx,yy,zz,sVal);
 	}
 	EndGroup();
+	if(a.nz>1)	delete []b;
 	delete []xx;	delete []yy;	delete []zz;	if(bb)	delete []b;
 }
 //-----------------------------------------------------------------------------
