@@ -3,17 +3,17 @@
  * Copyright (C) 2007 Alexey Balakin <balakin@appl.sci-nnov.ru>            *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
- *   it under the terms of the GNU General Public License as published by  *
- *   the Free Software Foundation; either version 2 of the License, or     *
- *   (at your option) any later version.                                   *
+ *   it under the terms of the GNU Library General Public License as       *
+ *   published by the Free Software Foundation; either version 3 of the    *
+ *   License, or (at your option) any later version.                       *
  *                                                                         *
  *   This program is distributed in the hope that it will be useful,       *
  *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
  *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
  *   GNU General Public License for more details.                          *
  *                                                                         *
- *   You should have received a copy of the GNU General Public License     *
- *   along with this program; if not, write to the                         *
+ *   You should have received a copy of the GNU Library General Public     *
+ *   License along with this program; if not, write to the                 *
  *   Free Software Foundation, Inc.,                                       *
  *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
  ***************************************************************************/
@@ -124,10 +124,9 @@ void mglGraph::Dots(const mglData &x, const mglData &y, const mglData &z, const
 	if(alpha>1)	alpha = 1;
 	mglColor c;
 
-	bool al=Alpha(true);
-	for(long i=0;i<n;i++)
-		AVertex(x.a[i], y.a[i], z.a[i], a.a[i], alpha);
-	Alpha(al);
+//	bool al=Alpha(true);
+	for(long i=0;i<n;i++)	AVertex(x.a[i], y.a[i], z.a[i], a.a[i], alpha);
+//	Alpha(al);
 	EndGroup();
 }
 //-----------------------------------------------------------------------------
