@@ -3,17 +3,17 @@
  * Copyright (C) 2007 Alexey Balakin <balakin@appl.sci-nnov.ru>            *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
- *   it under the terms of the GNU General Public License as published by  *
- *   the Free Software Foundation; either version 2 of the License, or     *
- *   (at your option) any later version.                                   *
+ *   it under the terms of the GNU Library General Public License as       *
+ *   published by the Free Software Foundation; either version 3 of the    *
+ *   License, or (at your option) any later version.                       *
  *                                                                         *
  *   This program is distributed in the hope that it will be useful,       *
  *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
  *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
  *   GNU General Public License for more details.                          *
  *                                                                         *
- *   You should have received a copy of the GNU General Public License     *
- *   along with this program; if not, write to the                         *
+ *   You should have received a copy of the GNU Library General Public     *
+ *   License along with this program; if not, write to the                 *
  *   Free Software Foundation, Inc.,                                       *
  *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
  ***************************************************************************/
@@ -107,19 +107,19 @@ void mgl_data_delete(HMDT dat, char dir, int at, int num)
 {	dat->Delete(dir,at,num);	}
 //-----------------------------------------------------------------------------
 /// Allocate memory and copy the data from the (mreal *) array
-void mgl_data_set_mreal(HMDT d, const mreal *A,int NX,int NY,int NZ)
+void mgl_data_set_float(HMDT d, const float *A,int NX,int NY,int NZ)
 {	d->Set(A,NX,NY,NZ);	}
 /// Allocate memory and copy the data from the (mreal *) array
 void mgl_data_set_double(HMDT d, const double *A,int NX,int NY,int NZ)
 {	d->Set(A,NX,NY,NZ);	}
 /// Allocate memory and copy the data from the (mreal **) array
-void mgl_data_set_mreal2(HMDT d, const mreal **A,int N1,int N2)
+void mgl_data_set_float2(HMDT d, const float **A,int N1,int N2)
 {	d->Set(A,N1,N2);	}
 /// Allocate memory and copy the data from the (mreal **) array
 void mgl_data_set_double2(HMDT d, const double **A,int N1,int N2)
 {	d->Set(A,N1,N2);	}
 /// Allocate memory and copy the data from the (mreal ***) array
-void mgl_data_set_mreal3(HMDT d, const mreal ***A,int N1,int N2,int N3)
+void mgl_data_set_float3(HMDT d, const float ***A,int N1,int N2,int N3)
 {	d->Set(A,N1,N2,N3);	}
 /// Allocate memory and copy the data from the (mreal ***) array
 void mgl_data_set_double3(HMDT d, const double ***A,int N1,int N2,int N3)
@@ -229,7 +229,7 @@ void mgl_data_transpose_(uintptr_t *d, const char *dim,int l)
 void mgl_data_norm_(uintptr_t *d, mreal *v1,mreal *v2,int *sym,int *dim)
 {	_DT_->Norm(*v1,*v2,*sym,*dim);	}
 /// Normalize the data to range [v1,v2]
-void mgl_data_norm_slice_(uintptr_t *d, mreal *v1,mreal *v2,char *dir,int *keep_en,int *sym,int l)
+void mgl_data_norm_slice_(uintptr_t *d, mreal *v1,mreal *v2,char *dir,int *keep_en,int *sym,int )
 {	_DT_->NormSl(*v1,*v2,*dir,*keep_en,*sym);	}
 /// Reduce size of the data
 void mgl_data_squeeze_(uintptr_t *d, int *rx,int *ry,int *rz,int *smooth)
@@ -260,25 +260,25 @@ void mgl_data_delete_(uintptr_t *d, const char *dir, int *at, int *num, int)
 {	_DT_->Delete(*dir,*at,*num);	}
 //-----------------------------------------------------------------------------
 /// Allocate memory and copy the data from the (mreal *) array
-void mgl_data_set_mreal1_(uintptr_t *d, const mreal *A,int *NX)
+void mgl_data_set_float1_(uintptr_t *d, const float *A,int *NX)
 {	_DT_->Set(A,*NX);	}
 /// Allocate memory and copy the data from the (mreal *) array
 void mgl_data_set_double1_(uintptr_t *d, const double *A,int *NX)
 {	_DT_->Set(A,*NX);	}
 /// Allocate memory and copy the data from the (mreal *) array
-void mgl_data_set_mreal_(uintptr_t *d, const mreal *A,int *NX,int *NY,int *NZ)
+void mgl_data_set_float_(uintptr_t *d, const float *A,int *NX,int *NY,int *NZ)
 {	_DT_->Set(A,*NX,*NY,*NZ);	}
 /// Allocate memory and copy the data from the (mreal *) array
 void mgl_data_set_double_(uintptr_t *d, const double *A,int *NX,int *NY,int *NZ)
 {	_DT_->Set(A,*NX,*NY,*NZ);	}
 /// Allocate memory and copy the data from the (mreal **) array
-void mgl_data_set_mreal2_(uintptr_t *d, const mreal *A,int *N1,int *N2)
+void mgl_data_set_float2_(uintptr_t *d, const float *A,int *N1,int *N2)
 {	_DT_->Set(A,*N1,*N2);	}
 /// Allocate memory and copy the data from the (mreal **) array
 void mgl_data_set_double2_(uintptr_t *d, const double *A,int *N1,int *N2)
 {	_DT_->Set(A,*N1,*N2);	}
 /// Allocate memory and copy the data from the (mreal ***) array
-void mgl_data_set_mreal3_(uintptr_t *d, const mreal *A,int *N1,int *N2,int *N3)
+void mgl_data_set_float3_(uintptr_t *d, const float *A,int *N1,int *N2,int *N3)
 {	_DT_->Set(A,*N1,*N2,*N3);	}
 /// Allocate memory and copy the data from the (mreal ***) array
 void mgl_data_set_double3_(uintptr_t *d, const double *A,int *N1,int *N2,int *N3)
@@ -466,7 +466,7 @@ HMDT mgl_data_evaluate_i(const HMDT dat, const HMDT idat, int norm)
 HMDT mgl_data_evaluate_ij(const HMDT dat, const HMDT idat, const HMDT jdat, int norm)
 {	return new mglData(dat->Evaluate(*idat,*jdat,norm));	}
 HMDT mgl_data_evaluate_ijk(const HMDT dat, const HMDT idat, const HMDT jdat, const HMDT kdat, int norm)
-{	return new mglData(dat->Evaluate(*idat,*jdat,*dat,norm));	}
+{	return new mglData(dat->Evaluate(*idat,*jdat,*kdat,norm));	}
 void mgl_data_envelop(HMDT dat, char dir)	{	dat->Envelop(dir);	}
 void mgl_data_sew(HMDT dat, const char *dirs, mreal da)
 {	dat->Sew(dirs,da);	}
@@ -530,3 +530,57 @@ void mgl_data_diff_par_(uintptr_t *d, uintptr_t *v1, uintptr_t *v2, uintptr_t *v
 	else	_DT_->Diff(_D_(v1),_D_(v2));
 }
 //-----------------------------------------------------------------------------
+mreal mgl_data_first(const HMDT d, const char *cond, int *i, int *j, int *k)
+{	return d->Find(cond,*i,*j,*k);	}
+mreal mgl_data_last(const HMDT d, const char *cond, int *i, int *j, int *k)
+{	return d->Last(cond,*i,*j,*k);	}
+int mgl_data_find(const HMDT d, const char *cond, char dir, int i, int j, int k)
+{	return d->Find(cond,dir,i,j,k);	}
+int mgl_data_find_any(const HMDT d, const char *cond)
+{	return d->FindAny(cond);	}
+mreal mgl_data_first_(uintptr_t *d, const char *cond, int *i, int *j, int *k, int l)
+{
+	char *s=new char[l+1];	memcpy(s,cond,l);	s[l]=0;
+	mreal res = _DT_->Find(s,*i,*j,*k);
+	delete []s;		return res;
+}
+mreal mgl_data_last_(uintptr_t *d, const char *cond, int *i, int *j, int *k, int l)
+{
+	char *s=new char[l+1];	memcpy(s,cond,l);	s[l]=0;
+	mreal res = _DT_->Last(s,*i,*j,*k);
+	delete []s;		return res;
+}
+int mgl_data_find_(uintptr_t *d, const char *cond, char *dir, int *i, int *j, int *k, int l, int)
+{
+	char *s=new char[l+1];	memcpy(s,cond,l);	s[l]=0;
+	int res = _DT_->Find(s,*dir,*i,*j,*k);
+	delete []s;		return res;
+}
+int mgl_data_find_any_(uintptr_t *d, const char *cond, int l)
+{
+	char *s=new char[l+1];	memcpy(s,cond,l);	s[l]=0;
+	int res = _DT_->FindAny(s);
+	delete []s;		return res;
+}
+//-----------------------------------------------------------------------------
+mreal mgl_data_max_int(const HMDT d, int *i, int *j, int *k)
+{	return d->Maximal(*i,*j,*k);	}
+mreal mgl_data_max_real(const HMDT d, mreal *i, mreal *j, mreal *k)
+{	return d->Maximal(*i,*j,*k);	}
+mreal mgl_data_min_int(const HMDT d, int *i, int *j, int *k)
+{	return d->Minimal(*i,*j,*k);	}
+mreal mgl_data_min_real(const HMDT d, mreal *i, mreal *j, mreal *k)
+{	return d->Minimal(*i,*j,*k);	}
+mreal mgl_data_momentum_mw(const HMDT d, char dir, mreal *m, mreal *w)
+{	return d->Momentum(dir,*m,*w);	}
+mreal mgl_data_max_int_(uintptr_t *d, int *i, int *j, int *k)
+{	return _DT_->Maximal(*i,*j,*k);	}
+mreal mgl_data_max_real_(uintptr_t *d, mreal *i, mreal *j, mreal *k)
+{	return _DT_->Maximal(*i,*j,*k);	}
+mreal mgl_data_min_int_(uintptr_t *d, int *i, int *j, int *k)
+{	return _DT_->Minimal(*i,*j,*k);	}
+mreal mgl_data_min_real_(uintptr_t *d, mreal *i, mreal *j, mreal *k)
+{	return _DT_->Minimal(*i,*j,*k);	}
+mreal mgl_data_momentum_mw_(uintptr_t *d, char *dir, mreal *m, mreal *w,int)
+{	return _DT_->Momentum(*dir,*m,*w);	}
+//-----------------------------------------------------------------------------
