@@ -3,17 +3,17 @@
  * Copyright (C) 2007 Alexey Balakin <balakin@appl.sci-nnov.ru>            *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
- *   it under the terms of the GNU General Public License as published by  *
- *   the Free Software Foundation; either version 2 of the License, or     *
- *   (at your option) any later version.                                   *
+ *   it under the terms of the GNU Library General Public License as       *
+ *   published by the Free Software Foundation; either version 3 of the    *
+ *   License, or (at your option) any later version.                       *
  *                                                                         *
  *   This program is distributed in the hope that it will be useful,       *
  *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
  *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
  *   GNU General Public License for more details.                          *
  *                                                                         *
- *   You should have received a copy of the GNU General Public License     *
- *   along with this program; if not, write to the                         *
+ *   You should have received a copy of the GNU Library General Public     *
+ *   License along with this program; if not, write to the                 *
  *   Free Software Foundation, Inc.,                                       *
  *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
  ***************************************************************************/
@@ -32,7 +32,8 @@
 #include "mgl/mgl_eval.h"
 #include "mgl/mgl_data.h"
 
-#define isn(ch)		((ch)<' ' && (ch)!='\t')
+//#define isn(ch)		((ch)<' ' && (ch)!='\t')
+#define isn(ch)		((ch)=='\n')
 //-----------------------------------------------------------------------------
 void mglData::Set(const char *v,int NX,int NY,int NZ)
 {
@@ -267,7 +268,7 @@ void mglData::Save(const char *fname,int ns) const
 	FILE *fp;
 	fp = fopen(fname,"w");
 	if(ns<0 || (ns>=nz && nz>1))	for(long k=0;k<nz;k++)
-	{	//   
+	{	// ñîõðàíÿåì âñå äàííûå
 		for(long i=0;i<ny;i++)
 		{
 			for(long j=0;j<nx;j++)
@@ -277,8 +278,8 @@ void mglData::Save(const char *fname,int ns) const
 		fprintf(fp,"\n");
 	}
 	else
-	{	//   
-		if(nz>1)		//  3D -- 
+	{	// ñîõðàíÿåì òîëüêî ñðåç
+		if(nz>1)		// äëÿ 3D -- ïëîñêîñòü
 		{
 			for(long i=0;i<ny;i++)
 			{
@@ -287,7 +288,7 @@ void mglData::Save(const char *fname,int ns) const
 				fprintf(fp,"\n");
 			}
 		}
-		else if(ns<ny)	//  2D -- 
+		else if(ns<ny)	// äëÿ 2D -- ëèíèÿ
 		{
 			for(long j=0;j<nx;j++)
 				fprintf(fp,"%g\t",a[j+nx*ns]);
@@ -681,8 +682,8 @@ mreal mglData::Minimal(mreal &x,mreal &y,mreal &z) const
 //-----------------------------------------------------------------------------
 void mglData::Modify(const char *eq,int dim)
 {
-	long i,j,k,i0;
-	mreal x,y,z,dx=nx>1?1/(nx-1.):0,dy=ny>1?1/(ny-1.):0;
+	long i,j,k;
+	mreal y,z,dx=nx>1?1/(nx-1.):0,dy=ny>1?1/(ny-1.):0, *aa;
 	mglFormula eqs(eq);
 	if(dim<0)	dim=0;
 	if(nz>1)	// 3D array
@@ -690,24 +691,21 @@ void mglData::Modify(const char *eq,int dim)
 		for(k=dim;k<nz;k++)
 		{
 			z = (nz>dim+1) ? (k-dim)/(nz-dim-1.) : 0;
-			for(j=0;j<ny;j++)	for(i=0;i<nx;i++)
-			{
-				x = dx*i;	y = dy*j;
-				i0 = i+nx*(j+ny*k);
-				a[i0] = eqs.Calc(x,y,z,a[i0]);
-			}
+			aa = a+nx*ny*k;
+//#pragma omp parallel for
+			for(i=0;i<nx*ny;i++)
+				aa[i] = eqs.Calc(dx*(i%nx),dy*(i/nx),z,aa[i]);
 		}
 	}
 	else		// 2D or 1D array
 	{
-		z = 0;
 		if(ny==1)	dim = 0;
 		dy = ny>dim+1 ? 1/(ny-dim-1.) : 0;
-		for(j=dim;j<ny;j++)	for(i=0;i<nx;i++)
+		for(j=dim;j<ny;j++)
 		{
-			x = dx*i;	y = dy*(j-dim);
-			i0 = i+nx*j;
-			a[i0] = eqs.Calc(x,y,0,a[i0]);
+			y = dy*(j-dim);		aa = a+nx*j;
+//#pragma omp parallel for
+			for(i=0;i<nx;i++)	aa[i] = eqs.Calc(dx*i,y,0,aa[i]);
 		}
 	}
 }
@@ -758,25 +756,25 @@ void mglData::Squeeze(int rx,int ry,int rz,bool smooth)
 	mreal *b;
 
 	// simple checking
-	if(rx<1)	rx=1;	if(rx>nx)	rx=nx;
-	if(ry<1)	ry=1;	if(ry>ny)	ry=ny;
-	if(rz<1)	rz=1;	if(rz>nz)	rz=nz;
+	if(rx>=nx)	rx=nx-1;	if(rx<1)	rx=1;
+	if(ry>=ny)	ry=ny-1;	if(ry<1)	ry=1;
+	if(rz>=nz)	rz=nz-1;	if(rz<1)	rz=1;
 	// new sizes
-	kx = nx/rx;	ky = ny/ry;	kz = nz/rz;
+	kx = 1+(nx-1)/rx;	ky = 1+(ny-1)/ry;	kz = 1+(nz-1)/rz;
 	b = new mreal[kx*ky*kz];
-//	if(!smooth)
-	for(i=0;i<kx;i++)  for(j=0;j<ky;j++)  for(k=0;k<kz;k++)
+	if(!smooth)	for(i=0;i<kx;i++)  for(j=0;j<ky;j++)  for(k=0;k<kz;k++)
 		b[i+kx*(j+ky*k)] = a[i*rx+nx*(j*ry+ny*rz*k)];
-/*	else
-		for(i=0;i<kx;i++)  for(j=0;j<ky;j++)  for(k=0;k<kz;k++)
-		{
-			long dx,dy,dz,i1,j1,k1;
-			dx = (i+1)*rx<=nx ? rx : nx-i*rx;
-			dy = (j+1)*ry<=ny ? ry : ny-j*ry;
-			dz = (k+1)*rz<=nz ? rz : nz-k*rz;
-			for(i1=i*rx;i1<i*rx+dx;i1++)	for(j1=j*ry;j1<j*ry+dz;j1++)	for(k1=k*rz;k1<k*rz+dz;k1++)
-				b[i+kx*(j+ky*k)] += a[i*rx+nx*(j*ry+ny*rz*k)]/(dx*dy*dz);
-		}*/
+	else		for(i=0;i<kx;i++)  for(j=0;j<ky;j++)  for(k=0;k<kz;k++)
+	{
+		long dx,dy,dz,i1,j1,k1;
+		dx = (i+1)*rx<=nx ? rx : nx-i*rx;
+		dy = (j+1)*ry<=ny ? ry : ny-j*ry;
+		dz = (k+1)*rz<=nz ? rz : nz-k*rz;
+		mreal s = 0;
+		for(i1=i*rx;i1<i*rx+dx;i1++)	for(j1=j*ry;j1<j*ry+dz;j1++)	for(k1=k*rz;k1<k*rz+dz;k1++)
+			s += a[i1+nx*(j1+ny*k1)];
+		b[i+kx*(j+ky*k)] = s/dx*dy*dz;
+	}
 	delete []a;
 	a = b;
 	nx = kx;  ny = ky;  nz = kz;	NewId();
@@ -918,7 +916,11 @@ void mglData::SaveHDF(const char *fname,const char *data,bool rewrite) const
 	hid_t hf,hd,hs;
 	hsize_t dims[3];
 	long rank = 3, res;
+#ifndef H5_USE_16_API
+	H5Eset_auto(H5E_DEFAULT,0,0);
+#else
 	H5Eset_auto(0,0);
+#endif
 	res=H5Fis_hdf5(fname);
 	if(res>0 && !rewrite)	hf = H5Fopen(fname, H5F_ACC_RDWR, H5P_DEFAULT);
 	else	hf = H5Fcreate(fname, H5F_ACC_TRUNC, H5P_DEFAULT, H5P_DEFAULT);
@@ -928,10 +930,18 @@ void mglData::SaveHDF(const char *fname,const char *data,bool rewrite) const
 	else	{	rank = 3;	dims[0] = nz;	dims[1] = ny;	dims[2] = nx;	}
 	hs = H5Screate_simple(rank, dims, 0);
 #if(MGL_USE_DOUBLE==1)
+#ifndef H5_USE_16_API
+	hd = H5Dcreate(hf, data, H5T_NATIVE_DOUBLE, hs, H5P_DEFAULT, H5P_DEFAULT, H5P_DEFAULT);
+#else  /* ! HAVE_HDF5_18 */
 	hd = H5Dcreate(hf, data, H5T_NATIVE_DOUBLE, hs, H5P_DEFAULT);
+#endif /* HAVE_HDF5_18 */
 	H5Dwrite(hd, H5T_NATIVE_DOUBLE, hs, hs, H5P_DEFAULT, a);
 #else
+#ifndef H5_USE_16_API
+	hd = H5Dcreate(hf, data, H5T_NATIVE_FLOAT, hs, H5P_DEFAULT, H5P_DEFAULT, H5P_DEFAULT);
+#else  /* ! HAVE_HDF5_18 */
 	hd = H5Dcreate(hf, data, H5T_NATIVE_FLOAT, hs, H5P_DEFAULT);
+#endif /* HAVE_HDF5_18 */
 	H5Dwrite(hd, H5T_NATIVE_FLOAT, hs, hs, H5P_DEFAULT, a);
 #endif
 	H5Dclose(hd);	H5Sclose(hs);	H5Fclose(hf);
@@ -943,7 +953,11 @@ void mglData::ReadHDF(const char *fname,const char *data)
 	hsize_t dims[3];
 	long rank;
 	hf = H5Fopen(fname, H5F_ACC_RDONLY, H5P_DEFAULT);
+#ifndef H5_USE_16_API
+	hd = H5Dopen(hf,data,H5P_DEFAULT);
+#else
 	hd = H5Dopen(hf,data);
+#endif
 	hs = H5Dget_space(hd);
 	rank = H5Sget_simple_extent_ndims(hs);
 	if(rank>0 && rank<=3)
@@ -966,8 +980,8 @@ void mglData::ReadHDF(const char *fname,const char *data)
 	H5Dclose(hd);	H5Sclose(hs);	H5Fclose(hf);
 }
 #else
-void mglData::SaveHDF(const char *fname,const char *data,bool rewrite) const {}
-void mglData::ReadHDF(const char *fname,const char *data)	{}
+void mglData::SaveHDF(const char *,const char *,bool ) const {}
+void mglData::ReadHDF(const char *,const char *)	{}
 #endif
 //-----------------------------------------------------------------------------
 bool mgl_add_file(long &kx,long &ky, long &kz, mreal *&b, mglData &d,bool as_slice)
@@ -1051,7 +1065,7 @@ bool mglData::ReadAll(const char *templ, bool as_slice)
 	for(i=0;i<res.gl_pathc;i++)
 		if(d.Read(res.gl_pathv[i]))	break;
 
-	if(i>=res.gl_pathc)	return false;
+	if(i>=res.gl_pathc)	{	delete []fname;	return false;	}
 	kx = d.nx;	ky = d.ny;	kz = d.nz;
 	b = (mreal *)malloc(kx*ky*kz*sizeof(mreal));
 	memcpy(b,d.a,kx*ky*kz*sizeof(mreal));
@@ -1060,7 +1074,7 @@ bool mglData::ReadAll(const char *templ, bool as_slice)
 	{
 		if(d.Read(res.gl_pathv[i]))
 			if(!mgl_add_file(kx,ky,kz,b,d,as_slice))
-				return false;
+			{	delete []fname;		return false;	}
 	}
 	Set(b,kx,ky,kz);
 
