@@ -3,22 +3,24 @@
  * Copyright (C) 2007 Alexey Balakin <balakin@appl.sci-nnov.ru>            *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
- *   it under the terms of the GNU General Public License as published by  *
- *   the Free Software Foundation; either version 2 of the License, or     *
- *   (at your option) any later version.                                   *
+ *   it under the terms of the GNU Library General Public License as       *
+ *   published by the Free Software Foundation; either version 3 of the    *
+ *   License, or (at your option) any later version.                       *
  *                                                                         *
  *   This program is distributed in the hope that it will be useful,       *
  *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
  *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
  *   GNU General Public License for more details.                          *
  *                                                                         *
- *   You should have received a copy of the GNU General Public License     *
- *   along with this program; if not, write to the                         *
+ *   You should have received a copy of the GNU Library General Public     *
+ *   License along with this program; if not, write to the                 *
  *   Free Software Foundation, Inc.,                                       *
  *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
  ***************************************************************************/
 #include <stdlib.h>
+#ifndef NO_PNG
 #include <png.h>
+#endif
 #include "mgl/mgl_data.h"
 #include "mgl/mgl_c.h"
 #include "mgl/mgl_f.h"
@@ -72,7 +74,7 @@ unsigned char *mgl_create_scheme(const char *scheme,long &num)
 		case 'P':	cc[3*np]=128;	cc[3*np+1]=0;	cc[3*np+2]=64;	np++;	break;
 		}
 	}
-	if(np<2)	return 0;
+	if(np<2)	{	delete []cc;	return 0;	}
 	for(i=0;i<np-1;i++)	nc+=mgl_col_dif(cc+3*i,cc+3*i+3,false);
 	c = new unsigned char[3*nc+3];
 	long dd,pos=0;
@@ -98,6 +100,7 @@ void mglData::Import(const char *fname,const char *scheme,mreal v1,mreal v2)
 	if(v1>=v2)	return;
 	long num=0;
 
+#ifndef NO_PNG
 	FILE *fp = fopen(fname, "rb");
 	if (!fp)	return;
 	png_structp png_ptr = png_create_read_struct(PNG_LIBPNG_VER_STRING, 0, 0, 0);
@@ -138,6 +141,7 @@ void mglData::Import(const char *fname,const char *scheme,mreal v1,mreal v2)
 	delete []c;
 	png_destroy_read_struct(&png_ptr, &info_ptr,&end_info);
 	fclose(fp);
+#endif
 }
 //-----------------------------------------------------------------------------
 void mglData::Export(const char *fname,const char *scheme,mreal v1,mreal v2,int ns) const
@@ -153,6 +157,7 @@ void mglData::Export(const char *fname,const char *scheme,mreal v1,mreal v2,int
 	}
 	if(v1==v2)	return;
 
+#ifndef NO_PNG
 	long num=0;
 	unsigned char *c = mgl_create_scheme(scheme,num);
 	if(num<2)	{	delete []c;		return;		}
@@ -192,6 +197,7 @@ void mglData::Export(const char *fname,const char *scheme,mreal v1,mreal v2,int
 
 	png_destroy_write_struct(&png_ptr, &info_ptr);
 	fclose(fp);	free(p);	free(d);
+#endif
 }
 //-----------------------------------------------------------------------------
 void mgl_data_export(HMDT dat, const char *fname, const char *scheme,mreal v1,mreal v2,int ns)
