@@ -3,17 +3,17 @@
  * Copyright (C) 2007 Alexey Balakin <balakin@appl.sci-nnov.ru>            *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
- *   it under the terms of the GNU General Public License as published by  *
- *   the Free Software Foundation; either version 2 of the License, or     *
- *   (at your option) any later version.                                   *
+ *   it under the terms of the GNU Library General Public License as       *
+ *   published by the Free Software Foundation; either version 3 of the    *
+ *   License, or (at your option) any later version.                       *
  *                                                                         *
  *   This program is distributed in the hope that it will be useful,       *
  *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
  *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
  *   GNU General Public License for more details.                          *
  *                                                                         *
- *   You should have received a copy of the GNU General Public License     *
- *   along with this program; if not, write to the                         *
+ *   You should have received a copy of the GNU Library General Public     *
+ *   License along with this program; if not, write to the                 *
  *   Free Software Foundation, Inc.,                                       *
  *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
  ***************************************************************************/
@@ -39,13 +39,6 @@ int mgl_compare_prim(const void *p1, const void *p2)
 	return 0;
 }
 //-----------------------------------------------------------------------------
-/// Create mglGraph object in PostScript mode.
-HMGL mgl_create_graph_ps(int width, int height)
-{    return new mglGraphPS(width,height);	}
-/// Create mglGraph object in PostScript mode.
-uintptr_t mgl_create_graph_ps_(int *width, int *height)
-{    return uintptr_t(new mglGraphPS(*width,*height));	}
-//-----------------------------------------------------------------------------
 mglGraphPS::mglGraphPS(int w,int h) : mglGraphAB(w,h)
 {	P = 0;	pMax = pNum = 0;	}
 //-----------------------------------------------------------------------------
@@ -61,7 +54,7 @@ void mglGraphPS::Ball(mreal x,mreal y,mreal z,mglColor col,mreal alpha)
 	mglPrim a;		a.m = '.';
 	a.x[0] = p[0];	a.y[0] = p[1];	a.z = a.zz[0]=p[2];
 	a.c[0] = col.r;	a.c[1] = col.g;	a.c[2] = col.b;
-	add_prim(&a);
+	add_prim(a);
 }
 //-----------------------------------------------------------------------------
 void mglGraphPS::ball(mreal *p,mreal *c)
@@ -69,7 +62,7 @@ void mglGraphPS::ball(mreal *p,mreal *c)
 	mglPrim a;	PostScale(p,1);		a.m = '.';
 	a.x[0] = p[0];	a.y[0] = p[1];	a.z = a.zz[0]=p[2];
 	memcpy(a.c,c,3*sizeof(mreal));
-	add_prim(&a);
+	add_prim(a);
 }
 //-----------------------------------------------------------------------------
 void mglGraphPS::mark_plot(mreal *p, char type)
@@ -77,10 +70,10 @@ void mglGraphPS::mark_plot(mreal *p, char type)
 	mglPrim a;		a.m = type;		a.s = MarkSize;//*175*font_factor;
 	a.x[0] = p[0];	a.y[0] = p[1];	a.z = a.zz[0]=p[2];
 	memcpy(a.c,CDef,3*sizeof(mreal));
-	add_prim(&a);
+	add_prim(a);
 }
 //-----------------------------------------------------------------------------
-void mglGraphPS::line_plot(mreal *p1,mreal *p2,mreal *c1,mreal *,bool all)
+void mglGraphPS::line_plot(mreal *p1,mreal *p2,mreal *c1,mreal *c2,bool all)
 {
 	if((PDef==0 && !all) || (fabs(p1[0]-p2[0])<0.01 && fabs(p1[1]-p2[1])<0.01))	return;
 	mglPrim a(1);
@@ -89,10 +82,10 @@ void mglGraphPS::line_plot(mreal *p1,mreal *p2,mreal *c1,mreal *,bool all)
 	if(pw>1)	a.z += pw-1;
 	a.x[0]=p1[0];	a.y[0]=p1[1];	a.x[1]=p2[0];	a.y[1]=p2[1];
 	a.zz[0]=p1[2];	a.zz[1]=p2[2];
-//	a.c[0]=(c1[0]+c2[0])/2;	a.c[1]=(c1[1]+c2[1])/2;	a.c[2]=(c1[2]+c2[2])/2;
-	a.c[0]=c1[0];	a.c[1]=c1[1];	a.c[2]=c1[2];
+	a.c[0]=(c1[0]+c2[0])/2;	a.c[1]=(c1[1]+c2[1])/2;	a.c[2]=(c1[2]+c2[2])/2;
+//	a.c[0]=c1[0];	a.c[1]=c1[1];	a.c[2]=c1[2];
 	a.SetStyle(all? 0xffff:PDef,int(pPos));
-	add_prim(&a);
+	add_prim(a);
 	pPos = fmod(pPos+hypot(p2[0]-p1[0], p2[1]-p1[1])/pw/1.5, 16);
 }
 //-----------------------------------------------------------------------------
@@ -113,7 +106,7 @@ void mglGraphPS::trig_plot(mreal *p3,mreal *p1,mreal *p2,mreal *c3,mreal *c1,mre
 	a.c[0]=(c1[0]+c2[0]+c3[0])/3;	a.c[1]=(c1[1]+c2[1]+c3[1])/3;
 	a.c[2]=(c1[2]+c2[2]+c3[2])/3;	a.c[3]=(c1[3]+c2[3]+c3[3])/3;
 	add_light(a.c, d2[2]*d1[1]-d2[1]*d1[2], d2[0]*d1[2]-d2[2]*d1[0], d2[1]*d1[0]-d2[0]*d1[1]);
-	add_prim(&a);
+	add_prim(a);
 }
 //-----------------------------------------------------------------------------
 void mglGraphPS::trig_plot_n(mreal *p3,mreal *p1,mreal *p2,
@@ -130,7 +123,7 @@ void mglGraphPS::trig_plot_n(mreal *p3,mreal *p1,mreal *p2,
 	a.c[0]=(c1[0]+c2[0]+c3[0])/3;	a.c[1]=(c1[1]+c2[1]+c3[1])/3;
 	a.c[2]=(c1[2]+c2[2]+c3[2])/3;	a.c[3]=(c1[3]+c2[3]+c3[3])/3;
 	add_light(a.c, (n1[0]+n2[0]+n3[0])/3, (n1[1]+n2[1]+n3[1])/3, (n1[2]+n2[2]+n3[2])/3);
-	add_prim(&a);
+	add_prim(a);
 }
 //-----------------------------------------------------------------------------
 void mglGraphPS::quad_plot(mreal *p0,mreal *p1,mreal *p2,mreal *p3,
@@ -156,7 +149,7 @@ void mglGraphPS::quad_plot(mreal *p0,mreal *p1,mreal *p2,mreal *p3,
 	a.c[0]=(c1[0]+c2[0]+c3[0]+c0[0])/4;	a.c[1]=(c1[1]+c2[1]+c3[1]+c0[1])/4;
 	a.c[2]=(c1[2]+c2[2]+c3[2]+c0[2])/4;	a.c[3]=(c1[3]+c2[3]+c3[3]+c0[3])/4;
 	add_light(a.c, d1[0], d1[1], d1[2]);
-	add_prim(&a);
+	add_prim(a);
 }
 //-----------------------------------------------------------------------------
 void mglGraphPS::quad_plot_n(mreal *p0,mreal *p1,mreal *p2,mreal *p3,
@@ -175,16 +168,16 @@ void mglGraphPS::quad_plot_n(mreal *p0,mreal *p1,mreal *p2,mreal *p3,
 	a.c[0]=(c1[0]+c2[0]+c3[0]+c0[0])/4;	a.c[1]=(c1[1]+c2[1]+c3[1]+c0[1])/4;
 	a.c[2]=(c1[2]+c2[2]+c3[2]+c0[2])/4;	a.c[3]=(c1[3]+c2[3]+c3[3]+c0[3])/4;
 	add_light(a.c, n1[0]+n2[0]+n3[0]+n0[0], n1[1]+n2[1]+n3[1]+n0[1], n1[2]+n2[2]+n3[2]+n0[2]);
-	add_prim(&a);
+	add_prim(a);
 }
 //-----------------------------------------------------------------------------
 void mglGraphPS::quad_plot_a(mreal *p0,mreal *p1,mreal *p2,mreal *p3,
 				mreal a0,mreal a1,mreal a2,mreal a3, mreal alpha)
 {
-	bool pnt = fabs(p1[0]-p2[0])<0.01 && fabs(p1[1]-p2[1])<0.01 &&
+/*	bool pnt = fabs(p1[0]-p2[0])<0.01 && fabs(p1[1]-p2[1])<0.01 &&
 				fabs(p1[0]-p3[0])<0.01 && fabs(p1[1]-p3[1])<0.01 &&
 				fabs(p1[0]-p0[0])<0.01 && fabs(p1[1]-p0[1])<0.01;
-	if(pnt)	return;
+	if(pnt)	return;*/
 	mglPrim a(3);
 	a.z = (p1[2]+p2[2]+p3[2]+p0[2])/4;
 	a.x[0]=p0[0];	a.y[0]=p0[1];	a.zz[0]=p0[2];
@@ -195,14 +188,15 @@ void mglGraphPS::quad_plot_a(mreal *p0,mreal *p1,mreal *p2,mreal *p3,
 	mglColor c(GetC(v,false));
 	a.c[0]=c.r;		a.c[1]=c.g;		a.c[2]=c.b;
 	a.c[3]=alpha>0 ? alpha*(v+1)*(v+1) : -alpha*(v-1)*(v-1);
-	add_prim(&a);
+	add_prim(a);
 }
 //-----------------------------------------------------------------------------
 void mglGraphPS::Glyph(mreal x, mreal y, mreal f, int s, long j, char col)
 {
 	mglPrim a(4);
 	a.s = fscl/PlotFactor;	a.w = ftet;
-	a.x[0] = xPos;	a.y[0] = yPos;
+	a.x[0] = (xPos - zoomx1*Width) /zoomx2;
+	a.y[0] = (yPos - zoomy1*Height)/zoomy2;
 	a.x[1] = x;		a.y[1] = y;
 	a.zz[1] = f/fnt->GetFact(s&3);
 	a.style = s;	a.m = j;
@@ -210,7 +204,7 @@ void mglGraphPS::Glyph(mreal x, mreal y, mreal f, int s, long j, char col)
 	mglColor cc = mglColor(col);
 	if(!cc.Valid())	cc = mglColor(CDef[0],CDef[1],CDef[2]);
 	a.c[0] = cc.r;	a.c[1] = cc.g;	a.c[2] = cc.b;	a.c[3] = CDef[3];
-	add_prim(&a);
+	add_prim(a);
 }
 //-----------------------------------------------------------------------------
 void mglGraphPS::Clf(mglColor  Back)
@@ -226,15 +220,10 @@ void mglGraphPS::Clf(mglColor  Back)
 		BDef[1] = (unsigned char)(255*Back.g);
 		BDef[2] = (unsigned char)(255*Back.b);
 		if(TranspType==2)	Back = mglColor(0,0,0);
-/*		mglPrim a(3);	a.z = -1e7;
-		a.x[0]=0;		a.y[0]=0;	a.x[1]=0;		a.y[1]=Height;
-		a.x[2]=Width;	a.y[2]=0;	a.x[3]=Width;	a.y[3]=Height;
-		a.c[0]=Back.r;	a.c[1]=Back.g;	a.c[2]=Back.b;
-		add_prim(&a);*/
 	}
 }
 //-----------------------------------------------------------------------------
-void mglGraphPS::add_prim(mglPrim *a)
+void mglGraphPS::add_prim(mglPrim &a)
 {
 	if(!P)
 	{
@@ -247,56 +236,33 @@ void mglGraphPS::add_prim(mglPrim *a)
 		P = (mglPrim *)realloc(P, pMax*sizeof(mglPrim));
 	}
 	// fog
-	mreal zz = FogDist*(a->z/Depth-0.5-FogDz);
+	mreal zz = FogDist*(a.z/Depth-0.5-FogDz);
 	if(zz<0)
 	{
 		mreal d = 1-exp(5*zz);
 		if(255*d>=254)	return;	// not visible under fog
 		mreal cb[3] = {BDef[0]/255., BDef[1]/255., BDef[2]/255.}, b = 1-d;
-		a->c[0] = a->c[0]*b + cb[0]*d;	a->c[1] = a->c[1]*b + cb[1]*d;
-		a->c[2] = a->c[2]*b + cb[2]*d;	a->c[3] = a->c[3]*b + d;
+		a.c[0] = a.c[0]*b + cb[0]*d;	a.c[1] = a.c[1]*b + cb[1]*d;
+		a.c[2] = a.c[2]*b + cb[2]*d;	a.c[3] = a.c[3]*b + d;
 	}
 
-	a->c[0] = int(a->c[0]*100)*0.01;	a->c[1] = int(a->c[1]*100)*0.01;
-	a->c[2] = int(a->c[2]*100)*0.01;	a->c[3] = UseAlpha ? int(a->c[3]*500)*0.002 : 1;
-//	if(a->c[3]<0)	a->c[3] = 0;		if(a->c[3]>1)	a->c[3] = 1;
-	a->x[0] = int(a->x[0]*100)*0.01;	a->x[1] = int(a->x[1]*100)*0.01;
-	a->x[2] = int(a->x[2]*100)*0.01;	a->x[3] = int(a->x[3]*100)*0.01;
-	a->y[0] = int(a->y[0]*100)*0.01;	a->y[1] = int(a->y[1]*100)*0.01;
-	a->y[2] = int(a->y[2]*100)*0.01;	a->y[3] = int(a->y[3]*100)*0.01;
-	a->id = ObjId;
-	memcpy(P+pNum,a,sizeof(mglPrim));
+	a.c[0] = int(a.c[0]*100)*0.01;	a.c[1] = int(a.c[1]*100)*0.01;
+	a.c[2] = int(a.c[2]*100)*0.01;	a.c[3] = UseAlpha ? int(a.c[3]*500)*0.002 : 1;
+//	i(a->c[3]<0)	a->c[3] = 0;		if(a->c[3]>1)	a->c[3] = 1;
+	a.x[0] = int(a.x[0]*100)*0.01;	a.x[1] = int(a.x[1]*100)*0.01;
+	a.x[2] = int(a.x[2]*100)*0.01;	a.x[3] = int(a.x[3]*100)*0.01;
+	a.y[0] = int(a.y[0]*100)*0.01;	a.y[1] = int(a.y[1]*100)*0.01;
+	a.y[2] = int(a.y[2]*100)*0.01;	a.y[3] = int(a.y[3]*100)*0.01;
+	a.id = ObjId;	P[pNum]=a;
 	pNum++;		Finished = false;
 }
 //-----------------------------------------------------------------------------
 void mglGraphPS::add_light(mreal *b, mreal n0,mreal n1, mreal n2)
 {
-	if(UseLight && (n0*n0+n1*n1+n2*n2)!=0)
-	{
-		mreal d0,d1,d2,nn;
-		register long i,j;
-
-		b[0]*=AmbBr;	b[1]*=AmbBr;	b[2]*=AmbBr;
-		for(i=0;i<10;i++)
-		{
-			if(!nLight[i])	continue;
-			j = 3*i;
-			nn = 2*(n0*pLight[j]+n1*pLight[j+1]+n2*pLight[j+2]) /
-					(n0*n0+n1*n1+n2*n2);
-			d0 = pLight[j] - n0*nn;
-			d1 = pLight[j+1]-n1*nn;
-			d2 = pLight[j+2]-n2*nn;
-			nn = 1 + d2/sqrt(d0*d0+d1*d1+d2*d2);
-
-			nn = exp(-aLight[i]*nn)*bLight[i]*2;
-			b[0] += nn*cLight[j];
-			b[1] += nn*cLight[j+1];
-			b[2] += nn*cLight[j+2];
-		}
-		b[0] = b[0]<1 ? b[0] : 1;
-		b[1] = b[1]<1 ? b[1] : 1;
-		b[2] = b[2]<1 ? b[2] : 1;
-	}
+	mreal n[3]={n0,n1,n2};
+	unsigned char r[4];
+	col2int(b,n,r);
+	b[0]=r[0]/255.;	b[1]=r[1]/255.;	b[2]=r[2]/255.;	b[3]=r[3]/255.;
 }
 //-----------------------------------------------------------------------------
 char *mgl_get_dash(unsigned short d, mreal w)
@@ -649,352 +615,36 @@ void mglGraphPS::Finish()
 #define imin(a,b)	(a)<(b) ? (a) : (b)
 void mglPrim::Draw(mglGraphPS *gr)
 {
-	unsigned char r[4]={(unsigned char)(255*c[0]),(unsigned char)(255*c[1]),
-				(unsigned char)(255*c[2]),(unsigned char)(255*c[3])};
-
-	if(type==0)
-		gr->mark_plot(int(x[0]),int(y[0]),m,r);
-	else if(type==2 && c[3]>0)	trig(gr,x,y,r);
-	else if(type==3 && c[3]>0)	quad(gr,x,y,r);
-	else if(type==1)	line(gr,x,y,r);
-	else if(type==4)
-	{
-		mreal p[12], f=zz[1], xx=x[1], yy=y[1],xp[5],yp[5];
-		// setup B[]
-		mreal pf=gr->PlotFactor;
-		gr->Push();
-		gr->SetPosScale(x[0],y[0],z,s*gr->PlotFactor);
-		gr->RotateN(w,0,0,1);
-		if(style&8)		// draw over-/under-line
-		{
-			mreal dy = 0.004;
-			p[0]=xx;			p[1]=yy+dy;		p[2]=0;
-			p[3]=fabs(f)+xx;	p[4]=yy+dy;		p[5]=0;
-			p[6]=xx;			p[7]=yy-dy;		p[8]=0;
-			p[9]=fabs(f)+xx;	p[10]=yy-dy;	p[11]=0;
-			gr->PostScale(p,4);
-			xp[0]=p[0];	xp[1]=p[3];	xp[2]=p[6];	xp[3]=p[9];	xp[4]=p[0];
-			yp[0]=p[1];	yp[1]=p[4];	yp[2]=p[7];	yp[3]=p[10];yp[4]=p[1];
-			if(style&4)
-			{
-				line(gr,xp,yp,r);		line(gr,xp+1,yp+1,r);
-				line(gr,xp+3,yp+3,r);	line(gr,xp+2,xp+2,r);
-			}
-			else	quad(gr,xp,yp,r);
-		}
-		else if(style&4)	// draw glyph
-		{
-			long ik,ii,il=0;
-			int nl=gr->fnt->GetNl(style&3,m);
-			const short *ln=gr->fnt->GetLn(style&3,m);
-			if(ln && nl>0)	for(ik=0;ik<nl;ik++)
-			{
-				ii = 2*ik;
-				if(ln[ii]==0x3fff && ln[ii+1]==0x3fff)	// line breakthrough
-				{	il = ik+1;	continue;	}
-				else if(ik==nl-1 || (ln[ii+2]==0x3fff && ln[ii+3]==0x3fff))
-				{	// enclose the circle. May be in future this block should be commented
-					p[0]=f*ln[ii]+xx;	p[1]=f*ln[ii+1]+yy;	p[2]=0;	ii=2*il;
-					p[3]=f*ln[ii]+xx;	p[4]=f*ln[ii+1]+yy;	p[5]=0;
-				}
-				else
-				{	// normal line
-					p[0]=f*ln[ii]+xx;	p[1]=f*ln[ii+1]+yy;	p[2]=0;	ii+=2;
-					p[3]=f*ln[ii]+xx;	p[4]=f*ln[ii+1]+yy;	p[5]=0;
-				}
-				gr->PostScale(p,2);
-				xp[0]=p[0];	xp[1]=p[3];
-				yp[0]=p[1];	yp[1]=p[4];
-				line(gr,xp,yp,r);
-			}
-		}
-		else
-		{
-			long ik,ii;
-			int nt=gr->fnt->GetNt(style&3,m);
-			const short *tr=gr->fnt->GetTr(style&3,m);
-			if(tr && nt>0)	for(ik=0;ik<nt;ik++)
-			{
-				ii = 6*ik;	p[0]=f*tr[ii]+xx;	p[1]=f*tr[ii+1]+yy;	p[2]=0;
-				ii+=2;		p[3]=f*tr[ii]+xx;	p[4]=f*tr[ii+1]+yy;	p[5]=0;
-				ii+=2;		p[6]=f*tr[ii]+xx;	p[7]=f*tr[ii+1]+yy;	p[8]=0;
-				gr->PostScale(p,3);
-				xp[0]=p[0];	xp[1]=p[3];	xp[2]=p[6];
-				yp[0]=p[1];	yp[1]=p[4];	yp[2]=p[7];
-				trig(gr,xp,yp,r);
-			}
-		}
-		gr->Pop();
-		gr->PlotFactor=pf;
-	}
+	mreal pp[12]={x[0],y[0],z,	x[1],y[1],z,	x[2],y[2],z,	x[3],y[3],z};
+	gr->draw_prim(this, pp, c);
 }
 //-----------------------------------------------------------------------------
-void mglPrim::line(mglGraphPS *gr, mreal x[2], mreal y[2], unsigned char r[4])
+void mglGraphPS::draw_prim(mglPrim *pr, mreal *pp, mreal *c)
 {
-	long y1,x1,y2,x2;
-	mreal d1[2],dxu,dxv,dyu,dyv;
-	register mreal u,v,xx,yy;
-	register long i,j;
-		mreal b=(w<1?1:w), dd;
-
-	bool hor = fabs(x[1]-x[0])>fabs(y[1]-y[0]), tt;
-	d1[0] = x[1]-x[0];	d1[1] = y[1]-y[0];
-	dd = sqrt(d1[0]*d1[0] + d1[1]*d1[1]);
-	if(dd<1e-5)	return;		// points lies on the vertical line
-
-	dxv = d1[1]/dd;	dyv =-d1[0]/dd;
-	dxu = d1[0]/dd;	dyu = d1[1]/dd;
-	long dp = int(w+3.5);
-
-	if(hor)
+	ObjId = pr->id;	memcpy(CDef,c,4*sizeof(mreal));
+	bool ul=UseLight;	UseLight=false;
+	int m = pr->m, s = pr->style;
+	switch(pr->type)
 	{
-		x1 = imin(long(x[0]),long(x[1])) - dp;	// bounding box
-		x2 = imax(long(x[0]),long(x[1])) + dp;
-		for(i=x1;i<=x2;i++)
-		{
-			y1 = int(y[0]+(y[1]-y[0])*(i-x[0])/(x[1]-x[0])) - dp;
-			y2 = int(y[0]+(y[1]-y[0])*(i-x[0])/(x[1]-x[0])) + dp;
-			for(j=y1;j<=y2;j++)
-			{
-				xx = (i-x[0]);	yy = (j-y[0]);
-				u = dxu*xx+dyu*yy;	v = dxv*xx+dyv*yy;	v = v*v;
-				if(u<0)			{	v += u*u;			u = 0;	}
-				else if(u>dd)	{	v += (u-dd)*(u-dd);	u = dd;	}
-				if(v>b)		continue;
-				tt = dash & (1<<long(fmod(u/w, 16)));
-				if(!tt)		continue;
-				u /= dd;
-				r[3] = (unsigned char)(255.f*exp(-6.f*v/b));
-				gr->pnt_plot(i,j,r);
-			}
-		}
-	}
-	else
-	{
-		y1 = imin(long(y[0]),long(y[1])) - dp;
-		y2 = imax(long(y[0]),long(y[1])) + dp;
-		for(j=y1;j<=y2;j++)
-		{
-			x1 = int(x[0]+(x[1]-x[0])*(j-y[0])/(y[1]-y[0])) - dp;
-			x2 = int(x[0]+(x[1]-x[0])*(j-y[0])/(y[1]-y[0])) + dp;
-			for(i=x1;i<=x2;i++)
-			{
-				xx = (i-x[0]);	yy = (j-y[0]);
-				u = dxu*xx+dyu*yy;	v = dxv*xx+dyv*yy;	v = v*v;
-				if(u<0)			{	v += u*u;			u = 0;	}
-				else if(u>dd)	{	v += (u-dd)*(u-dd);	u = dd;	}
-				if(v>b)		continue;
-				tt = dash & (1<<long(fmod(u/w, 16)));
-				if(!tt)		continue;
-				u /= dd;
-				r[3] = (unsigned char)(255.f*exp(-6.f*v/b));
-				gr->pnt_plot(i,j,r);
-			}
-		}
+	case 0:	mglGraphAB::mark_plot(pp,pr->m);	break;
+	case 1:	mglGraphAB::line_plot(pp,pp+3,c,c);	break;
+	case 2:	mglGraphAB::trig_plot(pp,pp+3,pp+6,c,c,c);	break;
+	case 3:	mglGraphAB::quad_plot(pp,pp+3,pp+6,pp+9,c,c,c,c);	break;
+	case 4:
+		mreal pf=PlotFactor;	Push();
+		SetPosScale(pp[0],pp[1],pp[2],pr->s*PlotFactor);	RotateN(pr->w,0,0,1);
+		mglGraphAB::Glyph(pr->x[1],pr->y[1],pr->zz[1]*fnt->GetFact(s&3),s,m,0);
+		Pop();	PlotFactor=pf;
+		break;
 	}
+	UseLight=ul;
 }
 //-----------------------------------------------------------------------------
-void mglPrim::quad(mglGraphPS *gr, mreal x[4], mreal y[4], unsigned char r[4])
-{
-	long y1,x1,y2,x2;
-	mreal d1[2],d2[2], dxu,dxv,dyu,dyv;
-	register mreal u,v,xx,yy;
-	register long i,j,g;
-
-	x1 = imin(imin(long(x[0]),long(x[3])),imin(long(x[1]),long(x[2])));	// bounding box
-	y1 = imin(imin(long(y[0]),long(y[3])),imin(long(y[1]),long(y[2])));
-	x2 = imax(imax(long(x[0]),long(x[3])),imax(long(x[1]),long(x[2])));
-	y2 = imax(imax(long(y[0]),long(y[3])),imax(long(y[1]),long(y[2])));
-
-	d1[0] = x[1]-x[0];	d2[0] = x[2]-x[0];
-	d1[1] = y[1]-y[0];	d2[1] = y[2]-y[0];
-	dxu = d2[0]*d1[1] - d1[0]*d2[1];
-	if(fabs(dxu)<1e-5)	return;		// points lies on the same line
-	dyv =-d1[0]/dxu;	dxv = d1[1]/dxu;
-	dyu = d2[0]/dxu;	dxu =-d2[1]/dxu;
-
-	for(i=x1;i<=x2;i++)	for(j=y1;j<=y2;j++)
-	{
-		xx = (i-x[0]);	yy = (j-y[0]);
-		u = dxu*xx+dyu*yy;	v = dxv*xx+dyv*yy;
-		g = u<0 || v<0 || u+v>1;
-		if(g)	continue;
-		gr->pnt_plot(i,j,r);
-	}
-
-	d1[0] = x[1]-x[3];	d2[0] = x[2]-x[3];
-	d1[1] = y[1]-y[3];	d2[1] = y[2]-y[3];
-	dxu = d2[0]*d1[1] - d1[0]*d2[1];
-	if(fabs(dxu)<1e-5)	return;		// points lies on the same line
-	dyv =-d1[0]/dxu;	dxv = d1[1]/dxu;
-	dyu = d2[0]/dxu;	dxu =-d2[1]/dxu;
-
-	for(i=x1;i<=x2;i++)	for(j=y1;j<=y2;j++)
-	{
-		xx = (i-x[3]);	yy = (j-y[3]);
-		u = dxu*xx+dyu*yy;	v = dxv*xx+dyv*yy;
-		g = u<0 || v<0 || u+v>1;
-		if(g)	continue;
-		gr->pnt_plot(i,j,r);
-	}
-}
-//-----------------------------------------------------------------------------
-void mglPrim::trig(mglGraphPS *gr, mreal x[3], mreal y[3], unsigned char r[4])
-{
-	long y1,x1,y2,x2;
-	mreal d1[2],d2[2], dxu,dxv,dyu,dyv;
-	register mreal u,v,xx,yy;
-	register long i,j,g;
-
-	d1[0] = x[1]-x[0];	d2[0] = x[2]-x[0];
-	d1[1] = y[1]-y[0];	d2[1] = y[2]-y[0];
-	x1 = imin(imin(long(x[0]),long(x[2])),long(x[1]));	// bounding box
-	y1 = imin(imin(long(y[0]),long(y[2])),long(y[1]));
-	x2 = imax(imax(long(x[0]),long(x[2])),long(x[1]));
-	y2 = imax(imax(long(y[0]),long(y[2])),long(y[1]));
-
-	dxu = d2[0]*d1[1] - d1[0]*d2[1];
-	if(fabs(dxu)<1e-5)	return;		// points lies on the same line
-	dyv =-d1[0]/dxu;	dxv = d1[1]/dxu;
-	dyu = d2[0]/dxu;	dxu =-d2[1]/dxu;
-
-	for(i=x1;i<=x2;i++)	for(j=y1;j<=y2;j++)
-	{
-		xx = (i-x[0]);	yy = (j-y[0]);
-		u = dxu*xx+dyu*yy;	v = dxv*xx+dyv*yy;
-		g = u<0 || v<0 || u+v>1;
-		if(g)	continue;
-		gr->pnt_plot(i,j,r);
-	}
-}
-//-----------------------------------------------------------------------------
-void mglGraphPS::pnt_plot(int x,int y,unsigned char c[4])
+void mglGraphPS::pnt_plot(long x,long y, mreal, unsigned char c[4])
 {
 	long i0=x+Width*(Height-1-y);
 	if(x<0 || x>=Width || y<0 || y>=Height)	return;
-	combine(G4+4*i0,c);
-}
-//-----------------------------------------------------------------------------
-void mglGraphPS::mark_plot(int x,int y, char type, unsigned char cs[4])
-{
-//	unsigned char cs[4]={(unsigned char)(255*cs[0]), (unsigned char)(255*cs[1]),
-//						(unsigned char)(255*cs[2]), (unsigned char)(255*cs[3])};
-	mreal v;
-	register long i,j,ss = long(MarkSize*0.35*font_factor);
-	if(type=='.' || ss==0)
-	{
-		ss = long(3.5+fabs(PenWidth));
-		for(i=-ss;i<=ss;i++)	for(j=-ss;j<=ss;j++)
-		{
-			v = hypot(i,j)/fabs(PenWidth)/3;
-			cs[3] = (unsigned char)(cs[3]*exp(-6*v*v));
-			if(cs[3]==0)	continue;
-			pnt_plot(x+i,y+j,cs);
-		}
-	}
-	else
-	{
-		switch(type)
-		{
-		case '+':
-			for(i=-ss;i<=ss;i++)
-			{
-				pnt_plot(x+i, y,cs);
-				pnt_plot(x, y+i,cs);
-			}
-			break;
-		case 'x':
-			for(i=-ss;i<=ss;i++)
-			{
-				pnt_plot(x+i, y-i,cs);
-				pnt_plot(x+i, y+i,cs);
-			}
-			break;
-		case 's':
-//			ss = int(ss*0.9);
-			for(i=-ss;i<=ss;i++)
-			{
-				pnt_plot(x+i, y-ss,cs);
-				pnt_plot(x+i, y+ss,cs);
-				pnt_plot(x+ss, y-i,cs);
-				pnt_plot(x-ss, y+i,cs);
-			}
-			break;
-		case 'd':
-			ss = int(ss*1.1);
-			for(i=0;i<=ss;i++)
-			{
-				pnt_plot(x+i, y+ss-i,cs);
-				pnt_plot(x-i, y+ss-i,cs);
-				pnt_plot(x+i, y-ss+i,cs);
-				pnt_plot(x-i, y-ss+i,cs);
-			}
-			break;
-		case '*':
-			for(i=-ss;i<=ss;i++)
-			{
-				pnt_plot(x+i, y,cs);
-				pnt_plot(int(x+i*0.6), int(y+i*0.8),cs);
-				pnt_plot(int(x-i*0.6), int(y+i*0.8),cs);
-			}
-			break;
-		case '^':
-			ss = int(ss*1.1);
-			for(i=-ss;i<=ss;i++)	pnt_plot(x+i, y-ss/2,cs);
-			for(i=-(ss/2);i<=ss;i++)
-			{
-				j = ss-(i+(ss/2))*ss/(ss+(ss/2));
-				pnt_plot(x+j, y+i,cs);
-				pnt_plot(x-j, y+i,cs);
-			}
-			break;
-		case 'v':
-			ss = int(ss*1.1);
-			for(i=-ss;i<=ss;i++)	pnt_plot(x+i, y+ss/2,cs);
-			for(i=-ss;i<=(ss/2);i++)
-			{
-				j = (i+ss)*ss/(ss+(ss/2));
-				pnt_plot(x+j, y+i,cs);
-				pnt_plot(x-j, y+i,cs);
-			}
-			break;
-		case 'o':
-			for(i=-4*ss;i<=4*ss;i++)
-			{
-				pnt_plot(int(x+ss*cos(i*M_PI_4/ss)), int(y+ss*sin(i*M_PI_4/ss)),cs);
-			}
-			break;
-		case 'O':
-			for(i=-ss;i<=ss;i++)	for(j=-ss;j<=ss;j++)
-			{
-				if(i*i+j*j>=ss*ss)	continue;
-				pnt_plot(x+i, y+j,cs);
-			}
-			break;
-		case 'S':
-			for(i=long(-ss);i<=long(ss);i++)	for(j=long(-ss);j<=long(ss);j++)
-				pnt_plot(x+i,y+j,cs);
-			break;
-		case 'D':
-			ss = int(ss*1.1);
-			for(i=long(-ss);i<=long(ss);i++)	for(j=long(-ss);j<=long(ss);j++)
-				if(abs(i)+abs(j)<=long(ss))
-					pnt_plot(x+i,y+j,cs);
-			break;
-		case 'T':
-			ss = int(ss*1.1);
-			for(i=long(-ss);i<=long(ss);i++)	for(j=long(-ss/2);j<=long(ss);j++)
-				if(1.5*abs(i)+j<=long(ss))
-					pnt_plot(x+i,y+j,cs);
-			break;
-		case 'V':
-			ss = int(ss*1.1);
-			for(i=long(-ss);i<=long(ss);i++)	for(j=long(-ss);j<=long(ss/2);j++)
-				if(1.5*abs(i)-j<=long(ss))
-					pnt_plot(x+i,y+j,cs);
-			break;
-		}
-	}
+	combine(G4+4*i0,c);	OI[i0]=ObjId;
 }
 //-----------------------------------------------------------------------------
 void mglGraphPS::put_line(FILE *fp, long i, mreal wp, mreal *cp,int st, const char *ifmt, const char *nfmt, bool neg)
@@ -1085,3 +735,10 @@ void mglGraphPS::put_desc(FILE *fp, const char *pre, const char *ln1, const char
 	delete []g;		delete []s;
 }
 //-----------------------------------------------------------------------------
+/// Create mglGraph object in PostScript mode.
+HMGL mgl_create_graph_ps(int width, int height)
+{    return new mglGraphPS(width,height);	}
+/// Create mglGraph object in PostScript mode.
+uintptr_t mgl_create_graph_ps_(int *width, int *height)
+{    return uintptr_t(new mglGraphPS(*width,*height));	}
+//-----------------------------------------------------------------------------
