@@ -3,24 +3,27 @@
  * Copyright (C) 2007 Alexey Balakin <balakin@appl.sci-nnov.ru>            *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
- *   it under the terms of the GNU General Public License as published by  *
- *   the Free Software Foundation; either version 2 of the License, or     *
- *   (at your option) any later version.                                   *
+ *   it under the terms of the GNU Library General Public License as       *
+ *   published by the Free Software Foundation; either version 3 of the    *
+ *   License, or (at your option) any later version.                       *
  *                                                                         *
  *   This program is distributed in the hope that it will be useful,       *
  *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
  *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
  *   GNU General Public License for more details.                          *
  *                                                                         *
- *   You should have received a copy of the GNU General Public License     *
- *   along with this program; if not, write to the                         *
+ *   You should have received a copy of the GNU Library General Public     *
+ *   License along with this program; if not, write to the                 *
  *   Free Software Foundation, Inc.,                                       *
  *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
  ***************************************************************************/
+#include <stdio.h>
 #include <string.h>
 #include <stdlib.h>
+#include <time.h>
+#ifndef NO_PNG
 #include <png.h>
-
+#endif
 #ifdef HAVE_JPEG
 extern "C" {
 #include <jpeglib.h>
@@ -37,18 +40,19 @@ int mgl_png_save(const char *fname, int w, int h, unsigned char **p);
 int mgl_pnga_save(const char *fname, int w, int h, unsigned char **p);
 int mgl_jpeg_save(const char *fname, int w, int h, unsigned char **p);
 //-----------------------------------------------------------------------------
-unsigned char **mglGraph::GetRGBLines(long &w, long &h, unsigned char *&f, bool )
+unsigned char **mglGraph::GetRGBLines(long &, long &, unsigned char *&f, bool )
 {	f=0;	return 0;	}
 //-----------------------------------------------------------------------------
 void mglGraph::WriteSVG(const char *,const char *)	{}
 void mglGraph::WriteIDTF(const char *,const char *)	{}
-void mglGraph::VertexColor(bool enable)	{}
-void mglGraph::TextureColor(bool enable){}
-void mglGraph::Compression(bool enable)	{}
-void mglGraph::Unrotate(bool enable)	{}
-void mglGraph::BallIsPoint(bool enable)	{}
-void mglGraph::StartGroup ( const char *name )		{}
-void mglGraph::StartAutoGroup ( const char *name )	{}
+void mglGraph::DoubleSided(bool )	{}
+void mglGraph::VertexColor(bool )	{}
+void mglGraph::TextureColor(bool ){}
+void mglGraph::Compression(bool )	{}
+void mglGraph::Unrotate(bool )	{}
+void mglGraph::BallIsPoint(bool )	{}
+void mglGraph::StartGroup ( const char * )		{}
+void mglGraph::StartAutoGroup ( const char * )	{}
 void mglGraph::EndGroup()	{}
 //-----------------------------------------------------------------------------
 void mglGraph::WriteJPEG(const char *fname,const char *)
@@ -106,14 +110,15 @@ void mglGraph::WriteEPS(const char *fname,const char *)
 //-----------------------------------------------------------------------------
 int mgl_pnga_save(const char *fname, int w, int h, unsigned char **p)
 {
+#ifndef NO_PNG
 	FILE *fp = fopen(fname, "wb");
 	if (!fp)	return 1;
 
 	png_structp png_ptr = png_create_write_struct(PNG_LIBPNG_VER_STRING,0,0,0);
-	if (!png_ptr)	return 1;
+	if (!png_ptr)	{	fclose(fp);	return 1;	}
 	png_infop info_ptr = png_create_info_struct(png_ptr);
 	if (!info_ptr)
-	{	png_destroy_write_struct(&png_ptr,0);	return 1;	}
+	{	png_destroy_write_struct(&png_ptr,0);	fclose(fp);	return 1;	}
 
 	png_init_io(png_ptr, fp);
 	png_set_filter(png_ptr, 0, PNG_ALL_FILTERS);
@@ -128,19 +133,21 @@ int mgl_pnga_save(const char *fname, int w, int h, unsigned char **p)
 
 	png_destroy_write_struct(&png_ptr, &info_ptr);
 	fclose(fp);
+#endif
 	return 0;
 }
 //-----------------------------------------------------------------------------
 int mgl_png_save(const char *fname, int w, int h, unsigned char **p)
 {
+#ifndef NO_PNG
 	FILE *fp = fopen(fname, "wb");
 	if (!fp)	return 1;
 
 	png_structp png_ptr = png_create_write_struct(PNG_LIBPNG_VER_STRING,0,0,0);
-	if (!png_ptr)	return 1;
+	if (!png_ptr)	{	fclose(fp);	return 1;	}
 	png_infop info_ptr = png_create_info_struct(png_ptr);
 	if (!info_ptr)
-	{	png_destroy_write_struct(&png_ptr,0);	return 1;	}
+	{	png_destroy_write_struct(&png_ptr,0);	fclose(fp);	return 1;	}
 
 	png_init_io(png_ptr, fp);
 	png_set_filter(png_ptr, 0, PNG_ALL_FILTERS);
@@ -155,6 +162,7 @@ int mgl_png_save(const char *fname, int w, int h, unsigned char **p)
 
 	png_destroy_write_struct(&png_ptr, &info_ptr);
 	fclose(fp);
+#endif
 	return 0;
 }
 //-----------------------------------------------------------------------------
