@@ -3,17 +3,17 @@
  * Copyright (C) 2007 Alexey Balakin <balakin@appl.sci-nnov.ru>            *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
- *   it under the terms of the GNU General Public License as published by  *
- *   the Free Software Foundation; either version 2 of the License, or     *
- *   (at your option) any later version.                                   *
+ *   it under the terms of the GNU Library General Public License as       *
+ *   published by the Free Software Foundation; either version 3 of the    *
+ *   License, or (at your option) any later version.                       *
  *                                                                         *
  *   This program is distributed in the hope that it will be useful,       *
  *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
  *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
  *   GNU General Public License for more details.                          *
  *                                                                         *
- *   You should have received a copy of the GNU General Public License     *
- *   along with this program; if not, write to the                         *
+ *   You should have received a copy of the GNU Library General Public     *
+ *   License along with this program; if not, write to the                 *
  *   Free Software Foundation, Inc.,                                       *
  *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
  ***************************************************************************/
@@ -184,8 +184,8 @@ void mgl_set_ambbr_(uintptr_t *gr, mreal *i)
 //		Scale and rotate
 //-----------------------------------------------------------------------------
 /// Clear transformation matrix.
-void mgl_identity_(uintptr_t *gr)
-{	_GR_->Identity();	}
+void mgl_identity_(uintptr_t *gr, int *rel)
+{	_GR_->Identity(*rel);	}
 /// Clear up the frame
 void mgl_clf_(uintptr_t *gr)
 {	_GR_->Clf();	}
@@ -205,6 +205,8 @@ void mgl_relplot_(uintptr_t *gr, mreal *x1,mreal *x2,mreal *y1,mreal *y2)
 {	_GR_->InPlot(*x1,*x2,*y1,*y2,true);	}
 void mgl_columnplot_(uintptr_t *gr, int *num, int *i)
 {	_GR_->ColumnPlot(*num,*i);	}
+void mgl_stickplot_(uintptr_t *gr, int *num, int *i, mreal *tet, mreal *phi)
+{	_GR_->StickPlot(*num, *i, *tet, *phi);	}
 /// Set aspect ratio for further plotting.
 void mgl_aspect_(uintptr_t *gr, mreal *Ax,mreal *Ay,mreal *Az)
 {	_GR_->Aspect(*Ax,*Ay,*Az);	}
@@ -252,12 +254,21 @@ void mgl_set_func_(uintptr_t *gr, const char *EqX,const char *EqY,const char *Eq
 	char *sx=new char[lx+1];	memcpy(sx,EqX,lx);	sx[lx]=0;
 	char *sy=new char[ly+1];	memcpy(sy,EqY,ly);	sy[ly]=0;
 	char *sz=new char[lz+1];	memcpy(sz,EqZ,lz);	sz[lz]=0;
-	_GR_->Axis(sx,sy,sz);
+	_GR_->SetFunc(sx,sy,sz);
 	delete []sx;	delete []sy;	delete []sz;
 }
+void mgl_set_func_ext_(uintptr_t *gr, const char *EqX,const char *EqY,const char *EqZ,const char *EqA,int lx,int ly,int lz,int la)
+{
+	char *sx=new char[lx+1];	memcpy(sx,EqX,lx);	sx[lx]=0;
+	char *sy=new char[ly+1];	memcpy(sy,EqY,ly);	sy[ly]=0;
+	char *sz=new char[lz+1];	memcpy(sz,EqZ,lz);	sz[lz]=0;
+	char *sa=new char[la+1];	memcpy(sa,EqA,la);	sa[la]=0;
+	_GR_->SetFunc(sx,sy,sz,sa);
+	delete []sx;	delete []sy;	delete []sz;	delete []sa;
+}
 /// Set predefined coordinate system
 void mgl_set_coor_(uintptr_t *gr, int *how)
-{	_GR_->Axis(*how);	}
+{	_GR_->SetCoor(*how);	}
 void mgl_set_cutoff_(uintptr_t *gr, const char *EqC, int l)
 {
 	char *s=new char[l+1];	memcpy(s,EqC,l);	s[l]=0;
@@ -293,38 +304,36 @@ void mgl_label_(uintptr_t *gr, const char *dir, const char *text,int,int l)
 	_GR_->Label(*dir, s);	delete []s;
 }
 /// Print the label \a text for axis \a dir.
-void mgl_label_ext_(uintptr_t *gr, const char *dir, const char *text, int *pos, mreal *size, mreal *shift,int,int l)
+void mgl_label_ext_(uintptr_t *gr, const char *dir, const char *text, mreal *pos, mreal *size, mreal *shift,int,int l)
 {
 	char *s=new char[l+1];	memcpy(s,text,l);	s[l]=0;
 	_GR_->Label(*dir, s, *pos, *size, *shift);	delete []s;
 }
-void mgl_set_xtt_(uintptr_t * gr, const char *templ, int n)
+void mgl_label_xy_(uintptr_t *gr, mreal *x, mreal *y, const char *txt, const char *fnt, mreal *size,int l,int n)
 {
-	static wchar_t t[256];
-	memset(t,0,256*sizeof(wchar_t));
-	if(n<256)
-	{	mbstowcs(t,templ,n);	_GR_->xtt = t;	}
+	char *s=new char[l+1];	memcpy(s,txt,l);	s[l]=0;
+	char *p=new char[n+1];	memcpy(p,fnt,n);	p[n]=0;
+	_GR_->Label(*x,*y,s,p,*size);	delete []s;	delete []p;
 }
-void mgl_set_ytt_(uintptr_t * gr, const char *templ, int n)
+void mgl_set_xtt_(uintptr_t * gr, const char *templ, int l)
 {
-	static wchar_t t[256];
-	memset(t,0,256*sizeof(wchar_t));
-	if(n<256)
-	{	mbstowcs(t,templ,n);	_GR_->ytt = t;	}
+	char *s=new char[l+1];	memcpy(s,templ,l);	s[l]=0;
+	_GR_->SetXTT(s);	delete []s;
 }
-void mgl_set_ztt_(uintptr_t * gr, const char *templ, int n)
+void mgl_set_ytt_(uintptr_t * gr, const char *templ, int l)
 {
-	static wchar_t t[256];
-	memset(t,0,256*sizeof(wchar_t));
-	if(n<256)
-	{	mbstowcs(t,templ,n);	_GR_->ztt = t;	}
+	char *s=new char[l+1];	memcpy(s,templ,l);	s[l]=0;
+	_GR_->SetYTT(s);	delete []s;
 }
-void mgl_set_ctt_(uintptr_t * gr, const char *templ, int n)
+void mgl_set_ztt_(uintptr_t * gr, const char *templ, int l)
 {
-	static wchar_t t[256];
-	memset(t,0,256*sizeof(wchar_t));
-	if(n<256)
-	{	mbstowcs(t,templ,n);	_GR_->ctt = t;	}
+	char *s=new char[l+1];	memcpy(s,templ,l);	s[l]=0;
+	_GR_->SetZTT(s);	delete []s;
+}
+void mgl_set_ctt_(uintptr_t * gr, const char *templ, int l)
+{
+	char *s=new char[l+1];	memcpy(s,templ,l);	s[l]=0;
+	_GR_->SetCTT(s);	delete []s;
 }
 //-----------------------------------------------------------------------------
 //		Simple drawing
@@ -481,7 +490,7 @@ void mgl_get_last_mouse_pos_(uintptr_t *gr, mreal *x, mreal *y, mreal *z)
 {
 	mglGraphAB *g = dynamic_cast<mglGraphAB *>((mglGraph *)(*gr));
 	if(g)
-	{	*x=g->LastMousePos.x;	*y=g->LastMousePos.y;	*y=g->LastMousePos.y;}
+	{	*x=g->LastMousePos.x;	*y=g->LastMousePos.y;	*z=g->LastMousePos.z;}
 }
 void mgl_calc_xyz_(uintptr_t *gr, int *xs, int *ys, mreal *x, mreal *y, mreal *z)
 {
@@ -492,6 +501,11 @@ void mgl_calc_xyz_(uintptr_t *gr, int *xs, int *ys, mreal *x, mreal *y, mreal *z
 		*x = p.x;	*y = p.y;	*z = p.z;
 	}
 }
+void mgl_calc_scr_(uintptr_t *gr, mreal *x, mreal *y, mreal *z, int *xs, int *ys)
+{
+	mglGraphAB *g = dynamic_cast<mglGraphAB *>((mglGraph *)(*gr));
+	if(g)	g->CalcScr(mglPoint(*x,*y,*z),xs,ys);
+}
 //-----------------------------------------------------------------------------
 mreal mgl_data_get_value_(uintptr_t *d, int *i, int *j, int *k)
 {	return	_DT_->a[*i+_DT_->nx*(*j+*k*_DT_->ny)];	}
