@@ -3,17 +3,17 @@
  * Copyright (C) 2007 Alexey Balakin <balakin@appl.sci-nnov.ru>            *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
- *   it under the terms of the GNU General Public License as published by  *
- *   the Free Software Foundation; either version 2 of the License, or     *
- *   (at your option) any later version.                                   *
+ *   it under the terms of the GNU Library General Public License as       *
+ *   published by the Free Software Foundation; either version 3 of the    *
+ *   License, or (at your option) any later version.                       *
  *                                                                         *
  *   This program is distributed in the hope that it will be useful,       *
  *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
  *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
  *   GNU General Public License for more details.                          *
  *                                                                         *
- *   You should have received a copy of the GNU General Public License     *
- *   along with this program; if not, write to the                         *
+ *   You should have received a copy of the GNU Library General Public     *
+ *   License along with this program; if not, write to the                 *
  *   Free Software Foundation, Inc.,                                       *
  *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
  ***************************************************************************/
@@ -135,7 +135,7 @@ mreal mglFont::Puts(const wchar_t *str,int font,int align, char col)
 	if(parse)
 	{
 		unsigned *wcs = new unsigned[size];
-		memcpy(wcs,str,size*sizeof(unsigned));
+		memcpy(wcs,str,size*sizeof(wchar_t));
 		Convert(str, wcs);
 		ww = w = Puts(wcs,0,0,1.f,0x10|font,col);	// find width
 		Puts(wcs,-w*(align&3)/2.f,-h,1.f,font,col);	// draw it really
@@ -176,7 +176,7 @@ mreal mglFont::Width(const wchar_t *str,int font)
 	if(parse)
 	{
 		unsigned *wcs = new unsigned[size];
-		memcpy(wcs,str,size*sizeof(unsigned));
+		memcpy(wcs,str,size*sizeof(wchar_t));
 		Convert(str, wcs);
 		w = Puts(wcs,0,0,1.,0x10+font,'k');
 		delete []wcs;
@@ -271,6 +271,8 @@ unsigned mglFont::Parse(const wchar_t *s)
 	else if(!wcscmp(s,L"underset"))	res = unsigned(-9);
 	else if(!wcscmp(s,L"stackr"))	res = unsigned(-10);
 	else if(!wcscmp(s,L"stackl"))	res = unsigned(-11);
+	else if(!wcscmp(s,L"sub"))		res = unsigned(-12);	// new
+	else if(!wcscmp(s,L"sup"))		res = unsigned(-13);	// new
 	else if(!wcscmp(s,L"b"))		res = MGL_FONT_BOLD;
 	else if(!wcscmp(s,L"i"))		res = MGL_FONT_ITAL;
 	else if(!wcscmp(s,L"bi"))		res = MGL_FONT_BOLD|MGL_FONT_ITAL;
@@ -371,7 +373,8 @@ void mglFont::draw_ouline(mglGraph *gr, int st, mreal x, mreal y, mreal f, mreal
 mreal mglFont::Puts(const unsigned *text, mreal x,mreal y,mreal f,int style,char col)
 {
 	if(numg==0)	return 0;
-	register long i,j,k;
+	register long j,k;
+	long i;
 	register unsigned s,ss;
 	mreal w=0;				// string width
 	int st = style;			// current style
@@ -419,6 +422,24 @@ mreal mglFont::Puts(const unsigned *text, mreal x,mreal y,mreal f,int style,char
 				draw_ouline(gr,st,x,y,f,fact[a],ww,ccol);
 			st = style;	yy = y;	ff = f;		// clear style modification
 		}
+		else if(s==unsigned(-12))	// sub
+		{
+			ww = get_ptr(i, str, &b1, &b2, w1, w2, ff, ff/4, st);
+			Puts(b1, x+(ww-w1)/2, yy, ff, (st&(~MGL_FONT_OLINE)&(~MGL_FONT_ULINE)), ccol);
+			Puts(b2, x+(ww-w2)/2, yy-250*ff/fact[a], ff/4, (st&(~MGL_FONT_OLINE)&(~MGL_FONT_ULINE)), ccol);
+			if(gr && !(style&0x10))	// add under-/over- line now
+				draw_ouline(gr,st,x,y,f,fact[a],ww,ccol);
+			st = style;	yy = y;	ff = f;		// clear style modification
+		}
+		else if(s==unsigned(-13))	// sup
+		{
+			ww = get_ptr(i, str, &b1, &b2, w1, w2, ff, ff/4, st);
+			Puts(b1, x+(ww-w1)/2, yy, ff, (st&(~MGL_FONT_OLINE)&(~MGL_FONT_ULINE)), ccol);
+			Puts(b2, x+(ww-w2)/2, yy+450*ff/fact[a], ff/4, (st&(~MGL_FONT_OLINE)&(~MGL_FONT_ULINE)), ccol);
+			if(gr && !(style&0x10))	// add under-/over- line now
+				draw_ouline(gr,st,x,y,f,fact[a],ww,ccol);
+			st = style;	yy = y;	ff = f;		// clear style modification
+		}
 		else if(s==unsigned(-11))	// stackl
 		{
 			ww = get_ptr(i, str, &b1, &b2, w1, w2, ff*0.45, ff*0.45, st);
@@ -577,7 +598,7 @@ bool mglFont::read_main(const char *base, const char *path, unsigned &cur)
 	int tmpi, tmpw, tmpnl, tmpnt;
 	unsigned s, tmppl, tmppt;
 #ifdef WIN32	// normal weight (should have most of symbols !!!)
-	if(path)	sprintf(str,"%s\%s.vfm",path,base);
+	if(path)	sprintf(str,"%s\\%s.vfm",path,base);
 #else
 	if(path)	sprintf(str,"%s/%s.vfm",path,base);
 #endif
@@ -638,6 +659,9 @@ bool mglFont::Load(const char *base, const char *path)
 {
 //	base = 0;
 	char *buf=0,sep='/';
+#ifdef WIN32
+	sep='\\';
+#endif
 	FILE *fp;
 	char str[256];
 	setlocale(LC_NUMERIC,"C");
@@ -645,9 +669,6 @@ bool mglFont::Load(const char *base, const char *path)
 	if(!path)	path = MGL_FONT_PATH;
 	if(base)
 	{
-#ifdef WIN32
-		sep='\\';
-#endif
 		buf = new char[strlen(base)+1];
 		strcpy(buf,base);
 		if(strchr(buf,sep))
@@ -663,7 +684,7 @@ bool mglFont::Load(const char *base, const char *path)
 	{	read_def(cur);	if(buf)	delete []buf;	return true;	}
 
 	//================== bold ===========================================
-	sprintf(str,"%s/%s_b.vfm",path,base);
+	sprintf(str,"%s%c%s_b.vfm",path,sep,base);
 	fp = fopen(str,"rt");
 	if(fp)		// this file may absent
 	{
@@ -678,12 +699,12 @@ bool mglFont::Load(const char *base, const char *path)
 	}
 
 	//================== italic =========================================
-	sprintf(str,"%s/%s_i.vfm",path,base);
+	sprintf(str,"%s%c%s_i.vfm",path,sep,base);
 	fp = fopen(str,"rt");
 	if(fp)	read_data(fp, fact+2, width[2], numl[2], ln[2], numt[2], tr[2], cur);
 
 	//================== bold-italic ====================================
-	sprintf(str,"%s/%s_bi.vfm",path,base);
+	sprintf(str,"%s%c%s_bi.vfm",path,sep,base);
 	fp = fopen(str,"rt");
 	if(fp)	read_data(fp, fact+3, width[3], numl[3], ln[3], numt[3], tr[3], cur);
 	numb = cur;
@@ -698,8 +719,10 @@ bool mglFont::Load(const char *base, const char *path)
 mglFont::mglFont(const char *name, const char *path)
 {
 	parse = true;	numg=0;	gr=0;
-	if(this==&mglDefFont)	Load(name, path);
-	else	Copy(&mglDefFont);
+//	if(this==&mglDefFont)	Load(name, path);	else	Copy(&mglDefFont);
+	if(this!=&mglDefFont)	Copy(&mglDefFont);
+	if(this==&mglDefFont && (!name || name[0]==0))	Load(MGL_DEF_FONT_NAME,0);
+	else if(name && name[0])	Load(name, path);
 }
 mglFont::~mglFont()	{	Clear();	}
 void mglFont::Restore()	{	Copy(&mglDefFont);	}
