@@ -3,22 +3,27 @@
  * Copyright (C) 2007 Alexey Balakin <balakin@appl.sci-nnov.ru>            *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
- *   it under the terms of the GNU General Public License as published by  *
- *   the Free Software Foundation; either version 2 of the License, or     *
- *   (at your option) any later version.                                   *
+ *   it under the terms of the GNU Library General Public License as       *
+ *   published by the Free Software Foundation; either version 3 of the    *
+ *   License, or (at your option) any later version.                       *
  *                                                                         *
  *   This program is distributed in the hope that it will be useful,       *
  *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
  *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
  *   GNU General Public License for more details.                          *
  *                                                                         *
- *   You should have received a copy of the GNU General Public License     *
- *   along with this program; if not, write to the                         *
+ *   You should have received a copy of the GNU Library General Public     *
+ *   License along with this program; if not, write to the                 *
  *   Free Software Foundation, Inc.,                                       *
  *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
  ***************************************************************************/
 #include <stdlib.h>
+#ifdef WIN32
+#include <io.h>
+#include <direct.h>
+#else
 #include <unistd.h>
+#endif
 #include <stdarg.h>
 #include <wchar.h>
 #include "mgl/mgl.h"
@@ -137,14 +142,22 @@ void mglGraph::Axis(mglPoint m1, mglPoint m2, mglPoint org)
 	RecalcBorder();
 }
 //-----------------------------------------------------------------------------
-void mglGraph::Axis(const char *EqX,const char *EqY,const char *EqZ)
+void mglGraph::SetFunc(const char *EqX,const char *EqY,const char *EqZ,const char *EqA)
 {
-	if(fx)	delete fx;
-	if(fy)	delete fy;
-	if(fz)	delete fz;
-	if(EqX && EqX[0])	fx = new mglFormula(EqX);	else	fx = 0;
-	if(EqY && EqY[0])	fy = new mglFormula(EqY);	else	fy = 0;
-	if(EqZ && EqZ[0])	fz = new mglFormula(EqZ);	else	fz = 0;
+	if(fa)	delete fa;	if(fx)	delete fx;
+	if(fy)	delete fy;	if(fz)	delete fz;
+	if(EqX && EqX[0] && (EqX[0]!='x' || EqX[1]!=0))
+		fx = new mglFormula(EqX);
+	else	fx = 0;
+	if(EqY && EqY[0] && (EqY[0]!='y' || EqY[1]!=0))
+		fy = new mglFormula(EqY);
+	else	fy = 0;
+	if(EqZ && EqZ[0] && (EqZ[0]!='z' || EqZ[1]!=0))
+		fz = new mglFormula(EqZ);
+	else	fz = 0;
+	if(EqA && EqA[0] && ((EqA[0]!='c' && EqA[0]!='a') || EqA[1]!=0))
+		fa = new mglFormula(EqA);
+	else	fa = 0;
 	RecalcBorder();
 }
 //-----------------------------------------------------------------------------
@@ -216,12 +229,15 @@ void mglGraph::Color(mreal a,mreal a1,mreal a2)
 	DefColor(c,AlphaDef);
 }
 //-----------------------------------------------------------------------------
-void mglGraph::SetScheme(const char *s)
+void mglGraph::SetScheme(const char *s, bool face)
 {
 	unsigned i;
 	mglColor c;
 	const char *col = "kwrgbcymhWRGBCYMHlenpquLENPQU";
-	if(!s || s[0]==0 || (!strchr(col,s[0]) && s[1]==0) || s[0]==':' )	return;
+	if(!s || s[0]==0 || s[0]==':')		return;
+	if(face && s[0]=='|' && s[1]==0)	{	SmoothColorbar=false;	return;	}
+	for(;*s;s++)	if(strchr(col,*s))	break;
+	if(s[0]==0)	return;
 	strcpy(last_style, s);
 	NumCol = 0;
 	for(i=0;i<NUM_COLOR;i++)	cmap[i] = NC;
@@ -229,9 +245,9 @@ void mglGraph::SetScheme(const char *s)
 	OnCoord = false;	SmoothColorbar = true;
 	for(i=0;i<strlen(s);i++)
 	{
-		if(s[i]=='d')	OnCoord = true;
-		if(s[i]=='|')	SmoothColorbar = false;
-		else if(s[i]==':')	break;
+		if(face && s[i]=='d')	OnCoord = true;
+		if(face && s[i]=='|')	SmoothColorbar = false;
+		else if(face && s[i]==':')	break;
 		else if(strchr(col,s[i]))
 		{
 			if(s[i+1]>='1' && s[i+1]<='9')
@@ -269,8 +285,15 @@ mglColor mglGraph::GetC(mreal x,mreal y,mreal z,bool simple)
 //-----------------------------------------------------------------------------
 mreal mglGraph::GetA(mreal a)
 {
-	a = (2*a-Cmin-Cmax)/(Cmax-Cmin);
-	a = fabs(a)>1 ? copysignf(1,a) : a;
+	mreal Amin=Cmin, Amax=Cmax;
+	if(fa)
+	{
+		Amin = fa->Calc(0,0,0,Cmin);
+		Amax = fa->Calc(0,0,0,Cmax);
+		a = fa->Calc(0,0,0,a);
+	}
+	a = (2*a-Amin-Amax)/(Amax-Amin);
+	a = a<1?(a>-1?a:-1):1;
 	return a;
 }
 //-----------------------------------------------------------------------------
@@ -390,12 +413,12 @@ char mglGraph::SelectPen(const char *p)
 {
 	mglColor c = NC;
 	int w=1;
-	char st='-',mk=0;
+	char st='-',mk=0,cc=0;
 
 	Arrow1 = Arrow2 = '-';
 	if(p && *p!=0)
 	{
-		strcpy(last_style, p);
+//		strcpy(last_style, p);
 		const char *col = "wkrgbcymhRGBCYMHWlenuqpLENUQP";
 		const char *stl = " -|;:ji";
 		const char *mrk = "*o+xsd.^v";
@@ -406,7 +429,7 @@ char mglGraph::SelectPen(const char *p)
 			if(strchr(stl,p[i]))	st  = p[i];
 			else if(strchr(mrk,p[i]))	mk = p[i];
 			else if(strchr(wdh,p[i]))	w = p[i]-'0';
-			else if(strchr(col,p[i]))	c.Set(p[i]);
+			else if(strchr(col,p[i]))	{	cc=p[i];	c.Set(p[i]);	}
 			else if(strchr(arr,p[i]))
 			{
 				if(Arrow1=='-')	Arrow1 = p[i];
@@ -427,12 +450,37 @@ char mglGraph::SelectPen(const char *p)
 			if(mk=='v')	mk = 'V';
 		}
 	}
-	else
-	{	last_style[0] = PalNames[(CurrPal+1)%NumPal];	last_style[1] = 0;	}
 	Pen(c, st, BaseLineWidth*w);
+	last_style[0]=cc?cc:(NumPal?PalNames[(CurrPal+1)%NumPal]:'k');
+	last_style[1]=st;	last_style[2]=w+'0';	last_style[3]=mk;	last_style[4]=0;
 	return mk;
 }
 //-----------------------------------------------------------------------------
+void mglGraph::SetPal(const char *colors)
+{
+	if(!colors || !colors[0])	colors = DefPal;	// restore default
+	memset(Pal,0,100*sizeof(mglColor));
+	memset(PalNames,0,101*sizeof(char));
+	int i,n = strlen(colors),k;
+	n = n<100? n:100;
+	for(i=k=0;i<n;i++)
+	{
+		if(!strchr("wkrgbcymhRGBCYMHWlenuqpLENUQP",colors[i]))	continue;
+		PalNames[k] = colors[i];	Pal[k] = mglColor(colors[i]);	k++;
+	}
+	NumPal = k;	//	CurrPal = 0;
+	if(k==0)	SetPal(0);	// if no colors then set default palette
+}
+//-----------------------------------------------------------------------------
+mglColor mglGraph::GetPal()
+{
+//	const char *col = "wkrgbcymhRGBCYMHWlenuqpLENUQP";
+	CurrPal = (CurrPal+1)%NumPal;
+//	if(*last_style==0)	{	last_style[0] = PalNames[CurrPal];	last_style[1]=0;	}
+//	else for
+	return 	Pal[CurrPal];
+}
+//-----------------------------------------------------------------------------
 void mglGraph::SubPlot(int nx,int ny,int m, mreal dx, mreal dy)
 {
 	mreal x1,x2,y1,y2;
@@ -495,44 +543,26 @@ void mglColor::Set(char p, mreal bright)
 		{	Set(mglColorIds[i].col, bright);	break;	}
 }
 //-----------------------------------------------------------------------------
-void mglGraph::ball(mreal *p,mreal *c)
-{
-	Ball(p[0],p[1],p[2],mglColor(c[0],c[1],c[2]),c[3]);
-}
+void mglGraph::ball(mreal *p, mreal *c)
+{	Ball(p[0],p[1],p[2],mglColor(c[0],c[1],c[2]),c[3]);	}
 //-----------------------------------------------------------------------------
 void mglGraph::ClearEq()
 {
-	if(fx)	delete fx;	if(fy)	delete fy;	if(fz)	delete fz;	if(fc)	delete fc;
-	fx = fy = fz = fc = 0;
+	if(fx)	delete fx;	if(fy)	delete fy;	if(fz)	delete fz;
+	if(fa)	delete fa;	if(fc)	delete fc;
+	fx = fy = fz = fc = fa = 0;
 	RecalcBorder();
 }
 //-----------------------------------------------------------------------------
-void mglGraph::SetPal(const char *colors)
-{
-	if(!colors || !colors[0])	colors = MGL_DEF_PAL;	// restore default
-	memset(Pal,0,100*sizeof(mglColor));
-	memset(PalNames,0,101*sizeof(char));
-	int i,n = strlen(colors),k;
-	n = n<100? n:100;
-	for(i=k=0;i<n;i++)
-	{
-		if(!strchr("wkrgbcymhRGBCYMHWlenuqpLENUQP",colors[i]))	continue;
-		PalNames[k] = colors[i];	Pal[k] = mglColor(colors[i]);	k++;
-	}
-	NumPal = k;	//	CurrPal = 0;
-	if(k==0)	SetPal(0);	// if no colors then set default palette
-}
-//-----------------------------------------------------------------------------
 void mglGraph::DefaultPlotParam()
 {
 	FontSize = 5;			BaseLineWidth = 1;
 	Ambient();				Ternary(false);
 	PlotId = "frame";		SelectPen("k-1");
 	SetScheme("BbcyrR");	SetPalette(MGL_DEF_PAL);
-	Cut = true;				InPlot(0,1,0,1);
-	SetTicks('x');	SetTicks('y');	SetTicks('z');
+	SetTicks('x');	SetTicks('y');	SetTicks('z');	SetTicks('c');
 	Axis(mglPoint(-1,-1,-1), mglPoint(1,1,1));
-	Axis(0,0,0);	CutOff(0);		Zoom(0,0,1,1);
+	SetFunc(0,0);			CutOff(0);
 	SetWarn(mglWarnNone);	Message = 0;
 	BarWidth = 0.7;			fit_res[0] = 0;
 	MarkSize = 0.02;		ArrowSize = 0.03;
@@ -544,19 +574,20 @@ void mglGraph::DefaultPlotParam()
 	ClearLegend();			LegendBox = true;
 	CutMin=mglPoint(0,0,0);	CutMax=mglPoint(0,0,0);
 	AutoOrg = true;			//CurFrameId = 0;
-	CirclePnts = 40;		FitPnts = 100;
+	CirclePnts=40;	FitPnts=100;	GridPnts=50;
 	DrawFace = true;		_tetx=_tety=_tetz=0;
 	TuneTicks= true;		_sx=_sy=_sz = 1;
-	Alpha(false);			Fog(0);
-	xtt=ytt=ztt=ctt=0;		FactorPos = 1.15;
+	Alpha(false);	Fog(0);	FactorPos = 1.15;
+	xtt[0]=ytt[0]=ztt[0]=ctt[0]=0;
 	AutoPlotFactor = true;	ScalePuts = true;
-	TickLen = 0.1;			st_t = 1;
-	TickStl[0] = SubTStl[0] = 'k';
+	TickLen = 0.1;	st_t = 1;	Cut = true;
+	TickStl[0] = SubTStl[0] = '-';
 	TickStl[1] = SubTStl[1] = 0;
 	PlotFactor = AutoPlotFactor ? 1.55f :2.f;
 	for(int i=0;i<10;i++)
 	{	Light(i, mglPoint(0,0,1));	Light(i,false);	}
-	Light(0,true);			Light(false);
+	Light(0,true);		Light(false);
+	InPlot(0,1,0,1);	Zoom(0,0,1,1);
 }
 //-----------------------------------------------------------------------------
 void mglGraph::SetTickStl(const char *stl, const char *sub)
@@ -596,7 +627,7 @@ void mglGraph::SimplePlot(const mglData &a, int type, const char *stl)
 	{
 	case 1:		DensA(a,stl);	break;
 	case 2:		ContA(a,stl);	break;
-	case 3:		CloudQ(a,stl);	break;
+	case 3:		Cloud(a,stl);	break;
 	case 4:		CloudP(a,stl);	break;
 	default:	Surf3(a,stl);	break;
 	}
@@ -765,7 +796,7 @@ void mglGraph::Mark(mglPoint p,char t)
 //-----------------------------------------------------------------------------
 void mglGraph::View(mreal tetx,mreal tetz,mreal tety)
 {	_tetx=tetx;	_tety=tety;	_tetz=tetz;	}
-void mglGraph::Identity()	{	InPlot(0,1,0,1);	}
+void mglGraph::Identity(bool rel)	{	InPlot(0,1,0,1,rel);	}
 //-----------------------------------------------------------------------------
 void mglGraph::Rotate(mreal TetX,mreal TetZ,mreal TetY)
 {
@@ -834,12 +865,37 @@ void mglGraph::Title(const char *str,const char *font,mreal size)
 	delete []wcs;
 }
 //-----------------------------------------------------------------------------
-void mglGraph::Label(char dir, const char *str, int pos, mreal size, mreal shift)
+void mglGraph::Labelw(mreal x, mreal y, const wchar_t *text, const char *fnt, mreal size, bool rel)
+{
+	Identity(rel);
+	mglFormula *ox=fx, *oy=fy, *oz=fz;
+	fx = fy = fz = NULL;
+	char *f = new char[strlen(fnt)+1];
+	strcpy(f,fnt);
+	for(int i=0;f[i];i++)	if(f[i]=='a' || f[i]=='A')	f[i]=' ';
+	Text(mglPoint((Min.x+Max.x)/2+PlotFactor*(Max.x-Min.x)*(x-0.5),
+				(Min.y+Max.y)/2+PlotFactor*(Max.y-Min.y)*(y-0.5),
+				Max.z), text, f, size);
+	delete []f;
+	fx=ox;	fy=oy;	fz=oz;
+	RestoreM();
+}
+//-----------------------------------------------------------------------------
+void mglGraph::Label(mreal x, mreal y, const char *str, const char *fnt, mreal size, bool rel)
 {
 	unsigned s = strlen(str)+1;
 	wchar_t *wcs = new wchar_t[s];
 	mbstowcs(wcs,str,s);
-	Label(dir, wcs, pos, size, shift);
+	Labelw(x,y,wcs, fnt, size, rel);
+	delete []wcs;
+}
+//-----------------------------------------------------------------------------
+void mglGraph::Label(char dir, const char *str, mreal pos, mreal size, mreal shift)
+{
+	unsigned s = strlen(str)+1;
+	wchar_t *wcs = new wchar_t[s];
+	mbstowcs(wcs,str,s);
+	Labelw(dir, wcs, pos, size, shift);
 	delete []wcs;
 }
 //-----------------------------------------------------------------------------
@@ -1018,19 +1074,11 @@ void mglGraph::SetAutoRanges(mreal x1, mreal x2, mreal y1, mreal y2, mreal z1, m
 void mglGraph::Colorbar(const char *sch,int where)
 {
 	SetScheme(sch);
-	mreal x=0,y=0;
-	if(where==2)	y=1;
-	if(where==0)	x=1;
-	Colorbar(where,x,y,1,1);
+	Colorbar(where, where==0?1:0, where==2?1:0, 1, 1);
 }
 //-----------------------------------------------------------------------------
 void mglGraph::Colorbar(const mglData &v, const char *sch,int where)
-{
-	mreal x=0,y=0;
-	if(where==2)	y=1;
-	if(where==0)	x=1;
-	Colorbar(v,sch,where,x,y,1,1);
-}
+{	Colorbar(v,sch,where, where==0?1:0, where==2?1:0, 1, 1);	}
 //-----------------------------------------------------------------------------
 void mglGraph::Colorbar(int where, mreal x, mreal y, mreal w, mreal h)
 {
@@ -1038,9 +1086,14 @@ void mglGraph::Colorbar(int where, mreal x, mreal y, mreal w, mreal h)
 	d = floor(d*pow(10,-floor(log10(d))));
 	long n = 50*(d<4?int(2*d+0.5):int(d+0.5))+1;
 	if(d==1.f)	n = 101;
-	mglData v(n);	v.Fill(Cmin,Cmax);
+	mglData v(n);
+	if(dc)	v.Fill(Cmin,Cmax);
+	else if(Cmax>Cmin && Cmin>0)
+	{	v.Fill(log(Cmin), log(Cmax));	v.Modify("exp(u)");		}
+	else if(Cmin<Cmax && Cmax<0)
+	{	v.Fill(log(-Cmin), log(-Cmax));	v.Modify("-exp(u)");	}
 	mglColor *c=new mglColor[n];
-	for(long i=0;i<n;i++)	c[i] = GetC(v.a[i],true);
+	for(long i=0;i<n;i++)	c[i] = GetC(v.a[i]);
 	colorbar(v, c, where, x, y, w, h);
 	delete []c;
 }
@@ -1061,34 +1114,35 @@ void mglGraph::ColumnPlot(int num, int i)
 	InPlot(0,1,d,d+w,true);
 }
 //-----------------------------------------------------------------------------
-void mglGraph::Axis(int how)
+void mglGraph::SetCoor(int how)
 {
 	switch(how)
 	{
-	case mglCartesian:	Axis(0,0,0);	break;
+	case mglCartesian:	SetFunc(0,0);	break;
 	case mglPolar:
-		Axis("x*cos(y)","x*sin(y)",0);	break;
+		SetFunc("x*cos(y)","x*sin(y)");	break;
 	case mglSpherical:
-		Axis("x*sin(y)*cos(z)","x*sin(y)*sin(z)","x*cos(y)");	break;
+		SetFunc("x*sin(y)*cos(z)","x*sin(y)*sin(z)","x*cos(y)");	break;
 	case mglParabolic:
-		Axis("x*y","(x*x-y*y)/2",0);	break;
+		SetFunc("x*y","(x*x-y*y)/2");	break;
 	case mglParaboloidal:
-		Axis("(x*x-y*y)*cos(z)/2","(x*x-y*y)*sin(z)/2","x*y");	break;
+		SetFunc("(x*x-y*y)*cos(z)/2","(x*x-y*y)*sin(z)/2","x*y");	break;
 	case mglOblate:
-		Axis("cosh(x)*cos(y)*cos(z)","cosh(x)*cos(y)*sin(z)","sinh(x)*sin(y)");	break;
-//		Axis("x*y*cos(z)","x*y*sin(z)","(x*x-1)*(1-y*y)");	break;	
+		SetFunc("cosh(x)*cos(y)*cos(z)","cosh(x)*cos(y)*sin(z)","sinh(x)*sin(y)");	break;
+//		SetFunc("x*y*cos(z)","x*y*sin(z)","(x*x-1)*(1-y*y)");	break;
 	case mglProlate:
-		Axis("sinh(x)*sin(y)*cos(z)","sinh(x)*sin(y)*sin(z)","cosh(x)*cos(y)");	break;
+		SetFunc("sinh(x)*sin(y)*cos(z)","sinh(x)*sin(y)*sin(z)","cosh(x)*cos(y)");	break;
 	case mglElliptic:
-		Axis("cosh(x)*cos(y)","sinh(x)*sin(y)",0);	break;
+		SetFunc("cosh(x)*cos(y)","sinh(x)*sin(y)");	break;
 	case mglToroidal:
-		Axis("sinh(x)*cos(z)/(cosh(x)-cos(y))","sinh(x)*sin(z)/(cosh(x)-cos(y))",
-		     "sin(y)/(cosh(x)-cos(y))");	break;
+		SetFunc("sinh(x)*cos(z)/(cosh(x)-cos(y))","sinh(x)*sin(z)/(cosh(x)-cos(y))",
+			"sin(y)/(cosh(x)-cos(y))");	break;
 	case mglBispherical:
-		Axis("sin(y)*cos(z)/(cosh(x)-cos(y))","sin(y)*sin(z)/(cosh(x)-cos(y))",
-		     "sinh(x)/(cosh(x)-cos(y))");	break;
+		SetFunc("sin(y)*cos(z)/(cosh(x)-cos(y))","sin(y)*sin(z)/(cosh(x)-cos(y))",
+			"sinh(x)/(cosh(x)-cos(y))");	break;
 	case mglBipolar:
-		Axis("sinh(x)/(cosh(x)-cos(y))","sin(y)/(cosh(x)-cos(y))",0);	break;
-	default:	Axis(0,0,0);	break;
+		SetFunc("sinh(x)/(cosh(x)-cos(y))","sin(y)/(cosh(x)-cos(y))");	break;
+	default:	SetFunc(0,0);	break;
 	}
 }
+//-----------------------------------------------------------------------------
