@@ -3,17 +3,17 @@
  * Copyright (C) 2007 Alexey Balakin <balakin@appl.sci-nnov.ru>            *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
- *   it under the terms of the GNU General Public License as published by  *
- *   the Free Software Foundation; either version 2 of the License, or     *
- *   (at your option) any later version.                                   *
+ *   it under the terms of the GNU Library General Public License as       *
+ *   published by the Free Software Foundation; either version 3 of the    *
+ *   License, or (at your option) any later version.                       *
  *                                                                         *
  *   This program is distributed in the hope that it will be useful,       *
  *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
  *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
  *   GNU General Public License for more details.                          *
  *                                                                         *
- *   You should have received a copy of the GNU General Public License     *
- *   along with this program; if not, write to the                         *
+ *   You should have received a copy of the GNU Library General Public     *
+ *   License along with this program; if not, write to the                 *
  *   Free Software Foundation, Inc.,                                       *
  *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
  ***************************************************************************/
@@ -27,7 +27,7 @@
 
 #ifdef WIN32
 #include <io.h>
-wchar_t *wcstokw32(wchar_t *wcs, const wchar_t *delim)	{	wcstok(wcs,delim);	}
+wchar_t *wcstokw32(wchar_t *wcs, const wchar_t *delim)	{	return wcstok(wcs,delim);	}
 #define wcstok(a,b,c) wcstokw32(a,b)
 #else
 #include <unistd.h>
@@ -78,6 +78,16 @@ void mgl_wcstombs(char *dst, const wchar_t *src, int size)
 	dst[j] = 0;
 }
 //-----------------------------------------------------------------------------
+mglCommand *mglParse::FindCommand(const wchar_t *com, bool prog)
+{
+	mglCommand tst, *rts, *cmd = prog?Prg:Cmd;
+	long i;
+	for(i=0;cmd[i].name[0];i++);	// determine the number of symbols
+	tst.name = com;
+	rts = (mglCommand *) bsearch(&tst, cmd, i, sizeof(mglCommand), mgl_cmd_cmp);
+	return rts;
+}
+//-----------------------------------------------------------------------------
 // return values : 0 -- OK, 1 -- wrong arguments, 2 -- wrong command, 3 -- unclosed string
 int mglParse::Exec(mglGraph *gr, const wchar_t *com, long n, mglArg *a, const wchar_t *var)
 {
@@ -85,10 +95,7 @@ int mglParse::Exec(mglGraph *gr, const wchar_t *com, long n, mglArg *a, const wc
 	for(i=0;i<10;i++)	k[i] = i<n ? a[i].type + 1 : 0;
 //	for(i=0;i<n;i++)	wcstombs(a[i].s, a[i].w, 1024);
 	for(i=0;i<n;i++)		mgl_wcstombs(a[i].s, a[i].w, 1024);
-	mglCommand tst, *rts;
-	for(i=0;Cmd[i].name[0];i++);	// determine the number of symbols
-	tst.name = com;
-	rts = (mglCommand *) bsearch(&tst, Cmd, i, sizeof(mglCommand), mgl_cmd_cmp);
+	mglCommand *rts=FindCommand(com);
 	if(!rts)	return 2;
 	if(rts->create)
 	{
@@ -126,6 +133,26 @@ mglVar::~mglVar()
 	if(next)	next->prev = prev;
 }
 //-----------------------------------------------------------------------------
+void mglNum::MoveAfter(mglNum *var)
+{
+	if(prev)	prev->next = next;
+	if(next)	next->prev = prev;
+	prev = next = 0;
+	if(var)
+	{
+		prev = var;
+		next = var->next;
+		var->next = this;
+	}
+	if(next)	next->prev = this;
+}
+//-----------------------------------------------------------------------------
+mglNum::~mglNum()
+{
+	if(prev)	prev->next = next;
+	if(next)	next->prev = prev;
+}
+//-----------------------------------------------------------------------------
 mglParse::mglParse(bool setsize)
 {
 	memset(this,0,sizeof(mglParse));
@@ -186,6 +213,26 @@ mglVar *mglParse::FindVar(const char *str)
 	return v;
 }
 //-----------------------------------------------------------------------------
+mglNum *mglParse::AddNum(const char *str)
+{
+	unsigned s = strlen(str)+1;
+	wchar_t *wcs = new wchar_t[s];
+	mbstowcs(wcs,str,s);
+	mglNum *v = AddNum(wcs);
+	delete []wcs;
+	return v;
+}
+//-----------------------------------------------------------------------------
+mglNum *mglParse::FindNum(const char *str)
+{
+	unsigned s = strlen(str)+1;
+	wchar_t *wcs = new wchar_t[s];
+	mbstowcs(wcs,str,s);
+	mglNum *v = FindNum(wcs);
+	delete []wcs;
+	return v;
+}
+//-----------------------------------------------------------------------------
 bool mglParse::AddParam(int n, const wchar_t *str, bool isstr)
 {
 	if(n<0 || n>9 || wcschr(str,'$'))	return false;
@@ -199,15 +246,11 @@ bool mglParse::AddParam(int n, const wchar_t *str, bool isstr)
 mglVar *mglParse::FindVar(const wchar_t *name)
 {
 	mglVar *v=DataList;
-//	wchar_t *s = mgl_wcsdup(name),*p;
-//	p = wcschr(s,'.');	if(p)	{	p[0]=0;	p++;	}
 	while(v)
 	{
-//		if(!wcscmp(s, v->s))	{	free(s);	return v;	}
 		if(!wcscmp(name, v->s))	return v;
 		v = v->next;
 	}
-//	free(s);
 	return 0;
 }
 //-----------------------------------------------------------------------------
@@ -222,14 +265,38 @@ mglVar *mglParse::AddVar(const wchar_t *name)
 	return v;
 }
 //-----------------------------------------------------------------------------
+mglNum *mglParse::FindNum(const wchar_t *name)
+{
+	mglNum *v=NumList;
+	while(v)
+	{
+		if(!wcscmp(name, v->s))	return v;
+		v = v->next;
+	}
+	return 0;
+}
+//-----------------------------------------------------------------------------
+mglNum *mglParse::AddNum(const wchar_t *name)
+{
+	mglNum *v = FindNum(name);
+	if(v)	return v;
+	v = new mglNum;
+	wcsncpy(v->s,name,256);
+	if(NumList)	v->MoveAfter(NumList);
+	else		NumList = v;
+	return v;
+}
+//-----------------------------------------------------------------------------
 int mglFindArg(const wchar_t *str)
 {
-	register long l=0,i;//,j,len=strlen(lst);
+	register long l=0,k=0,i;//,j,len=strlen(lst);
 	for(i=0;i<long(wcslen(str));i++)
 	{
-		if(l%2==0 && (str[i]=='#' || str[i]==';'))	return -i;
 		if(str[i]=='\'') l++;
-		if(l%2==0 && (str[i]<=' '))	return i;
+		if(str[i]=='{') k++;
+		if(str[i]=='}') k--;
+		if(l%2==0 && k==0 && (str[i]=='#' || str[i]==';'))	return -i;
+		if(l%2==0 && k==0 && (str[i]<=' '))	return i;
 	}
 	return 0;
 }
@@ -274,12 +341,13 @@ bool mgls_suffix(const wchar_t *p, mglData *d, mreal *v)
 //-----------------------------------------------------------------------------
 // convert substrings to arguments
 mglData mglFormulaCalc(const wchar_t *string, mglParse *arg);
-void mglParse::FillArg(int k, wchar_t **arg, mglArg *a)
+void mglParse::FillArg(mglGraph *gr, int k, wchar_t **arg, mglArg *a)
 {
 	register long n;
 	for(n=1;n<k;n++)
 	{
 		mglVar *v, *u;
+		mglNum *f;
 		a[n-1].type = -1;
 		if(arg[n][0]=='|')	a[n-1].type = -1;
 		else if(arg[n][0]=='\'')
@@ -288,11 +356,26 @@ void mglParse::FillArg(int k, wchar_t **arg, mglArg *a)
 			if(wcslen(arg[n]+1)>=2048)	arg[n][2048]=0;
 			wcscpy(a[n-1].w, arg[n]+1);
 		}
+		else if(arg[n][0]=='{')
+		{	// this is temp data
+			arg[n][wcslen(arg[n])-1] = 0;
+			u=new mglVar;	u->temp=true;
+			mglprintf(a[n-1].w,2048,L"/*%ls*/",arg[n]);
+			if(DataList)	u->MoveAfter(DataList);
+			else			DataList = u;
+			a[n-1].type = 0;	a[n-1].d = &(u->d);
+			ParseDat(gr, arg[n]+1, u->d);	// TODO: Check it
+		}
 		else if((v = FindVar(arg[n]))!=0)
 		{	// have to find normal variables (for data creation)
 			a[n-1].type = 0;		a[n-1].d = &(v->d);
 			wcscpy(a[n-1].w, v->s);
 		}
+		else if((f = FindNum(arg[n]))!=0)
+		{	// have to find normal variables (for data creation)
+			a[n-1].type = 2;	a[n-1].d = 0;	a[n-1].v = f->d;
+			wcscpy(a[n-1].w, f->s);
+		}
 		else
 		{	// parse all numbers and formulas by unified way
 			const mglData &d=mglFormulaCalc(arg[n], this);
@@ -313,7 +396,7 @@ void mglParse::FillArg(int k, wchar_t **arg, mglArg *a)
 //-----------------------------------------------------------------------------
 // return values: 0 - not found, 1 - OK, 2 - wrong arguments,
 //				3 - wrong command, 4 - string too long
-int mglParse::PreExec(mglGraph *gr, long k, wchar_t **arg, mglArg *a)
+int mglParse::PreExec(mglGraph *, long k, wchar_t **arg, mglArg *a)
 {
 	long n=0;
 	mglVar *v;
@@ -399,24 +482,84 @@ int mglParse::Parse(mglGraph *gr, const wchar_t *string, long pos)
 	str = s;
 	wcscpy(str,string);
 	wcstrim_mgl(str);
-
-	long n,k=0;
-	for(n=1;n<long(wcslen(str));n++)
+	long n,k=0,m=0;
+	// try parse ':'
+	for(n=k=0;n<long(wcslen(str));n++)
+	{
+		if(str[n]=='\'')	k++;
+		if(str[n]=='(' && k%2==0)	m++;
+		if(str[n]==')' && k%2==0)	m--;
+		if(str[n]=='#' && k%2==0)	break;
+		if(str[n]==':' && k%2==0 && m==0)
+		{
+			str[n]=0;
+			int res=Parse(gr,str,pos);
+			if(!res)	res=Parse(gr,str+n+1,pos);
+			delete []s;	return res;
+		}
+	}
+	// check if string is closed
+	for(n=1,k=0;n<long(wcslen(str));n++)
 		if(str[n]=='\'' && str[n-1]!='\\')	k++;
 	if(k%2)	return 4;	// strings is not closed
 	// define parameters or start cycle
-	if(!wcsncmp(str,L"define",6) && (str[6]==' ' || str[6]=='\t'))
+	if(!skip() && !wcsncmp(str,L"def",3) && (str[6]==' ' || str[6]=='\t'))
 	{
-		str += 7;	wcstrim_mgl(str);	int res = 1;
-		if(*str=='$' && str[1]>='0' && str[1]<='9')
+		if(!skip() && !wcsncmp(str+3,L"ine",3) && (str[6]==' ' || str[6]=='\t'))
 		{
-			int n=str[1]-'0';	res = 0;
-			str +=2;	mgl_wcstrim(str);
-			AddParam(n, str);
+			wchar_t *ss=str+7;	wcstrim_mgl(ss);//	int res = 1;
+			if(*ss=='$' && ss[1]>='0' && ss[1]<='9')
+			{
+				int n=ss[1]-'0';//	res = 0;
+				ss +=2;	mgl_wcstrim(ss);
+				AddParam(n, ss);
+				delete []s;	return 0;
+			}
+		}
+		if(!skip() && !wcsncmp(str+3,L"num",3) && (str[6]==' ' || str[6]=='\t'))
+		{
+			str += 7;	wcstrim_mgl(str);	int res = 1;
+			if(*str=='$' && str[1]>='0' && str[1]<='9')
+			{
+				int n=str[1]-'0';	res = 0;
+				str +=2;	mgl_wcstrim(str);
+				const mglData &d=mglFormulaCalc(str, this);
+				char *buf=new char[128];
+				sprintf(buf,"%g",d.a[0]);
+				AddParam(n, buf);
+				delete []buf;
+			}
+			delete []s;		return res;
+		}
+		if(!skip() && !wcsncmp(str+3,L"chr",3) && (str[6]==' ' || str[6]=='\t'))
+		{
+			str += 7;	wcstrim_mgl(str);	int res = 1;
+			if(*str=='$' && str[1]>='0' && str[1]<='9')
+			{
+				int n=str[1]-'0';	res = 0;
+				str +=2;	mgl_wcstrim(str);
+				const mglData &d=mglFormulaCalc(str, this);
+				wchar_t buf[2]={0,0};	buf[0] = wchar_t(d.a[0]);
+				AddParam(n, buf);
+			}
+			delete []s;		return res;
+		}
+		if(!skip() && !wcsncmp(str+3,L"pal",3) && (str[6]==' ' || str[6]=='\t'))
+		{
+			str += 7;	wcstrim_mgl(str);	int res = 1;
+			if(*str=='$' && str[1]>='0' && str[1]<='9')
+			{
+				int n=str[1]-'0';	res = 0;
+				str +=2;	mgl_wcstrim(str);
+				const mglData &d=mglFormulaCalc(str, this);
+				wchar_t buf[2]={0,0};
+				buf[0] = gr->DefPal[int(d.a[0])%gr->NumPal];
+				AddParam(n, buf);
+			}
+			delete []s;		return res;
 		}
-		delete []s;		return res;
 	}
-	if(!wcsncmp(str,L"for",3) && (str[3]==' ' || str[3]=='\t'))
+	if(!skip() && !wcsncmp(str,L"for",3) && (str[3]==' ' || str[3]=='\t'))
 	{
 		t = str+4;
 		while(*t && isspace(*t))	t++;
@@ -439,7 +582,8 @@ int mglParse::Parse(mglGraph *gr, const wchar_t *string, long pos)
 	}
 
 	wcstrim_mgl(str);
-	if(!wcscmp(str,L"stop"))	{	Stop = true;	delete []s;	return 0;	}
+	if(!skip() && !wcscmp(str,L"stop"))	{	Stop = true;	delete []s;	return 0;	}
+
 	for(k=0;k<1024;k++)	// parse string to substrings (by spaces)
 	{
 		n = mglFindArg(str);
@@ -459,11 +603,21 @@ int mglParse::Parse(mglGraph *gr, const wchar_t *string, long pos)
 	{
 		// fill arguments by its values
 		mglArg *a = new mglArg[k];
-		FillArg(k, arg, a);
+		FillArg(gr, k, arg, a);
 		// execute first special (program-flow-control) commands
 		n = FlowExec(gr, arg[0],k-1,a);
-		if(n)	{	delete []s;	delete []a;	return n-1;	}
-		if(Skip || (if_pos>0 && !(if_stack[if_pos-1]&1)))	{	delete []s;	delete []a;	return 0;	}
+		if(n)		{	delete []s;	delete []a;	return n-1;	}
+		if(skip())	{	delete []s;	delete []a;	return 0;	}
+		if(!wcscmp(arg[0],L"define"))
+		{
+			if(k==3)
+			{
+				mglNum *v=AddNum(arg[1]);
+				mglData d=mglFormulaCalc(arg[2],this);
+				v->d = d.a[0];
+			}
+			delete []s;	delete []a;	return k==3?0:1;
+		}
 		if(!wcscmp(arg[0],L"call"))
 		{
 			n = 1;
@@ -498,8 +652,9 @@ int mglParse::Parse(mglGraph *gr, const wchar_t *string, long pos)
 				}
 				if(n==0)
 				{
-					for(int i=9;i>0;i--)	for_stack[i] = for_stack[i-1];
-					for_stack[0] = r+1;		fval[r].nz = pos;
+					for(int i=9;i>0;i--)
+					{	for_stack[i]=for_stack[i-1];	if_for[i]=if_for[i-1];	}
+					for_stack[0] = r+1;		fval[r].nz = pos;	if_for[0]=if_pos;
 					wchar_t buf[32];		mglprintf(buf,32,L"%g",fval[r].a[0]);
 					AddParam(r, buf,true);	fval[r].ny = 1;
 				}
@@ -526,10 +681,52 @@ int mglParse::Parse(mglGraph *gr, const wchar_t *string, long pos)
 	return n;
 }
 //-----------------------------------------------------------------------------
-int mglParse::FlowExec(mglGraph *gr, const wchar_t *com, long m, mglArg *a)
+// return values: 0 - OK, 1 - wrong arguments, 2 - wrong command, 3 - string too long, 4 -- unclosed string
+int mglParse::ParseDat(mglGraph *gr, const wchar_t *string, mglData &res)
+{
+	wchar_t *str, *s = new wchar_t[wcslen(string)+1+parlen],*arg[32];
+	str = s;
+	wcscpy(str,string);	wcstrim_mgl(str);
+	long n,k=0;
+	for(k=0;k<32;k++)	// parse string to substrings (by spaces)
+	{
+		n = mglFindArg(str);
+		if(n<1)
+		{
+			if(str[-n]==';')	ProcOpt(gr,str-n);
+			if(n<0)	str[-n]=0;
+			break;
+		}
+		str[n]=0;	arg[k] = str;//	k++;
+		str = str+n+1;	wcstrim_mgl(str);
+	}
+	// try to find last argument
+	if(str[0]!=0 && str[0]!='#' && str[0]!=';')	{	arg[k] = str;	k++;	}
+	if(k<1) n =0;
+	else
+	{
+		// fill arguments by its values
+		mglArg *a = new mglArg[k+1];
+		FillArg(gr, k, arg, a+1);	a[0].type=0;	a[0].d=&res;
+		// alocate new arrays and execute the command itself
+		int kk[10], i;
+		for(i=0;i<10;i++)	kk[i] = i<=k ? a[i].type + 1 : 0;
+		for(i=0;i<=k;i++)	mgl_wcstombs(a[i].s, a[i].w, 1024);
+		mglCommand *rts=FindCommand(arg[0]);
+		if(!rts || !rts->create)	return 2;
+		if(out)	rts->save(out, k, a, kk);
+		n = rts->exec(gr, k, a, kk);
+		delete []a;
+	}
+	// restore plot settings if it was changed
+	ProcOpt(gr,0);	delete []s;
+	return n;
+}
+//-----------------------------------------------------------------------------
+int mglParse::FlowExec(mglGraph *, const wchar_t *com, long m, mglArg *a)
 {
 	int n=-1;
-	if(!wcscmp(com,L"once"))
+	if(!ifskip() && !wcscmp(com,L"once"))
 	{
 		if(a[0].type==2)
 		{
@@ -541,7 +738,7 @@ int mglParse::FlowExec(mglGraph *gr, const wchar_t *com, long m, mglArg *a)
 		}
 		else n = 1;
 	}
-	else if(!wcscmp(com,L"if"))
+	else if(!Skip && !wcscmp(com,L"if"))
 	{
 		int cond;
 		if(a[0].type==2)
@@ -549,22 +746,23 @@ int mglParse::FlowExec(mglGraph *gr, const wchar_t *com, long m, mglArg *a)
 			n = 0;	cond = (a[0].v!=0)?3:0;
 			if(out)	mglprintf(out,1024,L"if(%g!=0)\t{", a[0].v);
 		}
-		else if(m>1 && a[0].type==0 && a[1].type==1)
+		else if(a[0].type==0)
 		{
 			n = 0;	mgl_wcstombs(a[1].s, a[1].w, 1024);
-			cond = a[0].d->FindAny(a[1].s)?3:0;
-			if(out)	mglprintf(out,1024,L"if(%ls.FindAny(\"%s\"))\t{", a[0].w, a[1].s);
+			cond = a[0].d->FindAny((m>1 && a[1].type==1) ? a[1].s:"u")?3:0;
+			if(out)	mglprintf(out,1024,L"if(%ls.FindAny(\"%s\"))\t{",
+				a[0].w, (m>1 && a[1].type==1) ? a[1].s:"u");
 		}
 		else n = 1;
 		if(n==0)
 		{	if_stack[if_pos] = cond;	if_pos = if_pos<19 ? if_pos+1 : 19;	}
 	}
-	else if(!wcscmp(com,L"endif"))
+	else if(!Skip && !wcscmp(com,L"endif"))
 	{
 		if_pos = if_pos>0 ? if_pos-1 : 0;
 		n = 0;	if(out)	mglprintf(out,1024,L"}");
 	}
-	else if(!wcscmp(com,L"else"))
+	else if(!Skip && !wcscmp(com,L"else"))
 	{
 		if(if_pos>0)
 		{
@@ -573,26 +771,56 @@ int mglParse::FlowExec(mglGraph *gr, const wchar_t *com, long m, mglArg *a)
 		}
 		else n = 1;
 	}
-	else if(!wcscmp(com,L"elseif"))
+	else if(!Skip && !wcscmp(com,L"elseif"))
 	{
 		int cond;
 		if(if_pos<1 || m<1)	n = 1;
 		else if(if_stack[if_pos-1]&2)	{	n = 0;	cond = 2;	}
-		else if(a[0].type==2)	{	n = 0;	cond = (a[0].v!=0)?3:0;	}
-		else if(m>1 && a[0].type==0 && a[1].type==1)
-		{	n = 0;	mgl_wcstombs(a[1].s, a[1].w, 1024);	cond = a[0].d->FindAny(a[1].s)?3:0;	}
-		else n = 1;
-		if(n==0)
+		else if(a[0].type==2)
 		{
-			if_stack[if_pos-1] = cond;
-			if(out && a[0].type==2)
-				mglprintf(out,1024,L"else if(%g!=0)\t{", a[0].v);
-			if(out && m>1 && a[0].type==0 && a[1].type==1)
-				mglprintf(out,1024,L"else if(%ls.FindAny(\"%ls\"))\t{", a[0].w, a[1].w);
+			n = 0;	cond = (a[0].v!=0)?3:0;
+			if(out)	mglprintf(out,1024,L"else if(%g!=0)\t{", a[0].v);
 		}
+		else if(a[0].type==0)
+		{
+			n = 0;	mgl_wcstombs(a[1].s, a[1].w, 1024);
+			cond = a[0].d->FindAny((m>1 && a[1].type==1) ? a[1].s:"u")?3:0;
+			if(out)	mglprintf(out,1024,L"else if(%ls.FindAny(\"%s\"))\t{",
+							a[0].w, (m>1 && a[1].type==1) ? a[1].s:"u");
+		}
+		else n = 1;
+		if(n==0)	if_stack[if_pos-1] = cond;
 	}
-	else if(!wcscmp(com,L"next"))
+	else if(!ifskip() && !Skip && !wcscmp(com,L"break"))
 	{
+		if(if_pos==if_for[0])	if_pos = if_pos>0 ? if_pos-1 : 0;
+		if(out)	mglprintf(out,1024,L"break;");	for_br = true;
+	}
+	else if(!ifskip() && !Skip && !wcscmp(com,L"next"))
+	{
+		if(if_pos==if_for[0])	if_pos = if_pos>0 ? if_pos-1 : 0;
+		int r = for_stack[0]-1;
+		n = for_stack[0] ? 0:1;
+		if(for_stack[0])
+		{
+			if(fval[r].ny<fval[r].nx && !for_br)
+			{
+				wchar_t buf[32];		mglprintf(buf,32,L"%g",fval[r].a[fval[r].ny]);
+				AddParam(r, buf,true);	fval[r].ny += 1;
+				n = -fval[r].nz-1;
+			}
+			else
+			{
+				for(int i=0;i<9;i++)
+				{	for_stack[i]=for_stack[i+1];	if_for[i]=if_for[i+1];	}
+				for_stack[9] = 0;	for_br=false;
+			}
+		}
+		if(out)	mglprintf(out,1024,L"}");
+	}
+	else if(!ifskip() && !Skip && !wcscmp(com,L"continue"))
+	{
+		if(if_pos==if_for[0])	if_pos = if_pos>0 ? if_pos-1 : 0;
 		int r = for_stack[0]-1;
 		n = for_stack[0] ? 0:1;
 		if(for_stack[0])
@@ -603,12 +831,9 @@ int mglParse::FlowExec(mglGraph *gr, const wchar_t *com, long m, mglArg *a)
 				AddParam(r, buf,true);	fval[r].ny += 1;
 				n = -fval[r].nz-1;
 			}
-			else
-			{
-				for(int i=0;i<9;i++)	for_stack[i] = for_stack[i+1];
-				for_stack[9] = 0;
-			}
+			else	for_br = true;
 		}
+		if(out)	mglprintf(out,1024,L"continue;");
 	}
 	return n+1;
 }
@@ -620,6 +845,7 @@ void mgl_error_print(int line, int r, mglGraph *gr)
 	if(r==2)	printf("Wrong command in line %d\n", line);
 	if(r==3)	printf("String too long in line %d\n", line);
 	if(r==4)	printf("Unbalanced ' in line %d\n", line);
+	gr->Message[0]=0;
 }
 void mglParse::Execute(mglGraph *gr, FILE *fp, bool print)
 {
@@ -642,6 +868,7 @@ void mglParse::Execute(mglGraph *gr, int n, const wchar_t **text, void (*error)(
 {
 	if(gr==0 || n<1 || text==0)	return;
 	long i, r;
+	for_br=Skip=false;	if_pos=0;
 	for(i=0;i<n;i++)
 	{
 		r = Parse(gr,text[i],i+1);
@@ -763,94 +990,99 @@ void mglParse::ProcOpt(mglGraph *gr, wchar_t *str)
 	}
 	else
 	{
-		val[0] = gr->Min.x;		val[1] = gr->Max.x;
-		val[2] = gr->Min.y;		val[3] = gr->Max.y;
-		val[4] = gr->Min.z;		val[5] = gr->Max.z;
-		val[6] = gr->Cmin;		val[7] = gr->Cmax;
-		val[8] = gr->AmbBr;		val[10]= gr->Cut;
-		val[12]= gr->AlphaDef;	val[14]= gr->MeshNum;
-		val[16]= gr->FontSize;	val[17]= gr->MarkSize;
-		wchar_t *s = str,*a,*b;
+		wchar_t *q=str,*s,*a,*b,*c;
 		long n;
-		while((s=wcschr(s,';'))!=0)
+		wcstrim_mgl(q);		q++;
+		// NOTE: not consider '#' inside legend entry !!!
+		s=wcschr(q,'#');	if(s)	*s=0;
+		while(q && *q)
 		{
-			s++;	wcstrim_mgl(s);
-			n=mglFindArg(s);
-			if(n<1)	return;
-			s[n]=0;		a=s;	s=s+n+1;	wcstrim_mgl(a);
-			mreal ff = wcstod(s,0);
+			// NOTE: not consider ';' inside legend entry !!!
+			s=q;	q=wcschr(s,';');
+			if(q)	{	*q=0;	q++;	}
+			wcstrim_mgl(s);		a=s;
+			n=mglFindArg(s);	if(n>0)	{	s[n]=0;		s=s+n+1;	}
+			wcstrim_mgl(a);		b=s;
+			n=mglFindArg(s);	if(n>0)	{	s[n]=0;		s=s+n+1;	}
+			wcstrim_mgl(b);
+
+			mglData bb,ss;
+			bb = mglFormulaCalc(b, this);
+			mreal ff = bb.a[0];
 			if(!wcscmp(a+1,L"range"))
 			{
-				n=mglFindArg(s);	if(n<1)	return;
-				s[n]=0;		b=s;	s=s+n+1;	wcstrim_mgl(b);
+				n=mglFindArg(s);	c=s;
+				if(n>0)	{	s[n]=0;		s=s+n+1;	}
+				wcstrim_mgl(c);
+				ss = mglFormulaCalc(c, this);
 				if(a[0]=='x')
 				{
-					opt[0]=true;	gr->Min.x = wcstod(b,0);	gr->Max.x = wcstod(s,0);
+					val[0] = gr->Min.x;		val[1] = gr->Max.x;		opt[0]=true;
+					gr->Min.x = bb.a[0];	gr->Max.x = ss.a[0];
 					if(out)	mglprintf(buf,256,L"{mreal tx1=gr->Min.x, tx2=gr->Max.x;\tgr->Min.x=%g;\tgr->Max.x=%g;", wcstod(b,0), wcstod(s,0));
 					wcscat(op1,buf);
 				}
 				else if(a[0]=='y')
 				{
-					opt[1]=true;	gr->Min.y = wcstod(b,0);	gr->Max.y = wcstod(s,0);
+					val[2] = gr->Min.y;		val[3] = gr->Max.y;		opt[1]=true;
+					gr->Min.y = bb.a[0];	gr->Max.y = ss.a[0];
 					if(out)	mglprintf(buf,256,L"{mreal ty1=gr->Min.y, ty2=gr->Max.y;\tgr->Min.y=%g;\tgr->Max.y=%g;", wcstod(b,0), wcstod(s,0));
 					wcscat(op1,buf);
 				}
 				else if(a[0]=='z')
 				{
-					opt[2]=true;	gr->Min.z = wcstod(b,0);	gr->Max.z = wcstod(s,0);
+					val[4] = gr->Min.z;		val[5] = gr->Max.z;		opt[2]=true;
+					gr->Min.z = bb.a[0];	gr->Max.z = ss.a[0];
 					if(out)	mglprintf(buf,256,L"{mreal tz1=gr->Min.z, tz2=gr->Max.z;\tgr->Min.z=%g;\tgr->Max.z=%g;", wcstod(b,0), wcstod(s,0));
 					wcscat(op1,buf);
 				}
 				else if(a[0]=='c')
 				{
-					opt[3]=true;	gr->Cmin = wcstod(b,0);		gr->Cmax = wcstod(s,0);
+					val[6] = gr->Cmin;		val[7] = gr->Cmax;		opt[3]=true;
+					gr->Cmin = bb.a[0];		gr->Cmax = ss.a[0];
 					if(out)	mglprintf(buf,256,L"{mreal tc1=gr->Cmin, tc2=gr->Cmax;\tgr->Cmin=%g;\tgr->Cmax=%g;", wcstod(b,0), wcstod(s,0));
 					wcscat(op1,buf);
 				}
 			}
 			else if(!wcscmp(a,L"cut"))
 			{
-				opt[5]=true;	gr->Cut = (ff!=0 || !wcsncmp(s,L"on",2));
+				val[10]= gr->Cut;	opt[5]=true;
+				gr->Cut = (ff!=0 || !wcsncmp(s,L"on",2));
 				if(out)	mglprintf(buf,256,L"{bool tct=gr->Cut;\tgr->Cut=%s;", gr->Cut?"true":"false");
 				wcscat(op1,buf);
 			}
 			else if(!wcscmp(a,L"meshnum"))
 			{
-				opt[7]=true;	gr->MeshNum = int(ff);
+				val[14]= gr->MeshNum;	opt[7]=true;	gr->MeshNum = int(ff);
 				if(out)	mglprintf(buf,256,L"{int tmn=gr->MeshNum;\tgr->MeshNum=%d;", gr->MeshNum);
 				wcscat(op1,buf);
 			}
-			else if(!wcscmp(a,L"alphadef"))
+			else if(!wcscmp(a,L"alphadef") || !wcscmp(a,L"alpha"))
 			{
-				opt[6]=true;	gr->AlphaDef = ff;
-				if(out)	mglprintf(buf,256,L"{mreal tad=gr->AlphaDef;\tgr->AlphaDef=%g;", ff);
-				wcscat(op1,buf);
-			}
-			else if(!wcscmp(a,L"alpha"))
-			{
-				opt[6]=true;	gr->AlphaDef = ff;
+				val[12]= gr->AlphaDef;	opt[6]=true;	gr->AlphaDef = ff;
 				if(out)	mglprintf(buf,256,L"{mreal tad=gr->AlphaDef;\tgr->AlphaDef=%g;", ff);
 				wcscat(op1,buf);
 			}
 			else if(!wcscmp(a,L"fontsize"))
 			{
-				opt[8]=true;	gr->FontSize = ff>0 ? ff : -ff*gr->FontSize;
+				val[16]= gr->FontSize;	opt[8]=true;
+				gr->FontSize = ff>0 ? ff : -ff*gr->FontSize;
 				if(out)	mglprintf(buf,256,L"{mreal tfs=gr->FontSize;\tgr->FontSize=%g;", gr->FontSize);
 				wcscat(op1,buf);
 			}
 			else if(!wcscmp(a,L"ambient"))
 			{
-				opt[4]=true;	gr->Ambient(ff);
+				val[8] = gr->AmbBr;		opt[4]=true;	gr->Ambient(ff);
 				if(out)	mglprintf(buf,256,L"{mreal tam=gr->AmbBr;\tgr->Ambient(%g);", ff);
 				wcscat(op1,buf);
 			}
 			else if(!wcscmp(a,L"marksize"))
 			{
-				opt[9]=true;	gr->MarkSize = ff/50;
+				val[17]= gr->MarkSize;	opt[9]=true;	gr->MarkSize = ff/50;
 				if(out)	mglprintf(buf,256,L"{mreal tad=gr->MarkSize;\tgr->MarkSize=%g/50;", ff);
 				wcscat(op1,buf);
 			}
-			else if(!wcscmp(a,L"legend"))
+			else if(!wcscmp(a,L"legend"))	// TODO: parsing of string tail (other options???)
 			{
 				b = wcschr(s+1,'\'');	if(b)	*b=0;
 				opt[10]=true;	wcscpy(leg,s+1);
@@ -893,6 +1125,25 @@ void mglParse::AddCommand(mglCommand *cmd, int mc)
 	Cmd = buf;
 }
 //-----------------------------------------------------------------------------
+mglCommand mglParse::Prg[]={
+	{L"break",L"Break for-cycle",L"break", 0, 0, 0, 5},
+	{L"call",L"Execute script in external file",L"call 'filename'", 0, 0, 0, 5},
+	{L"continue",L"Skip commands and iterate for-cycle again",L"continue", 0, 0, 0, 5},
+	{L"defchr",L"Define parameter as character",L"defchr $N val", 0, 0, 0, 5},
+	{L"define",L"Define constant or parameter",L"define $N sth | Var val", 0, 0, 0, 5},
+	{L"defnum",L"Define parameter as numerical value",L"defnum $N val", 0, 0, 0, 5},
+	{L"defpal",L"Define parameter as palette color",L"defpal $N val", 0, 0, 0, 5},
+	{L"delete",L"Deleta variable",L"delete Dat", 0, 0, 0, 5},
+	{L"else",L"Execute if condition is false",L"else", 0, 0, 0, 5},
+	{L"elseif",L"Conditional operator",L"elseif val|Dat ['cond']", 0, 0, 0, 5},
+	{L"endif",L"Finish if/else block",L"endif", 0, 0, 0, 5},
+	{L"for",L"For cycle",L"for $N v1 v2 [dv] | $N Dat", 0, 0, 0, 5},
+	{L"if",L"Conditional operator",L"if val|Dat ['cond']", 0, 0, 0, 5},
+	{L"next",L"Start next for-cycle iteration",L"next", 0, 0, 0, 5},
+	{L"once",L"Start/close commands which should executed only once",L"once val", 0, 0, 0, 5},
+	{L"stop",L"Stop execution",L"stop", 0, 0, 0, 5},
+{L"",0,0, 0, 0, 0, 0}};
+//-----------------------------------------------------------------------------
 HMPR mgl_create_parser()		{	return new mglParse;	}
 void mgl_delete_parser(HMPR p)	{	delete p;	}
 void mgl_add_param(HMPR p, int id, const char *str)			{	p->AddParam(id,str);	}
