@@ -3,17 +3,17 @@
  * Copyright (C) 2007 Alexey Balakin <balakin@appl.sci-nnov.ru>            *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
- *   it under the terms of the GNU General Public License as published by  *
- *   the Free Software Foundation; either version 2 of the License, or     *
- *   (at your option) any later version.                                   *
+ *   it under the terms of the GNU Library General Public License as       *
+ *   published by the Free Software Foundation; either version 3 of the    *
+ *   License, or (at your option) any later version.                       *
  *                                                                         *
  *   This program is distributed in the hope that it will be useful,       *
  *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
  *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
  *   GNU General Public License for more details.                          *
  *                                                                         *
- *   You should have received a copy of the GNU General Public License     *
- *   along with this program; if not, write to the                         *
+ *   You should have received a copy of the GNU Library General Public     *
+ *   License along with this program; if not, write to the                 *
  *   Free Software Foundation, Inc.,                                       *
  *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
  ***************************************************************************/
@@ -243,7 +243,7 @@ mglData mglQO2d(const char *ham, const mglData &ini_re, const mglData &ini_im, c
 	for(i=0;i<nx;i++)	a[i+nx/2] = dual(ini_re.a[i],ini_im.a[i]);	// ini
 	for(i=0;i<2*nx;i++)	{	rx[i] = ru[i] = 1;	}
 	mglFormula h(ham);
-	mreal var[MGL_VS], dr = r/nx, dk = M_PI/(k0*r), tt, x1, hh, B1, pt0;
+	mreal var[MGL_VS], dr = r/(nx-1), dk = M_PI*(nx-1)/(k0*r*nx), tt, x1, hh, B1, pt0;
 	memset(var,0,MGL_VS*sizeof(mreal));
 	gsl_fft_complex_wavetable *wtx = gsl_fft_complex_wavetable_alloc(2*nx);
 	gsl_fft_complex_workspace *wsx = gsl_fft_complex_workspace_alloc(2*nx);
@@ -255,13 +255,13 @@ mglData mglQO2d(const char *ham, const mglData &ini_re, const mglData &ini_im, c
 			res.a[i+k*nx]=abs(a[i+nx/2])*sqrt(ra[0].ch/ra[k].ch);
 		if(xx && yy)	for(i=0;i<nx;i++)	// prepare xx, yy
 		{
-			x1 = (2*i-nx)*dr;
+			x1 = (2*i-nx+1)*dr;
 			xx->a[i+k*nx] = ray.a[7*k] + ra[k].x1*x1;	// new coordiantes
 			yy->a[i+k*nx] = ray.a[7*k+1] + ra[k].y1*x1;
 		}
 		memcpy(px,rx,2*nx*sizeof(double));
 		memcpy(pu,ru,2*nx*sizeof(double));
-		hh = ra[k].pt*(1/sqrt(sqrt(1.041))-1);
+		hh = ra[k].pt*(1/sqrt(sqrt(1.041))-1);	// 0.041=0.45^4 -- minimal value of h
 		var['x'-'a'] = ray.a[7*k];	var['y'-'a'] = ray.a[7*k+1];
 		var['p'-'a'] = ray.a[7*k+3] + ra[k].x0*hh;
 		var['q'-'a'] = ray.a[7*k+4] + ra[k].y0*hh;	var['u'-'a'] = 0;
@@ -274,7 +274,7 @@ mglData mglQO2d(const char *ham, const mglData &ini_re, const mglData &ini_im, c
 			var['q'-'a'] = ray.a[7*k+4] + ra[k].y0*hh;	var['u'-'a'] = 0;
 			h0 = dual(h.Calc(var), -h.CalcD(var,'i'));
 			// x terms
-			x1 = 2*(i-nx)*dr;
+			x1 = (2*i-2*nx+1)*dr;
 			var['x'-'a'] = ray.a[7*k] + ra[k].x1*x1;	// new coordiantes
 			var['y'-'a'] = ray.a[7*k+1] + ra[k].y1*x1;
 			hh = 1 - ra[k].t1*x1;	hh = sqrt(sqrt(0.041+hh*hh*hh*hh));
@@ -357,7 +357,7 @@ mglData mglAF2d(const char *ham, const mglData &ini_re, const mglData &ini_im, c
 	register int i;
 	for(i=0;i<nx;i++)	a[i+nx/2] = dual(ini_re.a[i],ini_im.a[i]);	// ini
 	mglFormula h(ham);
-	mreal var[MGL_VS], dr = r/nx, dk = M_PI/(k0*r), tt, x1, hh, B1;
+	mreal var[MGL_VS], dr = r/(nx-1), dk = M_PI/(k0*r), tt, x1, hh, B1;
 	memset(var,0,MGL_VS*sizeof(mreal));
 	gsl_fft_complex_wavetable *wtx = gsl_fft_complex_wavetable_alloc(2*nx);
 	gsl_fft_complex_workspace *wsx = gsl_fft_complex_workspace_alloc(2*nx);
@@ -369,7 +369,7 @@ mglData mglAF2d(const char *ham, const mglData &ini_re, const mglData &ini_im, c
 			res.a[i+k*nx]=abs(a[i+nx/2])*sqrt(ra[0].ch/ra[k].ch);
 		if(xx && yy)	for(i=0;i<nx;i++)	// prepare xx, yy
 		{
-			x1 = (2*i-nx)*dr;
+			x1 = (2*i-nx+1)*dr;
 			xx->a[i+k*nx] = ray.a[7*k] + ra[k].x1*x1;	// new coordiantes
 			yy->a[i+k*nx] = ray.a[7*k+1] + ra[k].y1*x1;
 		}
