@@ -3,21 +3,20 @@
  * Copyright (C) 2007 Alexey Balakin <balakin@appl.sci-nnov.ru>            *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
- *   it under the terms of the GNU General Public License as published by  *
- *   the Free Software Foundation; either version 2 of the License, or     *
- *   (at your option) any later version.                                   *
+ *   it under the terms of the GNU Library General Public License as       *
+ *   published by the Free Software Foundation; either version 3 of the    *
+ *   License, or (at your option) any later version.                       *
  *                                                                         *
  *   This program is distributed in the hope that it will be useful,       *
  *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
  *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
  *   GNU General Public License for more details.                          *
  *                                                                         *
- *   You should have received a copy of the GNU General Public License     *
- *   along with this program; if not, write to the                         *
+ *   You should have received a copy of the GNU Library General Public     *
+ *   License along with this program; if not, write to the                 *
  *   Free Software Foundation, Inc.,                                       *
  *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
  ***************************************************************************/
-
 #include <FL/Fl_Pixmap.H>
 #include <FL/fl_ask.H>
 #include <FL/Fl_Double_Window.H>
@@ -97,7 +96,7 @@ void Fl_MathGL::update(mglGraph *gr)
 {
 	if(gr==0)	gr=graph;
 	if(gr==0 || draw_func==0)	return;
-	gr->DefaultPlotParam();
+	if(gr!=graph || graph->ClfOnUpdate)	gr->DefaultPlotParam();
 	gr->Alpha(flag&1);
 	gr->Light(flag&2);
 	gr->View(tet,phi);
