@@ -3,17 +3,17 @@
  * Copyright (C) 2007 Alexey Balakin <balakin@appl.sci-nnov.ru>            *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
- *   it under the terms of the GNU General Public License as published by  *
- *   the Free Software Foundation; either version 2 of the License, or     *
- *   (at your option) any later version.                                   *
+ *   it under the terms of the GNU Library General Public License as       *
+ *   published by the Free Software Foundation; either version 3 of the    *
+ *   License, or (at your option) any later version.                       *
  *                                                                         *
  *   This program is distributed in the hope that it will be useful,       *
  *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
  *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
  *   GNU General Public License for more details.                          *
  *                                                                         *
- *   You should have received a copy of the GNU General Public License     *
- *   along with this program; if not, write to the                         *
+ *   You should have received a copy of the GNU Library General Public     *
+ *   License along with this program; if not, write to the                 *
  *   Free Software Foundation, Inc.,                                       *
  *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
  ***************************************************************************/
@@ -191,7 +191,7 @@ void QMathGL::update(mglGraph *gr)
 {
 	if(gr==0)	gr = graph;
 	if(gr==0 || draw_func==0)	return;
-	gr->DefaultPlotParam();
+	if(gr!=graph || graph->ClfOnUpdate)	gr->DefaultPlotParam();
 	gr->Alpha(alpha);	gr->Light(light);
 	gr->View(tet,phi);	gr->Org = mglPoint(NAN,NAN,NAN);
 	gr->Perspective(per);
@@ -289,10 +289,7 @@ void QMathGL::mouseMoveEvent(QMouseEvent *ev)
 QString setExtension(QString &fname, const char *ext)
 {
 	QString oname;
-	if(fname.right(4)!="."+QString(ext))
-//		oname = fname.left(fname.length()-3)+ext;
-//	else
-		oname = fname+"."+QString(ext);
+	if(fname.right(4)!="."+QString(ext))	oname = fname+"."+QString(ext);
 	return oname;
 }
 //-----------------------------------------------------------------------------
@@ -409,20 +406,11 @@ void QMathGL::setMGLFont(QString path)
 void QMathGL::setSize(int w, int h)
 {	graph->SetSize(w,h);	resize(w, h);	update();	};
 //-----------------------------------------------------------------------------
-void QMathGL::adjust()
-{
-	graph->Adjust();
-}
+void QMathGL::adjust()		{	graph->Adjust();	}
 //-----------------------------------------------------------------------------
-void QMathGL::nextSlide()
-{
-	graph->NextFrame();
-}
+void QMathGL::nextSlide()	{	graph->NextFrame();	}
 //-----------------------------------------------------------------------------
-void QMathGL::prevSlide()
-{
-	graph->PrevFrame();
-}
+void QMathGL::prevSlide()	{	graph->PrevFrame();	}
 //-----------------------------------------------------------------------------
 void QMathGL::animation(bool st)
 {
@@ -603,7 +591,7 @@ void mglGraphQT::Window(int argc, char **argv, int (*draw)(mglGraph *gr, void *p
 	else	Wnd->show();
 }
 //-----------------------------------------------------------------------------
-HMGL mgl_create_graph_qt(HMDR dr, const char *title)
+HMGL mgl_create_graph_qt_dr(HMDR dr, const char *title)
 {
 	mglGraphQT *g = new mglGraphQT;
 	g->Window(0,0,dr,title);
@@ -637,12 +625,9 @@ void *mgl_qt_tmp(void *)	{	mglQtRun();	return 0;	}
 	pthread_create(&tmp, 0, mgl_qt_tmp, 0);
 	pthread_detach(tmp);
 }*/
-void mgl_qt_run()	{	mglQtRun();	}
 //-----------------------------------------------------------------------------
 void mgl_qt_run_()	{	mglQtRun();	}
 //-----------------------------------------------------------------------------
-//void mgl_qt_thread_()	{	mgl_qt_thread();	}
-//-----------------------------------------------------------------------------
 #define TR	QObject::tr
 void mglGraphQT::makeMenu()
 {
